<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
* @category Nostress
* @package Nostress_Nscexport
*
*/

class Nostress_Nscexport_Block_Adminhtml_Nscexport extends Mage_Adminhtml_Block_Widget_Grid_Container
{
	const CON_INDEX_TRIAL = "trial";
	const CON_INDEX_LIVE = "live";
	const CON_INDEX_CUSTOM = "custom";
	const UNSORTED_INDEX = 100;
	
	protected $_feedsLoaded;
	
  public function __construct()
  {
    $this->_controller = 'adminhtml_nscexport';
    $this->_blockGroup = 'nscexport';
    $this->_headerText = Mage::helper('nscexport')->__('Koongo Connector').' - '.Mage::helper('nscexport')->__('profiles');
    $this->_addButtonLabel = Mage::helper('nscexport')->__('Add New Export Profile');
    
    
    
    if(Mage::helper('nscexport')->isDebugMode())
    {
    	$this->addReloadTaxonomyButton();
    	$this->addReloadFeedConfigButton();
    }
    else
    {
    	$this->addReloadFeedAndTaxonomiesButton();
    }
    
    
    //this event may add some buttons
    $this->dispatchProfileGridEvent();
    $this->addLiveChatButton();
    parent::__construct();
	
	$this->disableButtons();
	$this->initActivationData();
  }
  
  protected function addReloadTaxonomyButton()
  {
  	$this->_addButton('reloadenginecategories', array(
       'label'     => Mage::helper('nscexport')->__('Update Feed Taxonomies'),
       'onclick'   => "setLocation('".$this->getUrl("*/*/reloadtaxonomy")."');",
       'class'     => 'reload',
      ), -100);
  }    
  
  protected function addReloadFeedConfigButton()
  {
  	$label = 'Update Feed List';
  	$feedsLoaded = Mage::getModel( 'nscexport/feed' )->feedsLoaded();
  	
  	if(!$feedsLoaded)
  		$label = 'Get Feed List';
  	
  	$this->_addButton('reloadfeedconfig', array(
       'label'     => Mage::helper('nscexport')->__($label),
       'onclick'   => "setLocation('".$this->getUrl("*/*/reloadfeedconfig")."');",
       'class'     => 'reload',
      ), -100);
  }
  
  protected function addCheckLicenseStatusButton()
  {
  	$label = $this->__("Check License Status");  	
   	 
  	$this->_addButton('checklicensestatus', array(
  			'label'     => Mage::helper('nscexport')->__($label),
  			'onclick'   => "setLocation('".$this->getUrl("adminhtml/nscexport_action/checklicensegrid")."');",
  			'class'     => 'reload',
  	), -100);
  }
  
  protected function addLiveChatButton() {
            
      $this->_addButton('livechat', $this->helper('nscexport')->getLivechatButtonOptions(), -200);
  }
  
  protected function addReloadFeedAndTaxonomiesButton()
  {
  	$label = 'Update Feeds and Taxonomies';
  	$feedsLoaded = $this->getFeedsLoaded();
  	 
  	if(!$feedsLoaded)
  		$label = 'Get Feeds and Taxonomies';
  	 
  	$this->_addButton('reloadfeedtaxonomies', array(
  			'label'     => Mage::helper('nscexport')->__($label),
  			'onclick'   => "setLocation('".$this->getUrl("*/*/reloadfeedtaxonomies")."');",
  			'class'     => 'reload',
  	), -100);
  }
  
  protected function disableButtons()
  {
  	$valid = Mage::helper('nscexport/version')->isLicenseValid();
  	if(!$valid)
  	{
  		$this->disableButton('add');
  		$this->disableButton('reloadfeedconfig');
  		$this->disableButton('reloadenginecategories');
  		$this->disableButton('reloadfeedtaxonomies');
  	}
  	else if(!Mage::helper('nscexport/version')->isLicenseValid(true))
  	{
  		$this->disableButton('reloadfeedconfig');
  		$this->disableButton('reloadenginecategories');
  		$this->disableButton('reloadfeedtaxonomies');
  		$this->addCheckLicenseStatusButton();
  		
  	}
  	
  	$feedsLoaded = $this->getFeedsLoaded();
  	if(!$feedsLoaded)
  		$this->disableButton('add');
  }
  
  protected function disableButton($name)
  {
  	$this->updateButton($name,'disabled',"disabled");
  }
  
  protected function dispatchProfileGridEvent()
  {
  	Mage::dispatchEvent('nscexport_define_profile_grid', array('grid_container'=> $this));
  }
  
  protected function getButtonHtml($url,$class,$title,$id,$label)
  {
  	$html = "<button style=\"\" onclick=\"setLocation('{$url}')\" class=\"{$class}\" type=\"button\" title=\"{$title}\" id=\"{$id}\">
	  			{$label}
  			</button>";
  	return $html;
  }
  
  protected function getLicenseConfigUrl()
  {
  	return Mage::helper("adminhtml")->getUrl('adminhtml/system_config/edit/section/koongo_license_and_plugins');
  }
  
  protected function getActivateFormUrl($code)
  {
	return Mage::helper("adminhtml")->getUrl('adminhtml/nscexport_activation/',array("code" => $code));
  }
  
  protected function initActivationData()
  {
  		if(!Mage::helper('nscexport/version')->isLicenseEmpty())
  			return;
  		
  		$trialInfo = new Varien_Object();
  		$liveInfo = new Varien_Object();
  		$customConnectors = array();
  		
  		$info = Mage::helper('nscexport/data_client')->getConnectorsInfo();
  		if(!empty($info))
  		{
  			if(!empty($info[self::CON_INDEX_TRIAL]))
  				$trialInfo->addData($info[self::CON_INDEX_TRIAL]);
  			if(!empty($info[self::CON_INDEX_LIVE]))
  				$liveInfo->addData($info[self::CON_INDEX_LIVE]);
  			
  			if(!empty($info[self::CON_INDEX_CUSTOM]) && is_array($info[self::CON_INDEX_CUSTOM]))
  			{
  				$index = (int)self::UNSORTED_INDEX;
  				foreach ($info[self::CON_INDEX_CUSTOM] as $item)
  				{
  					$itemObject = new Varien_Object($item);
  					if(!empty($item['priority']))
  						$customConnectors[(int)$item['priority']] = $itemObject;
  					else
  					{
  						$customConnectors[$index] = $itemObject;
  						$index++;
  					}
  				}
  				ksort($customConnectors);
  			}
  		}
  		
  		$this->setTrialConnectorInfo($trialInfo);
  		$this->setLiveConnectorInfo($liveInfo);
  		$this->setCustomConnectors($customConnectors);
  }
  
  protected function getFeedsLoaded()
  {
  	if(!isset($this->_feedsLoaded))
  		$this->_feedsLoaded = Mage::getModel( 'nscexport/feed' )->feedsLoaded();
  	return $this->_feedsLoaded;
  }
}