<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
* @category Nostress
* @package Nostress_Nscexport
*/

class Nostress_Nscexport_Block_Adminhtml_Nscexport_Grid extends Mage_Adminhtml_Block_Widget_Grid
{
	public function __construct()
	{
	    parent::__construct();
	    $this->setId('nscexportGrid');
	    $this->setDefaultSort('export_id');
	    //$this->setDefaultDir('ASC');
	    $this->setSaveParametersInSession(true);
	}
	
	protected function _prepareCollection()
	{
	    $collection = Mage::getModel('nscexport/profile')->getCollection();
	    $collection->joinFeed();
	    $this->setCollection($collection);
	    return parent::_prepareCollection();
	}

    protected function _prepareColumns()
    {
		     	
      	$this->addColumn('export_id', array(
    	  'header'    => Mage::helper('nscexport')->__('ID'),
      	  'align'     =>'right',
      	  'width'     => '50px',
      	  'index'     => 'export_id',
      	));
           
     	$this->addColumn('name', array(
        	'header'    => Mage::helper('nscexport')->__('Profile Name'),
        	'index'     => 'name',
     		'filter_index' =>    'main_table.name',
     		//'renderer'  => 'nscexport/adminhtml_nscexport_grid_renderer_channel',
      	));
     	
		$feeds = $this->helper('nscexport/data_feed')->getOptions("link",true);
      	$this->addColumn('feed_link', array(
        	'header'    => Mage::helper('nscexport')->__('Feed'),
        	'type'      => 'options',
        	'align'     => 'center',
        	'index'     => 'link',
//       		'width'     => '90px',
        	'options'   => $feeds,
      		'renderer'  => 'nscexport/adminhtml_nscexport_grid_renderer_channel',
      	));
    
		$types = $this->helper('nscexport/data_feed')->getOptions("type",true);
   		$this->addColumn('feed_type', array(
   			'header'    => Mage::helper('nscexport')->__('Feed Type'),
   			'type'      => 'options',
   			'align'     => 'left',
   			'index'     => 'type',
   			'filter_index' =>    'feed_table.type',
   			'options'   => $types,
    	));
	    
	    $fileTypes = $this->helper('nscexport/data_feed')->getOptions("file_type",true);
	    $this->addColumn('feed_file', array(
	    		'header'    => Mage::helper('nscexport')->__('Format'),
	    		'type'      => 'options',
	    		'align'     => 'left',
	    		'index'     => 'file_type',
	    		'width'     => '40px',
	    		'options'   => $fileTypes,
	    ));
	    
	    $this->addColumn('category_mapping', array(
	            'header'    => Mage::helper('nscexport')->__('Category Mapping'),
	            'index'     => 'category_mapping',
	    		'align'		=>'center',
	    		'width'     => '95px',
	            'renderer'  => 'nscexport/adminhtml_nscexport_grid_renderer_categorymapping',
	    		'filter'    => false,
	    		'sortable'  => false,
	    ));

    	//get store view name option collection
    	$storeOptionsArray = array();
    	$storeCollection = Mage::getSingleton('adminhtml/system_store')->getStoreCollection();
    	foreach ($storeCollection as $store)
    	{
    		$storeOptionsArray[$store->getId()] = Mage::helper('nscexport')->getFullStoreName($store);
    	}
    	$this->addColumn('store_title', array(
            'header'        => Mage::helper('core')->__('Store Name'),
            'type'     		=> 'options',
            'align'         => 'left',
            'index'         => 'store_id',
            'options'   	=> $storeOptionsArray,
    	));
    	
    	$this->addColumn('link', array(
    			'header'    => Mage::helper('nscexport')->__('Feed Preview'),
    			'index'     => 'url',
    			'align'		=>'center',
    			'width'     => '95px',
    			'renderer'  => 'nscexport/adminhtml_nscexport_grid_renderer_link',
    			'filter'    => false,
    			'sortable'  => false,
    	));
    
    	$this->addColumn('status', array(
			'header'    => Mage::helper('nscexport')->__('Status'),
			'align'     => 'left',
			'type'      => 'longtext',
			'index'     => 'message',
    		'width'     => '300px',
    		'filter'    => false,
    		'renderer'  => 'nscexport/adminhtml_nscexport_grid_renderer_status'
		));
	
		$this->addColumn('update_time', array(
			'header'    => Mage::helper('nscexport')->__('Last Run Time'),
			'align'     => 'left',
			'width'     => '120px',
			'type'      => 'datetime',
			'default'   => '--',
			'index'     => 'update_time',
			'renderer' => 'nscexport/adminhtml_nscexport_grid_renderer_datetime'
		));
	

		$this->addColumn('button', array(
            'header'   => Mage::helper('nscexport')->__('Action'),
            'filter'   => false,
            'sortable' => false,
			'align'		=>'center',
            'width'    => '100',
            'renderer' => 'nscexport/adminhtml_nscexport_grid_renderer_action'
        ));
		
		$this->addColumn('upload', array(
	        'header'   => Mage::helper('nscexport')->__('Submission'),
	        'filter'   => false,
	        'sortable' => false,
			'align'		=>'center',
	        'width'    => '100',
	        'renderer' => 'nscexport/adminhtml_nscexport_grid_renderer_submission'
		));
    	
		return parent::_prepareColumns();
  }
  
	/**
     * Row click url
     *
     * @return string
     */
    public function getRowUrl($row)
    {
        return $this->getUrl('*/*/edit', array('id' => $row->getId()));
    }
    
    protected function _prepareMassaction()
    {
        $this->setMassactionIdField('entity_id');
        $this->getMassactionBlock()->setFormFieldName('profile');

        $this->getMassactionBlock()->addItem('delete', array(
             'label'=> Mage::helper('catalog')->__('Delete'),
             'url'  => $this->getUrl('*/*/massDelete'),
             'confirm' => Mage::helper('catalog')->__('Are you sure?')
        ));
        
        $this->getMassactionBlock()->addItem('generate', array(
             'label'=> Mage::helper('nscexport')->__('Execute'),
             'url'  => $this->getUrl('*/*/massGenerate')
        ));

        return $this;
    }
}
