<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Block_Adminhtml_Schedule_Grid extends Mage_Adminhtml_Block_Widget_Grid
{
	/**
	 *
	 */
	public function __construct()
	{
		parent::__construct();
		$this->setId('cron_schedule_grid');
		$this->setUseAjax(true);
		$this->setDefaultSort('scheduled_at');
		$this->setDefaultDir('ASC');
		$this->setSaveParametersInSession(true);
	}

	/**
	 *
	 */
	protected function _prepareCollection()
	{
		/**
		 * @var Mage_Cron_Model_Mysql4_Schedule_Collection $collection
		 */
		$collection = Mage::getModel('cron/schedule')->getCollection();

		// Filter Finished Schedules
		//$collection->addFieldToFilter('finished_at', '0000-00-00 00:00:0');                
			
        $collection->addFieldToFilter('status', array('pending', 'running', 'missed'));
		$collection->addFieldToFilter('job_code', Nostress_Nscexport_Model_Schedule_Code::CRONTAB_JOB_CODE);
        
		// Order Collection
		$collection->addOrder($this->getOrderSort(), $this->getOrderDir());

		$this->setCollection($collection);
		return parent::_prepareCollection();
	}

	/**
	 *
	 */
	protected function _prepareColumns()
	{
		$this->addColumn('schedule_id', array(
			'header' => $this->moduleHelper()->__('Id'),
			'width' => '5px',
			'type' => 'text',
			'index' => 'schedule_id',
			'filter' => false,
		));

		$this->addColumn('job_code', array(
			'header' => $this->moduleHelper()->__('Code'),
			'width' => '200px',
			'type' => 'options',
			'index' => 'job_code',
			'options' => Mage::getSingleton('nscexport/schedule_code')->getProcessesCodes(),
		));

		$this->addColumn('status', array(
			'header' => $this->moduleHelper()->__('Status'),
			'width' => '5px',
			'type' => 'options',
			'index' => 'status',
			'options' => Mage::getSingleton('nscexport/schedule_status')->getStatesSchedule(),
		));

		$this->addColumn('starting_in', array(
			'header' => $this->moduleHelper()->__('Starting In'),
			'index' => 'starting_in',
			'sortable' => false,
			'filter' => false,
			'type' => 'text',
			'width' => '80px',
			'renderer' => 'nscexport/adminhtml_widget_grid_column_renderer_starting',
		));

		$this->addColumn('scheduled_at', array(
			'header' => $this->moduleHelper()->__('Scheduled'),
			'index' => 'scheduled_at',
			'type' => 'datetime',
			'width' => '80px',
			'renderer' => 'nscexport/adminhtml_widget_grid_column_renderer_datetime',
		));

		$this->addColumn('executed_at', array(
			'header' => $this->moduleHelper()->__('Executed'),
			'index' => 'executed_at',
			'type' => 'datetime',
			'width' => '80px',
			'renderer' => 'nscexport/adminhtml_widget_grid_column_renderer_datetime',
		));

		$this->addColumn('finished_at', array(
			'header' => $this->moduleHelper()->__('Finished'),
			'index' => 'finished_at',
			'type' => 'datetime',
			'width' => '80px',
			'renderer' => 'nscexport/adminhtml_widget_grid_column_renderer_datetime',
		));

        $this->addColumn('runtime', array(
            'header' => $this->moduleHelper()->__('Runtime'),
            'index' => 'runtime',
            'sortable' => false,
            'filter' => false,
            'type' => 'text',
            'width' => '60px',
            'renderer' => 'nscexport/adminhtml_widget_grid_column_renderer_runtime',
        ));

		$this->addColumn('created_at', array(
			'header' => $this->moduleHelper()->__('Created'),
			'index' => 'created_at',
			'type' => 'datetime',
			'width' => '80px',
			'renderer' => 'nscexport/adminhtml_widget_grid_column_renderer_datetime',
		));

		$this->addColumn('messages', array(
			'header' => $this->moduleHelper()->__('Messages'),
			'width' => '50px',
			'type' => 'text',
			'index' => 'messages',
		));

		if(Mage::getSingleton('admin/session')->isAllowed('config/nscexport/actions/edit')) {
			$this->addColumn('action',
					array(
						'header' => $this->moduleHelper()->__('Action'),
						'width' => '50px',
						'type' => 'action',
						'getter' => 'getId',
						'actions' => array(
							array(
								'caption' => $this->moduleHelper()->__('Edit'),
								'url' => array('base' => '*/*/edit'),
								'field' => 'schedule_id'
							)
						),
						'filter' => false,
						'sortable' => false,
						'index' => 'stores',
						'is_system' => true,
			));
		}

		return parent::_prepareColumns();
	}

	protected function _prepareMassaction()
	{
		$this->setMassactionIdField('id');
		$this->getMassactionBlock()->setFormFieldName('cron_ids');
		$this->getMassactionBlock()->setUseSelectAll(false);

		$this->getMassactionBlock()->addItem('delete_cron', array(
			'label' => Mage::helper('sales')->__('Delete'),
			'url' => $this->getUrl('*/*/massDelete'),
		));


		return $this;
	}

	/**
	 *
	 */
	protected function getOrderSort()
	{
		return $this->getRequest()->getParam($this->getVarNameSort(), $this->_defaultSort);
	}

	/**
	 *
	 */
	protected function getOrderDir()
	{
		return $this->getRequest()->getParam($this->getVarNameDir(), $this->_defaultDir);
	}

	/**
	 *
	 */
	public function getRowUrl($row)
	{
		if(Mage::getSingleton('admin/session')->isAllowed('config/nscexport/actions/edit')) {
			return $this->getUrl('*/*/edit', array('schedule_id' => $row->getId()));
		}
		return false;
	}

	/**
	 *
	 */
	public function getGridUrl()
	{
		return $this->getUrl('*/*/grid', array('_current' => true));
	}

	/**
	 *
	 * @return nscexport_Helper_Data
	 */
	protected function moduleHelper()
	{
		return Mage::helper('nscexport');
	}

}
