<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */
/**
 *
 * @category   Nostress
 * @package    Nostress_Nscexport
 * @author     NoStress Commerce Team <info@nostresscommerce.cz>
 */

class Nostress_Nscexport_Helper_Version extends Mage_Core_Helper_Abstract
{
	const HTTPS = "https://";
	const HTTP = "http://";
	const URL_DELIMITER = "/";
	const PORT_DELIMITER = ":";
	const VRE = "/[0-9]+.[0-9]+/";
	const NEW_LICENSE_URL = "https://store.koongo.com/koongo-connector-for-magento.html";
	const RENEW_LICENSE_URL = "https://store.koongo.com/checkout/cart/add";	
	const RENEW_LICENSE_SUFFIX = "?return_url=https://store.koongo.com/onestepcheckout&product=428&qty=1&options[486]={{license_key}}&options[483]=1203&nocookie=1";
	const LATEST_VERSION = 'latest_version';
	const LICENSE_VALIDITY = 'license_validity';
	const VALIDITY = 'validity';
	protected $_moduleLabel = 'Nscexport';
	protected $_ts = "l41rt";
	protected $_validForVersion = '1.4+';
	protected $_s = 'mage547nostress123';
	const TD = "28-02-2013";
	const STEP = 2;
	const ENDL = 7;
	const DL = 6;
	const SHF = 5;
	const DDLM = '-';
	const ASX = 2;
	const AEH = 4;
	const YR = 20;
	const TP = "tr14l";
	protected $_setupName = "nscexport_setup";
	const MODULE_UPDATE_LINK = "module_update_link";
	const PLUGIN_UPDATE_LINK = "plugin_update_link";
	const PLUGIN_LIST_LINK = "plugin_list_link";
	const LICENSE_NOT_VALID = "License invalid";

	public function getModuleStatusHtml() {
		if ($this->isVersionValid()) {
			return "<span style='color:green;font-weight:bold'>".$this->__('OK')."</span>";
		}
		else {
			return "<span style='color:red;font-weight:bold'>".$this->__('Not Valid')." - <a href=\"".$this->__(self::LICENSE_URL)."\">".$this->__('Buy New License')."</a></span>";
		}
	}

	public function getLicenseKeyStatusHtml($addButtons = true) {
		if (!$this->getLicenseKey()) {
			return $this->getLicenseHtml('Buy New License',$this->__('Please enter the License key'));
		}

		$message = "";
		if (!$this->isLicenseKeyValid())
		{
		    $message .= $this->__('Not Valid');
		    if($this->isLicenseKeyT())
		        $message .= " - ".$this->__('Trial period expired on %s.',$this->gLD())." ";

			$message = $this->getLicenseHtml('Buy New License',$message,self::NEW_LICENSE_URL);
		}
		else if (!$this->isDateValid())
		{
			$message .= $this->__('Module Support & Updates period expired on %s ',$this->gLD());
			$renwLicenseUrl = $this->getRenewLicenseUrl($this->getLicenseKey());
			$message = $this->getLicenseHtml('Renew Support & Updates',$message,$renwLicenseUrl);
			
			$label = $this->__("Check License Status");
			$link = Mage::getModel('adminhtml/url')->getUrl("adminhtml/nscexport_action/checklicense");
			if($addButtons)
				$message .= '<br><button style="" onclick="setLocation(\''.$link.'\');" class="scalable reload" type="button" title="'.$label.'" id="btn-checklicense"><span><span><span>'.$label.'</span></span></span></button>';
		}

		if(empty($message))
		{
			$message = $this->__('OK')." - ".$this->__('Module Support & Updates period expires on %s ',$this->gLD());;
			$message = "<span style='color:green;font-weight:bold'>".$message."</span>";
		}

		return $message;
	}

	protected function getLicenseHtml($label,$message,$licenseUrl = self::NEW_LICENSE_URL,$params = "")
	{
		$message .= " - <a target=\"_blank\" href=\"".$this->__($licenseUrl).$params."\">".$this->__($label)."</a>";
		$message = "<span style='color:red;font-weight:bold'>".$message."</span>";
		return $message;
	}

	public function isModuleValid()
	{
		return $this->isVersionValid() && $this->isLicenseValid();
	}

	public function isVersionValid() {
		$curVersion = substr($this->getMagentoVersion(),2,1);
		$match = array();
		if(preg_match(self::VRE ,Mage::getVersion(),$match))
			$match = explode(".",$match[0]);
		if(isset($match[0]) && $match[0] < 1)
			return false;
		if(isset($match[1]))
			$curVersion = $match[1];

		switch($this->_validForVersion)
		{
			case '3':
				return $curVersion == $this->_validForVersion;
				break;
			default:
				if ((float)$curVersion >= (float)'4')
					return true;
				break;
		}
		return false;
	}

	public function getMagentoVersion()
	{
		return Mage::getVersion();
	}

	public function processModuleInfo($info)
	{
		if(isset($info[self::LATEST_VERSION]))
		{
			Mage::helper('nscexport')->setGeneralConfig(self::LATEST_VERSION, trim($info[self::LATEST_VERSION]));
		}
	}

	public function processLicenseData($lincenseData)
	{
		if(isset($lincenseData[self::VALIDITY]) && $lincenseData[self::VALIDITY] != self::LICENSE_NOT_VALID)
		{
			Mage::helper('nscexport')->setGeneralConfig(self::LICENSE_VALIDITY, trim($lincenseData[self::VALIDITY]));
		}
	}

	public function isLicenseEmpty()
	{
		$licenseKey = $this->getLicenseKey();
		if(empty($licenseKey))
			return true;
		else
			return false;
	}
	
	public function isLicenseValid($chackDate = false) {
		if($chackDate && $this->isLicenseKeyValid() && $this->isDateValid())
		{
			return true;
		}
		else if(!$chackDate && $this->isLicenseKeyValid())
			return true;

		return false;
	}

	protected function isLicenseKeyValid() {
		$storeKey = $this->getLicenseKey();
		$genKey = $this->generateLicenseKey();

		if($a = $this->cmpKeyz($genKey, $storeKey) && ($this->isDateValid() || !$this->isDateT()))
			return $a;

		$genKey = $this->generateLicenseKey(true);
		if(($bb = $this->cmpKeyz($genKey, $storeKey)) && $this->isDateValid())
			return $bb;

		return false;
	}

	protected function cmpKeys($key1, $key2) {
		if (!isset($key1) || !isset($key2) || empty($key1))
			return false;

		$length = strlen($key1);
		if ($length != strlen($key2)) {
			return false;
		}

		for ($i = 1;$i < $length;$i += self::STEP) {
			if ($key1[$i] != $key2[$i])
				return false;
		}

		return true;
	}

	protected function getEvenChars($input) {
		$result = "";
		$addCh = true;
		foreach ($input as $char) {
			if ($addCh)
				$result .= $char;

			$addCh = !$addCh;
		}
		return $result;
	}

	public function isDateT()
	{
		$date = $this->gLD();
		$ld = Mage::helper('nscexport')->getDateTime($date);
		$tDate = Mage::helper('nscexport')->getDateTime(self::TD);
		if ($tDate >= $ld)
			return true;
		else
			return false;
	}

	public function isDateValid() {
		$date = $this->gLD();
		$ld = Mage::helper('nscexport')->getDateTime($date);
		$now = Mage::helper('nscexport')->getDateTime('today');
		if ($ld >= $now)
			return true;
		else
			return false;
	}

	public function isLicenseKeyT()
	{
		$storeKey = $this->getLicenseKey();
		$genKey = $this->generateLicenseKey(true);
		if(($bb = $this->cmpKeyz($genKey, $storeKey)))
			return $bb;

		return false;
	}

	public function gLD()
	{
		$validity = Mage::helper('nscexport')->getGeneralConfig(self::LICENSE_VALIDITY);
		if(empty($validity) || $this->isLicenseKeyT())
			$validity = $this->dtProcess($this->getLicenseKey());
		
		//Format price e.g. 17 Aug 2020
		$validity = date("d M Y",strtotime($validity));		
		
		return $validity;
	}

	public function getLicenseKey()
	{
		$licenseKey =  Mage::getStoreConfig('koongo_license_and_plugins/license/license_key_'.$this->_getModuleName());
		if(empty($licenseKey))
			$licenseKey = Mage::getStoreConfig('nostress_modules/nostress_dashboard/license_key_'.$this->_getModuleName());
		return $licenseKey;
	}
	
	public function saveLicenseKey($key)
	{
		$path = 'koongo_license_and_plugins/license/license_key_'.$this->_getModuleName();
		Mage::helper('nscexport')->setGeneralConfig(self::LICENSE_VALIDITY, "");
		$config = Mage::getConfig();
		$config->saveConfig($path, $key);
		$config->cleanCache();
        $config->reinit();
	}
	
	public function resetLicenseDate()
	{
		$validity = Mage::helper('nscexport')->getGeneralConfig(self::LICENSE_VALIDITY);
		if(!isset($validity))
			return;
		Mage::helper('nscexport')->setGeneralConfig(self::LICENSE_VALIDITY, "");
		$config = Mage::getConfig();
		$config->cleanCache();
		$config->reinit();
	}

	public function generateLicenseKey($t = false) {
		$serverName = $this->getServerName();
		$p = self::TP;$s = $this->_ts; if(!$t){$p=$s="";}
		return md5(sha1($p.$serverName.$this->_getModuleName().$this->_s.$s));
	}

	protected function dtProcess($input) {
		$date = "";
		$addCh = true;
		$inputLength = strlen($input);
		for ($i = 1; $i <= $inputLength; $i += 2) {
			$date .= $input[$i];
			if (strlen($date) >= self::ENDL)
				break;
		}
		$date = $date >> self::SHF;
		$date = $this->addZeros($date, self::DL);

		$date = $this->frD($date);

		return $date;
	}

	protected function frD($date) {
		$date = (string)$date;
		$rd = "";
		for ($i = 0; $i < self::DL; $i++) {
			$rd .= $date[$i];
			if (!$i)
				continue;
			if (!(($i+1)%self::ASX) && ($i+1)!=self::DL)
				$rd .= self::DDLM;
			if (!(($i+1)%self::AEH))
				$rd .= self::YR;
		}
		return $rd;
	}

	protected function addZeros($date, $length) {
		while (strlen($date) != $length)
			$date = "0".$date;
		return $date;
	}

	public function getDashboardFooterHtml() {
		return "
		</table>".Mage::helper('adminhtml/js')->getScript("
            $('koongo_license_and_plugins_license-head').setStyle('background: none;');
            $('koongo_license_and_plugins_license-state').value = 1;
            $('koongo_license_and_plugins_license-head').writeAttribute('onclick', 'return false;');
            $('koongo_license_and_plugins_license').show();
		");
	}

	public function getServerId()
	{
		$name = $this->getServerName();
		$id = trim(base64_encode($name),"=");
		return $id;
	}

	public function addLiveChatButton() {
	    return '
            <a title="'.Mage::helper('nscexport')->__('Start Live Chat').'" style="display: block;  float: right;" href="javascript:void(0);" onclick="'.Mage::helper('nscexport')->getLivechatLink().'">
                <img width="100" src="'.Mage::helper('nscexport')->getLivechatImage().'" align="absmiddle" border="0" />
            </a>';
	}
	
	public function getDashboardHeaderHtml() {
		return
		$this->addLiveChatButton().
		"
		<table cellspacing=\"15\">
			<tr>
				<th>{$this->__('Module Name')}</th>
				<th>{$this->__('Module Version')}</th>
				<th>{$this->__('Your License for Magento')}</th>
				<th>{$this->__('Module Status')}</th>
				<th>{$this->__('Enter License Key')}</th>
				<th>{$this->__('License Status')}</th>
			</tr>";
	}

	public function getDashboardEntryHtml()
	{
		$currentVersion = $this->getModuleVersion();
		$latestVersion = $this->getLatestModuleVersion();

		if($this->cmpVersions($currentVersion,$latestVersion))
		{
			$updateUrl = $this->getModuleUpdateLink();
			$message = $this->__('Get latest version %s',$latestVersion);
			$currentVersion = "<span style='color:red;font-weight:bold'>{$currentVersion} - <a title=\"{$message}\" style='color:red' href=\"{$updateUrl}\">{$message}</a></span>";
		}

		$url = Mage::getModel('adminhtml/url');
		return "
		<tr>
			<td><a href=\"".$url->getUrl('*/*/*',array('section'=>'koongo_config'))."#{$this->_getModuleName()}-head\">{$this->getModuleLabel()}</a></td>
			<td>{$currentVersion}</td>
			<td>{$this->_validForVersion}</td>
			<td>{$this->getModuleStatusHtml()}</td>
			<td><input id=\"{$this->_getModuleName()}_licensekey\" class=\"input-text\" type=\"text\" value=\"".$this->getLicenseKey()."\" name=\"groups[license][fields][license_key_{$this->_getModuleName()}][value]\" /></td>
			<td>{$this->getLicenseKeyStatusHtml()}</td>
		</tr>";
	}

	public function isLatestVersionInstalled()
	{
		$currentVersion = $this->getModuleVersion();
		$latestVersion = $this->getLatestModuleVersion();
		return !$this->cmpVersions($currentVersion,$latestVersion);
	}

	public function getModuleUpdateLink()
	{
		return Mage::helper('nscexport')->getHelpUrl(self::MODULE_UPDATE_LINK);
	}

	protected function cmpVersions($currentVersion,$latestVersion)
	{
		return Mage::helper('nscexport')->cmpVersions($currentVersion,$latestVersion);
	}

	public function getModuleVersion() {
		return Mage::helper('nscexport')->getModuleVersion($this->_setupName);
	}

	public function getLatestModuleVersion()
	{
		$latestVersion = Mage::helper('nscexport')->getGeneralConfig(self::LATEST_VERSION);
		if(empty($latestVersion))
			$latestVersion = $this->getModuleVersion();
		return $latestVersion;
	}

	public function getModuleLabel() {
		return $this->__("Koongo Connector");
	}

	public function validateLicenceBackend()
	{
		if (!self::isLicenseValid()) {
			Mage::helper("nscexport")->log("Module is not valid. Domain name:". $this->getServerName());
			Mage::getSingleton('adminhtml/session')->addError($this->__('Your License is not valid. ').$this->__('Module %s is deactivated!',$this->getModuleLabel()));
			header("Location: ".Mage::getModel('adminhtml/url')->addSessionParam()->getUrl('adminhtml/system_config/edit',array('section'=>'koongo_license_and_plugins')));
			exit;
		}
		return true;
	}

	public function checkLicense()
	{
		if($this->isLicenseEmpty())
			return;
		if(!self::isLicenseValid()) {
			$url = Mage::getModel('adminhtml/url')->addSessionParam()->getUrl('adminhtml/system_config/edit',array('section'=>'koongo_license_and_plugins'));
			$url = "<a href=\"".$url." \">{$this->__("More details.")}</a>";
			Mage::helper("nscexport")->log("Module is not valid. Domain name:". $this->getServerName());

            $message = $this->__('Your License is not valid. ');
			if($this->isLicenseKeyT())
			    $message = $this->__('Your FREE Trial period expired on %s. ',$this->gLD());
			Mage::getSingleton('adminhtml/session')->addError($message.$this->__('Module %s is deactivated!',$this->getModuleLabel())." ".$url);
		}
	}

	protected function cmpKeyz($key1, $key2)
	{
		if (!isset($key1) || !isset($key2) || empty($key1))
			return false;

		$length = strlen($key1);
		if ($length != strlen($key2)) {
			return false;
		}

		for ($i = 0;$i < $length;$i += self::STEP) {
			if ($key1[$i] != $key2[$i])
				return false;
		}

		return true;
	}

	public function getServerName()
	{
		$url = 	Mage::app()->getStore(Mage_Catalog_Model_Abstract::DEFAULT_STORE_ID)->getBaseUrl(Mage_Core_Model_Store::URL_TYPE_LINK, true);
		$serverName = str_replace(self::HTTPS,"",$url);
		$serverName = str_replace(self::HTTP,"",$serverName);
		$portDelimiterIndex = strpos($serverName,self::PORT_DELIMITER);
		$index = strpos($serverName,self::URL_DELIMITER);
		if($index !== false && $portDelimiterIndex != false)
		{
			if($portDelimiterIndex < $index)
				$index = $portDelimiterIndex;
		}
		else if($portDelimiterIndex != false)
		{
			$index = $portDelimiterIndex;
		}

		if($index !== false)
			$serverName = substr($serverName,0,$index);
		return $serverName;
	}
	
	public function getLicenseInvalidMessage()
	{
		$licenseKey = $this->getLicenseKey();
		$date = $this->gLD();
		$url = $this->getRenewLicenseUrl($licenseKey);
		$dateString = "<span style='color:red;font-weight:bold'>".$date."</span>";
		$message = $this->__("Your Support & Updates period has expired on %s. You can renew Support & Updates by following ",$dateString);
		$message .= "<a target=\"_blank\" href=\"".$url."\">".$this->__("this link").".</a>";
		return $message;
	}
	
	protected function getRenewLicenseUrl($licenseKey)
	{
		$url = self::RENEW_LICENSE_URL.str_replace("{{license_key}}", $licenseKey, self::RENEW_LICENSE_SUFFIX);
		return $url;
	}
	
	public function getModuleName()
	{
		return $this->_getModuleName();
	}
}
