<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/** 
 * Model for Export
 * 
 * @category Nostress 
 * @package Nostress_Nscexport
 * 
 */

class Nostress_Nscexport_Model_Categoryproducts extends Mage_Core_Model_Abstract 
{
	const PRODUCTS_IDS = 'product_ids';
	const PRODUCTS_ID = 'product_id';
	const CATEGORY_ID = 'category_id';
	const INDEX_CATEGORY_PRODUCTS = 'category_products';
	const INDEX_CATEGORY_IDS = 'category_ids';
	const PROFILE_ID = "profile_id";
	protected $_store;
	
	public function _construct() 
	{
		parent::_construct ();
		$this->_init ( 'nscexport/categoryproducts' );
	}
	
	public function updateProductAssignment($product)
	{		
		$relationsArray = array();	
		$processedProfileIds = array();
		$removeProductFromAllProfiles = true;
		
		foreach($product->getStoreIds() as $productStoreViewId)
		{
			$this->_store = Mage::app()->getStore($productStoreViewId);
			$profileCollection =  Mage::getModel('nscexport/profile')->getCollectionByStoreId($productStoreViewId);
			$categoryIds = $product->getCategoryIds();

			foreach($profileCollection as $profile)
			{
				if(!$profile->autoAddNewProducts())
				{	
					$removeProductFromAllProfiles = false;
					continue;					
				}
				
				if(!in_array($profile->getId(),$processedProfileIds))
					$processedProfileIds[] =  $profile->getId();
				
				foreach($categoryIds as $categoryId)
				{
					if($this->isCategoryInProfile($categoryId,$profile->getId()))
					{
						$relationsArray[] = array(self::CATEGORY_ID => $categoryId,self::PROFILE_ID => $profile->getId());												
						
					}
				}
			}			
		}	

		if($removeProductFromAllProfiles)
			$this->removeProductFromAllProfiles($product->getId());
		else 
		{
			$productId = $product->getId();
			foreach ($processedProfileIds as $profileId)
				$this->removeProductFromProfile($productId,$profileId);
		}
		
		
		$productId = $product->getId();
		$relationsUpdated = false;
		foreach ($relationsArray as $relation) 
		{
			$this->addProductToProfile($productId,$relation[self::CATEGORY_ID],$relation[self::PROFILE_ID]);
			$relationsUpdated = true;
		}
			
		if($relationsUpdated)
			Mage::getSingleton('adminhtml/session')->addSuccess(
                	Mage::helper('nscexport')->__('The export profile-product relations has been updated.')
            	);
	} 
	
	public function updateCategoryProducts($profileId,$categoryproducts,$storeId)
	{
		$this->_store = Mage::app()->getStore($storeId);		
		$data = $this->_parseCategoryProducts($categoryproducts);
		$categoryIds = $data[self::INDEX_CATEGORY_IDS];
		$catProdParsed = $data[self::INDEX_CATEGORY_PRODUCTS];
		$this->_runQuery($profileId,$catProdParsed,true,$categoryIds);
	}
	
	public function duplicateCategoryProducts($oldProfileId, $newProfileId)
	{
		$this->getResource()->duplicateCategoryProducts($oldProfileId, $newProfileId);
	}
	
	public function getExportCategoryProducts($profileId)
	{
		/*GROUP_CONCAT can be limited to 1024 -> increase limit in this session*/
		Mage::getSingleton('core/resource')->getConnection('core_write')->query('SET SESSION group_concat_max_len = 1000000;');
		
		$collection = $this->_getCategoryProductsCollection($profileId);				
		$collection->getSelect()->columns(array(self::PRODUCTS_IDS => "(GROUP_CONCAT(".self::PRODUCTS_ID." SEPARATOR ','))"));
		$collection->getSelect()->group(self::CATEGORY_ID);			
		$data = $collection->getData();
		
		return $this->_prepareCategoryProductResult($data);				
	}
	
	public function getExportCategories($profileId)
	{
		$collection = $this->_getCategoryProductsCollection($profileId);			
		$select = $collection->getSelect();
		$select->distinct();					
		//$q = $collection->getSelect()->__toString();				
		$data = $collection->getData();		
		
		return $this->_prepareCategoriesArray($data);
	}
	
	public function isCategoryInProfile($categoryId,$profileId)
	{
		return $this->getResource()->isCategoryInProfile($categoryId,$profileId);
	}
	
	public function getCategoryProductsCount($profileId)
	{
		return $this->getResource()->getProfileRecordCount($profileId);
	}
	
	public function addProductToProfile($productId,$categoryId,$profileId)
	{
		$categoryproducts = array($categoryId => array($productId));
		$this->_runQuery($profileId,$categoryproducts,false);
	}
	
	public function removeProductFromAllProfiles($productId)
	{
		$this->getResource()->deleteRecords(null,$productId);
	}
	
	public function removeProductFromProfile($productId,$profileId)
	{
		$this->getResource()->deleteRecords($profileId,$productId);
	}
	
	private function _prepareCategoriesArray($data)
	{		
		$result = array();	
		foreach($data as $record)
		{
			$result[] = $record[self::CATEGORY_ID];
		}
		return $result;
	}
	
	private function _prepareCategoryProductResult($data)
	{
		$result = "";
		$first = true;
		foreach($data as $record)
		{
			if($first)
				$first = false;
			else
				$result .= "|";
			$result.= $record[self::CATEGORY_ID].",".$record[self::PRODUCTS_IDS];
		}
		unset($data);
		return $result;
	}
	
	protected function _getCategoryProductsCollection($profileId)
	{
		$collection = $this->getCollection()->addFieldToFilter('export_id',$profileId);
		$collection->addFieldToSelect(self::CATEGORY_ID);		
		return $collection;
	}
	
	private function _runQuery($profileId,$categoryproducts,$deleteRecords,$categoryIds=null)
    {
	    try 
	    {
	    	if($deleteRecords)
            	$this->getResource()->deleteRecords($profileId);
			$this->getResource()->insertRecords($profileId,$categoryproducts);
			$this->getResource()->selectInsertRecords($profileId,$categoryIds,$this->_store->getId());
				
		} 
		catch (Exception $e) 
		{
            Mage::getSingleton('adminhtml/session')->addError($e->getMessage()
                //Mage::helper('catalogrule')->__('Unable to apply rules.')
            );
            throw $e;
	    }
    }  
	
	private function _parseCategoryProducts($categoryproducts)
	{
	    $categoryIds = array();
		$result = Array();
		$groups = explode("|",$categoryproducts);
		foreach($groups as $group)
		{
			$members = explode(",",$group);
			if(count($members) == 0 || $members[0] == "" || !is_numeric($members[0]))
				continue;
			$categoryId = $members[0];	
			unset($members[0]);
			$productIds = Array();
			
			$deleted = false;
			foreach($members as $productId)
			{
				if($productId == 'deleted')
				{
					$deleted = true;
					break;
				}
				if(is_numeric($productId))
				{
					$productIds[] = $productId;
				}
			}
			if($deleted)
				continue;
			
			if(count($productIds) == 0)
			{
				$categoryIds[] = $categoryId;
			}
			else
				$result[$categoryId] = $productIds;
		}
		return array(self::INDEX_CATEGORY_PRODUCTS => $result,self::INDEX_CATEGORY_IDS => $categoryIds);
	}
}