<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@nostresscommerce.cz so we can send you a copy immediately.
 *
 * @copyright Copyright (c) 2009 NoStress Commerce (http://www.nostresscommerce.cz)
 *
 */

/**
 *
 * @category Nostress
 * @package Nostress_Nscexport
 *
 */

class Nostress_Nscexport_Model_Export_Api_Adapter_Amazon extends Nostress_Nscexport_Model_Export_Api_Adapter_Abstract
{
    const DEFAULT_FILE_TYPE = '_POST_FLAT_FILE_LISTINGS_DATA_';
    
    // for mapping adapter to feeds - only inventory loaders are supported
    const API_FILE_TYPE = '_POST_FLAT_FILE_INVLOADER_DATA_';
    
    protected $_code = 'amazon';
    
    protected $_label = 'Amazon API';
    
    protected $_requiredFields = array( 'merchant_id', 'marketplace', 'api_key', 'api_secret');
    
    protected $_marketplaces = array(
        'na_region' => array(
            'ATVPDKIKX0DER' => array(
                    'service'=>"https://mws.amazonservices.com",
                    'label' => "Amazon.com"
            ),
            'A2EUQ1WTGCTBG2' => array(
                    'service'=> "https://mws.amazonservices.ca", // Canada
                    'label' => "Amazon.ca"
            ),
        ),
        'eu_region' => array(
            'A1F83G8C2ARO7P' => array(
                    'service'=> "https://mws.amazonservices.co.uk", // United Kingdom
                    'label' => "Amazon.co.uk"
            ),
            'A1PA6795UKMFR9' => array(
                    'service'=> "https://mws.amazonservices.de", // Germany
                    'label' => "Amazon.de"
            ),
            'A1RKKUPIHCS9HS' => array(
                    'service'=> "https://mws.amazonservices.es", // Spain
                    'label' => "Amazon.es"
            ),
            'A13V1IB3VIYZZH' => array(
                    'service'=> "https://mws.amazonservices.fr", // France
                    'label' => "Amazon.fr"
            ),
            'APJ6JRA9NG5V4' => array(
                    'service'=> "https://mws.amazonservices.it", // Italy
                    'label' => "Amazon.it"
            ),
            'A21TJRUUN4KGV' => array(
                    'service'=> "https://mws.amazonservices.in", // India
                    'label' => "Amazon.in"
            ),
        ),
        'fe_region' => array(
            'A1VC38T7YXB528' => array(
                    'service'=> "https://mws.amazonservices.jp",  // Japan
                    'label' => "Amazon.co.jp",
                    'encoding' => 'SJIS'
            ),
        ),
//         'cn_region' => array(
//             'AAHKV2X7AFYLW' => array(
//                'service'=> "https://mws.amazonservices.com.cn", // China
//                'label' => "Amazon.com.cn"
//             ),
//         )
    );    
    
    public function getMarketplacesToOptionArray()
    {
        $optionArray = array();
        
        // show only marketplaces with account
        $marketplaceRegions = $this->_marketplaces;
        foreach( $marketplaceRegions as $regionCode => $marketplaces) {
            $code = $accountCode;            
            foreach( $marketplaces as $ID => $data) {
            	$optionArray[] = array('value'=> $ID, 'label'=> $data['label']);
            }            
        }
    
        return $optionArray;
    }
    
    public function getServiceUrl( $marketplaceId) {
        
        return $this->getMarketplaceData($marketplaceId, 'service');
    }
    
    public function getMarketplaceData( $marketplaceId, $column) {
        foreach( $this->_marketplaces as $region => $marketplaces) {
            if( isset( $marketplaces[$marketplaceId][$column])) {
                return $marketplaces[$marketplaceId][$column];
            }
        }
        return false;
    }   
    
    public function checkConnection( array $config = array()) {
        
        try {
            if( !$this->isFilled( $config)) {
                throw new Zend_Exception( $this->helper()->__('Required fields are missing'));
            }
            
            $request = new MarketplaceWebService_Model_GetFeedSubmissionListRequest( array(
                'Merchant' => $config['merchant_id'],
                'FeedProcessingStatusList' => array ('Status' => array ('_SUBMITTED_')),
            ));
            $response = $this->getService( $config['marketplace'], $config['api_key'], $config['api_secret'])->getFeedSubmissionList($request);
            if( !$response->isSetGetFeedSubmissionListResult()) {
                throw new Zend_Exception( $this->helper()->__('Wrong response from amazon!'));
            }
            return array( 'error'=>false, 'message'=>$this->helper()->__('Connection Successful!'));
        } catch (Exception $e) {
                        
            return array( 'error'=>true, 'message'=>$this->helper()->__('Error: '.$e->getMessage()."!"));
        }
    }
    
    /**
     * @param string $marketplaceID
     * @param string $apiKey Developer account API key
     * @param string $apiSecret Developer account API secret
     * @return MarketplaceWebService_Client
     */
    public function getService( $marketplaceID, $apiKey, $apiSecret) {
        
        $serviceUrl = $this->getServiceUrl( $marketplaceID);
        if( !$serviceUrl) {
            throw new Zend_Exception( $this->helper()->__('Marketplace ID %s is not supported!', $marketplaceID));
        }        
        $serviceConfig = array (
            'ServiceURL' => $serviceUrl,
            'ProxyHost' => null,
            'ProxyPort' => -1,
            'MaxErrorRetry' => 3,
        );
        $service = new MarketplaceWebService_Client(
            $apiKey,
            $apiSecret,
            $serviceConfig,
            "Koongo",
            "1.0.0"
        );
        return $service;
    }
       
    public function uploadFlatFile() {
        
        $profile = $this->getProfile();
        $config = $profile->getExportApiConfig( $this->_code);
        
        if( !$this->isFilled( $config)) {
            throw new Zend_Exception( Mage::helper('nscexport')->__('Amazon config is not complete! Check your config by test connection feature.'), Nostress_Nscexport_Model_Profile::CODE_NOT_ENABLED);
        }
               
        $fullFilename = $profile->getRealFileUrl();

        if( !is_file($fullFilename)) {
            throw new Zend_Exception( Mage::helper('nscexport')->__('Feed file does not exists!'), Nostress_Nscexport_Model_Profile::CODE_ERROR);
        }
        
        $marketplaceIdArray = array("Id" => array( $config['marketplace']));
        
        $feedType = $profile->getFeedApiFileType();
        if( empty( $feedType)) {
            $feedType = self::DEFAULT_FILE_TYPE;
        }
        
        $feed = file_get_contents( $fullFilename);
        $feedHandle = @fopen('php://temp', 'rw+');
        fwrite($feedHandle, $feed);
        rewind($feedHandle);
        $parameters = array (
            'Merchant' => $config['merchant_id'],
            'MarketplaceIdList' => $marketplaceIdArray,
            'FeedType' => $feedType,
            'FeedContent' => $feedHandle,
            'PurgeAndReplace' => false, //isset($config['purge_and_replace']) ? (bool) $config['purge_and_replace'] : false,
            'ContentMd5' => base64_encode(md5(stream_get_contents($feedHandle), true))
        );
        rewind($feedHandle);
        
        $request = new MarketplaceWebService_Model_SubmitFeedRequest($parameters);
        $response = $this->getService( $config['marketplace'])->submitFeed($request);
        if( !$response->isSetSubmitFeedResult()) {
            throw new Zend_Exception( $this->helper()->__('Wrong response from amazon!'));
        }
                
        $submitFeedResult = $response->getSubmitFeedResult();
        if ($submitFeedResult->isSetFeedSubmissionInfo()) {
            $submissionInfo = $submitFeedResult->getFeedSubmissionInfo();
            
            $profileSubmit = Mage::getModel( 'nscexport/submission_report');
            $profileSubmit
                ->setProfileId( $profile->getId())
                ->setMerchantId( $config['merchant_id'])
                ->setSubmissionId( $submissionInfo->getFeedSubmissionId())
                ->setSubmitDate( $submissionInfo->getSubmittedDate()->format('Y-m-d H:i:s'))
                ->setStatus( trim( $submissionInfo->getFeedProcessingStatus(),"_"))
                ->save();
        }
        
        return true;
    }
         
    public function getSubmissionList( array $params = array()) {
        
        try {
            $profile = $this->getProfile();
            $config = $profile->getExportApiConfig( $this->_code);
            
            if( !$this->isFilled( $config)) {
                throw new Zend_Exception( Mage::helper('nscexport')->__('Amazon config is not complete! Check your config by test connection feature.'), Nostress_Nscexport_Model_Profile::CODE_NOT_ENABLED);
            }
            
            $profileSubmitCollection = $profile->getSubmitCollection( $config['merchant_id']);
            $firstGenerate = false;
            $listIDs = array();
            if( !count( $profileSubmitCollection->load())) {
                $firstGenerate = true;
            } else {
                foreach( $profileSubmitCollection as $profileSubmit) {
                    // cannot reache limit 100 IDs per one API call
                    if( count( $listIDs) >= 90) {
                        break;
                    }
                    if( $profileSubmit->getProfileId() == $profile->getId() && !$profileSubmit->isDone()) {
                        $listIDs[] = $profileSubmit->getSubmissionId();
                    }
                }
            }

            if( $firstGenerate || count( $listIDs)) {
                $request = new MarketplaceWebService_Model_GetFeedSubmissionListRequest( array(
                        'Merchant' => $config['merchant_id'],
                ));
                if( count( $listIDs)) {
                    $fsil = new MarketplaceWebService_Model_IdList();
                    $fsil->setId( $listIDs);
                    $request->setFeedSubmissionIdList( $fsil);
                }
                $response = $this->getService( $config['marketplace'])->getFeedSubmissionList($request);
                if( !$response->isSetGetFeedSubmissionListResult()) {
                    throw new Zend_Exception( $this->helper()->__('Wrong response from amazon!'));
                }
                $list = $response->getGetFeedSubmissionListResult()->getFeedSubmissionInfoList();
                
                foreach( $list as $submissionInfo) {
                                
                    $profileSubmit = Mage::getModel( 'nscexport/submission_report');
                    $profileSubmit->load( $submissionInfo->getFeedSubmissionId(), 'submission_id');
                    if( !$profileSubmit->getId()) {
                        $profileSubmit
                            ->setProfileId( $profile->getId())
                            ->setMerchantId( $config['merchant_id'])
                            ->setSubmissionId( $submissionInfo->getFeedSubmissionId())
                            ->setSubmitDate( $submissionInfo->getSubmittedDate()->format('Y-m-d H:i:s T'))
                            ;
                    }
                    $profileSubmit
                        ->setStatus( trim( $submissionInfo->getFeedProcessingStatus(),"_"))
                        ->save()
                        ;
                }
            }
            
            // actualize submit collection for new submissions
            $profileSubmitCollection = $profile->getSubmitCollection();
            $limit = 10;
            $index = 0;
            foreach( $profileSubmitCollection as $profileSubmit) {
                
                // remove old submission
                if( $index++ >= $limit) {
                    $profileSubmitCollection->removeItemByKey( $profileSubmit->getId());
                    $profileSubmit->delete();
                    @unlink( $this->_getSubmitReportPath($profile, $profileSubmit->getSubmissionId()));
                }
                
                // get submissionResult for new submissions
                if( $profileSubmit->isDone() && !$profileSubmit->getReportMessage()) {
                    
                    $response = $this->getSubmissionResult( $profileSubmit->getSubmissionId());
                    if( isset( $response['filepath'])) {
                        $encoding = $this->getMarketplaceData( $config['marketplace'], 'encoding');
                        $result = $this->_parseResult( $response['filepath'], $encoding);
                        $profileSubmit->setReportMessage( $result);
                        $profileSubmit->save();
                    }
                }
            }

            return array( 'error'=>false, 'list'=> $profileSubmitCollection);
        } catch( Exception $e) {
            return array( 'error'=>true, 'message'=>$this->helper()->__('Error: '.$e->getMessage()."!"));
        }
    }
    
    public function _parseResult( $filepath, $encoding = null) {
        
        if( $encoding === false) $encoding = null;
        
        $result = array();
        $handle = @fopen( $filepath, "r");
        if ($handle) {
            while (($buffer = fgets($handle, 4096)) !== false) {
                $buffer = trim( $buffer);
                
                if( !mb_check_encoding( $buffer, 'UTF-8') ) {
                    $buffer = @mb_convert_encoding( $buffer, 'UTF-8', $encoding);
                }
                
                if( empty( $buffer)) {
                    break;
                }
                $result[] = $buffer;
            }
            fclose($handle);
        }
        return implode("<br />", $result);
    }
    
    protected function _getSubmitReportPath( $profile, $submissionID) {
                
        return $profile->getRealFileLocation()."submission_result_$submissionID.txt";
    }
    
    public function getSubmissionResult( $submissionID) {
    
        try {
            $profile = $this->getProfile();
            $filepath = $this->_getSubmitReportPath($profile, $submissionID);
            
            if (! is_file ( $filepath ) || ! is_readable ( $filepath )) {
                
                $config = $profile->getExportApiConfig( $this->_code);
                if( !$this->isFilled( $config)) {
                    throw new Zend_Exception( Mage::helper('nscexport')->__('Amazon config is not complete! Check your config by test connection feature.'), Nostress_Nscexport_Model_Profile::CODE_NOT_ENABLED);
                }
        
                
                file_put_contents($filepath, "");
                $request = new MarketplaceWebService_Model_GetFeedSubmissionResultRequest(array (
                     'Merchant' => $config['merchant_id'],
                     'FeedSubmissionId' => $submissionID,
                     'FeedSubmissionResult' => @fopen($filepath, 'rw+')
                ));
                $response = $this->getService( $config['marketplace'])->getFeedSubmissionResult($request);
        
                if( !$response->isSetGetFeedSubmissionResultResult()) {
                    throw new Zend_Exception( $this->helper()->__('Wrong response from amazon!'));
                }
                if (! is_file ( $filepath ) || ! is_readable ( $filepath )) {
                    throw new Zend_Exception( $this->helper()->__('File does not exist or not readable!'));
                }
            }
    
            return array( 'error'=>false, 'filepath'=>$filepath);
        } catch( Exception $e) {
    
            return array( 'error'=>true, 'message'=>$this->helper()->__('Error: '.$e->getMessage()."!"));
        }
    }
    
    public function initFormElements( Varien_Data_Form_Element_Fieldset $fieldset, array $config = array()) {
        
        $fieldset->addField('merchant_id', 'text', array(
            'label' => Mage::helper('nscexport')->__("Seller ID"),
            'name' => "merchant_id",
            'note' => Mage::helper('nscexport')->__("Seller ID, API key and API secret might be retrieved from your Amazon Seller account. <a tartet=\"_blank\" href=\"https://docs.koongo.com/pages/viewpage.action?pageId=23691846#id-1Inventory-Addproducts(M1)-ConnecttoAmazonusingAPI\">Learn More.</a>"),
        ));
        
        $marketplaceSource = $this->getMarketplacesToOptionArray();
        $fieldset->addField('marketplace', 'select', array(
            'label' => Mage::helper('nscexport')->__("Marketplace"),
            'name' => "marketplace",
            'title' => Mage::helper('nscexport')->__("Marketplace"),
            'values' => $marketplaceSource
        ));
        
        $fieldset->addField('api_key', 'text', array(
        		'label' => Mage::helper('nscexport')->__("API Key"),
        		'name' => "api_key",
        		'title' => Mage::helper('nscexport')->__("API Key")        		
        ));
        
        $fieldset->addField('api_secret', 'text', array(
        		'label' => Mage::helper('nscexport')->__("API Secret"),
        		'name' => "api_secret",
        		'title' => Mage::helper('nscexport')->__("API Secret")        		
        ));
        
        // $yesnoSource = Mage::getModel('adminhtml/system_config_source_yesno')->toOptionArray();
         
        //         $form->addField('purge_and_replace', 'select', array(
        //             'label' => Mage::helper('nscexport')->__("Purge and Replace"),
        //             'name' => "config[export_api][$code][purge_and_replace]",
        //             'title' => Mage::helper('nscexport')->__("Purge all my listing and replace then with this file!"),
        //             'values' => $yesnoSource,
        //             'class' => 'form-control',
        //             'value' => 0,
        //         ));
    
        $urlTestExportApiConnection = Mage::getUrl("*/*/testExportApiConnection");
        $fieldset->addField('test_connection', 'nostress_button', array(
                'label' => Mage::helper('nscexport')->__("Test Connection"),
                'name' => 'test_connection',
                'class' => 'save',
                'onclick' => "testExportApiConnection('$urlTestExportApiConnection');"
        ));
    }
    
    public function getHelpId() {
    
        return 'export_api_beslist';
    }
    
    /////////////////////////////////ORDER SYNC/////////////////////////////
    public function syncOrders()
    {
    	return false;
    }
}