<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
 * Export profile main class
 *
 * @category Nostress
 * @package Nostress_Koongo
 *
 */


class Nostress_Nscexport_Model_Io_Ftp extends Varien_Io_Ftp implements Nostress_Nscexport_Model_Io_ListableInterface
{
	/**
	 * Write a file from string, file or stream
	 *
	 * @param string $filename
	 * @param string|resource $src filename, string data or source stream
	 * @return int|boolean
	 */
	public function write($filename, $src, $mode=null)
	{
		//Koongo modification of original - check if filename is shorter than 1000 characters
		if (is_string($src) && strlen($src) < 1000 && is_readable($src)) {
			return @ftp_put($this->_conn, $filename, $src, $this->_config['file_mode']);
		} else {
			if (is_string($src)) {
				$stream = tmpfile();
				fputs($stream, $src);
				fseek($stream, 0);
			} elseif (is_resource($src)) {
				$stream = $src;
			} else {
				$this->_error = self::ERROR_INVALID_SOURCE;
				return false;
			}
	
			$result = ftp_fput($this->_conn, $filename, $stream, $this->_config['file_mode']);
			if (is_string($src)) {
				fclose($stream);
			}
			return $result;
		}
	}
	
    public function getItems()
    {
    
        if (is_array($children = @ftp_rawlist($this->_conn, '.'))) {
            $items = array(
                '..' => array(
                    'type' => self::DIR,
                    'path' => dirname($this->pwd())
                 )
            );
            
            foreach ($children as $child) {
                $chunks = preg_split("/\s+/", $child);
                list($item['permissions'], $item['number'], $item['uid'], $item['gid'], $item['size'], $item['month'], $item['day'], $item['time']) = $chunks;
                $item['type'] = $chunks[0]{0} === 'd' ? self::DIR : self::FILE;
                array_splice($chunks, 0, 8);
                
                if (strpos($item['time'], ':') !== false) {
                    $item['year'] = date('Y');
                } else {
                    $item['year'] = $item['time'];
                    $item['time'] = null;
                }
                $item['mtime'] = $item['day'].". ".$item['month']." ".$item['year'];
                if ($item['time']) {
                    $item['mtime'] .= " ".$item['time'];
                }
                
                $items[implode(" ", $chunks)] = $item;
            }
    
            return $items;
        }
    
        // Throw exception or return false < up to you
        return false;
    }
    
    public function lsDetailed() {
    
        if (is_array($children = @ftp_rawlist( $this->_conn, '.'))) {
            $items = array();
            foreach ($children as $child) {
                $chunks = preg_split("/\s+/", $child);
                list($item['rights'], $item['number'], $item['user'], $item['group'], $item['size'], $item['month'], $item['day'], $item['time']) = $chunks;
                $item['type'] = $chunks[0]{0} === 'd' ? 'directory' : 'file';
                array_splice($chunks, 0, 8);
                $items[implode(" ", $chunks)] = $item;
            }
    
            return $items;
        }
    
        // Throw exception or return false < up to you
        return false;
    }
}
