<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Cache_Categories extends Nostress_Nscexport_Model_Mysql4_Cache
{   	
	protected $_cacheName = 'Categories';
    protected $_lowestLevel = 0;
	const CATEGORY_PREFIX = 'category_';
	
    public function _construct()
    {    
        $this->_init('nscexport/cache_categories', 'product_id');
    }
    
	protected function defineColumns()
    {
		parent::defineColumns();       	     
    }
    
    protected function reloadTable()
    {
    	$sql = $this->getProductCategoriesUpdateSql();
//      	echo $sql."<br>";
//      	exit();
    	$this->_runQuery('SET SESSION group_concat_max_len = 1000000;'); //Group contcat can have size just 1024 chars
    	$this->_runQuery($sql);
    }
    
    public function setLowestLevel($level)
    {
    	if(isset($level) && is_numeric($level) && $level >= 0)
    		$this->_lowestLevel = $level;
    }
    
    public function setCategoryPathDelimiter($delimiter)
    {
    	$data = $this->_data;
    	if(empty($data))
    		$data = array();
    	$data[self::CATEGORY_PATH_DELIMITER] = $delimiter;
    	$this->setData($data);
    }
    
    protected function getProductCategoriesUpdateSql()
    {
    	$storeId = $this->getStoreId();
    	$sql = $this->getCleanMainTableSql();
		$sql .=  "INSERT INTO {$this->getMainTable()} ";
		$sql .= $this->getSubSelectTable($this->getProductCategoriesSql()).";";
		return $sql;
    }
    
    protected function getProductCategoriesSql()
    {
    	$mainTable = $this->getTable('catalog/category_product_index');
	    $mainTableAlias = self::CCPI;
	    $joinTableAlias = $this->getCategoryFlatTable(true);
		$joinTable = $this->getCategoryFlatTable();
	    
	    $select = $this->getEmptySelect();
	    $select->from(array($mainTableAlias => $mainTable), array( "product_id","({$this->getStoreId()}) as store_id"));
	    $select->join(
                    array($joinTableAlias => $joinTable),
                    $joinTableAlias.'.entity_id='.$mainTableAlias.'.category_id',
                    null
                    );
        $select->where($mainTableAlias.'.store_id=?', $this->getStoreId());
        $select->where($joinTableAlias.'.level>=?', $this->_lowestLevel);
        if(!$this->allowInactiveCategoriesExport())
        	$select->where($joinTableAlias.'.is_active=?', self::CATEGORY_ACTIVE);
	    $select->group('product_id');
	    $select = $this->_joinCategoryPath($select,false);
	    	    
    	$coreUrlRewriteColumns = $this->getColumns(self::CCUR,null,false,true);
	    if(!empty($coreUrlRewriteColumns))
	    {
		    $joinTableAlias = self::CCUR;
		    $joinTable =  $this->getTable('core/url_rewrite');
		    
		    $condition =  "{$joinTableAlias}.category_id = {$mainTableAlias}.category_id ";
		    $condition .=  "AND {$joinTableAlias}.store_id = {$this->getStoreId()} ";
		    $condition .=  "AND {$joinTableAlias}.product_id IS NULL ";
		    $select->joinLeft(
		    		array($joinTableAlias => $joinTable),
		    		$condition,
		    		null
		    );	    	    
	    }   	    

	    //define concat columns
	    $columns = $this->getCacheColumns();
	    $select->columns($this->groupConcatColumns($columns));
	    
	    return $select;
    }
    
    public function getCacheColumns()
    {
        parent::getCacheColumns();
        $columns = array_merge($this->getColumns($this->getCategoryFlatTable(true),null,false,true),$this->getColumns(self::NCCP,null,false,true),$this->getColumns(self::CCUR,null,false,true));
        $columns = $this->removeColumnPrefix($columns);
        return $columns;
    }
    
    protected function removeColumnPrefix($columns)
    {
        $result = array();
        foreach($columns as $alias => $column)
        {
            $alias = str_replace(self::CATEGORY_PREFIX,"",$alias);
            $result[$alias] = $column;            
        }
        return $result;
    }
	
	protected function getCleanMainTableSql()
	{
		return "DELETE FROM {$this->getMainTable()} WHERE store_id = {$this->getStoreId()};";
	}
}
