<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Cache_Mediagallery extends Nostress_Nscexport_Model_Mysql4_Cache
{   	  	
	protected $_cacheName = 'Media gallery';
		
    public function _construct()
    {    
        $this->_init('nscexport/cache_mediagallery', 'product_id');
    }
    
	protected function defineColumns()
    {
		parent::defineColumns();   

       	$this->_columns[self::CPAMG] = 
	        array(  "product_id" => "entity_id",
	        		"store_id" => "({$this->getStoreId()})",);  
	    $this->_columns[self::CPAMGV] = array(  "label" => "label");  
	        
    }
    
    protected function reloadTable()
    {
    	$sql = $this->getProductInfoUpdateSql();
    	//echo $sql."<br>";
    	//exit();
		$this->_runQuery('SET SESSION group_concat_max_len = 1000000;'); //Group contcat can have size just 1024 chars
    	$this->_runQuery($sql);
    }
    
    protected function getProductInfoUpdateSql()
    {
    	$storeId = $this->getStoreId();
    	$sql = $this->getCleanMainTableSql();
		$sql .=  "INSERT INTO {$this->getMainTable()} ";
		$sql .= $this->getSubSelectTable($this->getMediaGallerySql()).";";
		return $sql;
    }

    protected function getMediaGallerySql()
    {
    	$mainTable = $this->getTable('catalog/product_attribute_media_gallery');
	    $mainTableAlias = self::CPAMG;	    	    
	    
	    $subSelect = $this->getEmptySelect();
	    $subSelect->from(array($mainTableAlias => $mainTable), $this->getColumns($mainTableAlias));
	    
	    $joinTableValueAlias = 'value';
	    $joinTableAlias = $joinTableValueAlias;
		$joinTable = $this->getTable('catalog/product_attribute_media_gallery_value');
		
	    $subSelect->joinLeft(
                    array($joinTableAlias => $joinTable),
                    "{$joinTableAlias}.value_id={$mainTableAlias}.value_id AND {$joinTableAlias}.store_id = {$this->getStoreId()}",
                    null
                    );
                    
        $joinTableDefaultValueAlias = 'defaultValue'; 
        $joinTableAlias = $joinTableDefaultValueAlias;
		
       	$subSelect->joinLeft(
                    array($joinTableAlias => $joinTable),
                    "{$joinTableAlias}.value_id={$mainTableAlias}.value_id AND {$joinTableAlias}.store_id = 0",
                    null
                    );
		                    
       	$subSelect->columns(array("value","label" => "IFNULL(IFNULL({$joinTableValueAlias}.label,{$joinTableDefaultValueAlias}.label),'')"));       
        $subSelect->order(array($mainTableAlias.".entity_id" ,$joinTableValueAlias.".position",$joinTableDefaultValueAlias.".position",));
        
        //add condition to configuration
        //add condition - export only non excluded images
        if(!$this->excludedImagesExportEnabled())
        {
        	$subSelect->where('value.disabled = ?',0);
        	$subSelect->orWhere('value.disabled IS NULL AND defaultValue.disabled = 0');
        }
        
        $select = $this->getEmptySelect();
        $columns = array_keys($this->getColumns($mainTableAlias));           
        $select->from(array("concat_media_gallery" => $subSelect), $columns);
        $select->columns($this->groupConcatColumns($this->getCacheColumns()));
        $select->group("product_id");    
        return $select;
    }
    
    public function getCacheColumns()
    {    	    	
        parent::getCacheColumns();        
        $columns = array("label"=>"label","value"=>"value");        
        return $columns;        
    }
	
	protected function getCleanMainTableSql()
	{
		return "DELETE FROM {$this->getMainTable()} WHERE store_id = {$this->getStoreId()};";
	}
	
	protected function getMediaGalleryAttributeCode()
	{
		return $this->helper()->getGeneralConfig(Nostress_Nscexport_Helper_Data::PARAM_MEDIA_GALLERY);
	}
	
	protected function excludedImagesExportEnabled()
	{
		return $this->helper()->getGeneralConfig(Nostress_Nscexport_Helper_Data::PARAM_ALLOW_EXCLUDED_IMAGES_EXPORT);
	}
}