<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Cache_Price extends Nostress_Nscexport_Model_Mysql4_Cache
{   	
	const DEF_BUNDLE_OPTIONS_REQUIRED_ONLY = 1;
	const DEF_BUNDLE_OPTIONS_DEFAULT_ITEMS_ONLY = 0;
	
	protected $_cacheName = 'Price';
	protected $_commonColumns = array("product_id","store_id","customer_group_id","min_price","price");
	protected $_mainTableAlias = self::NCPRICE;
	
	/**
	 * Cutromer group id for price cache
	 * @var unknown_type
	 */
	protected $_customerGroupId = self::DEF_CUSTOMER_GROUP_NOT_LOGGED_IN;

	/**
	 * Fetch only required options during bundle product stock and price calculation.
	 * @var unknown_type
	 */
	protected $_bundleOptionsRequiredOnly = self::DEF_BUNDLE_OPTIONS_REQUIRED_ONLY;
	
	/**
	 * Fetch only default option items during bundle product stock and price calculation.
	 * @var unknown_type
	 */
	protected $_bundleOptionsDefaultItmesOnly = self::DEF_BUNDLE_OPTIONS_DEFAULT_ITEMS_ONLY;
	
    public function _construct()
    {    
        $this->_init('nscexport/cache_price', 'product_id');
    }    
    
    public function setCustomerGroupId($customerGroupId)
    {
    	if(isset($customerGroupId) && is_numeric($customerGroupId) && $customerGroupId >= 0)
    		$this->_customerGroupId = $customerGroupId;
    }
    
    public function setBundleOptionsRequiredOnly($bundleOptionsRequiredOnly)
    {
    	if(isset($bundleOptionsRequiredOnly))
    		$this->_bundleOptionsRequiredOnly = $bundleOptionsRequiredOnly;
    }
    
    public function setBundleOptionsDefaultItmesOnly($bundleOptionsDefaultItmesOnly)
    {
    	if(isset($bundleOptionsDefaultItmesOnly))
    		$this->_bundleOptionsDefaultItmesOnly = $bundleOptionsDefaultItmesOnly;
    }
    
    protected function getCustomerGroupId()
    {
    	return $this->_customerGroupId;
    }  

    protected function getBundleOptionsRequiredOnly()
    {
    	return $this->_bundleOptionsRequiredOnly;
    }
    
    protected function getBundleOptionsDefaultItmesOnly()
    {
    	return $this->_bundleOptionsDefaultItmesOnly;
    }
    
    protected function reloadTable()
    {
    	$this->cleanMainTable();
    	$this->insertRecords();
    	$this->updateConfigurablePrices();
    	$this->updateBundlePrices();
    }
    
    protected function getStore()
    {
    	return Mage::app()->getWebsite($this->getWebsiteId())->getDefaultStore();
    }
    
    /**
     * Insert records with columns min_price, price, qty
    */
    protected function insertRecords()
    {
    	$sql = $this->getInsertRecordsSql();
    	$this->_runQuery($sql,$this->getMainTable(), "Insert records. Filled columns: product_id, store_id, min_price, price.");
    }
    
    /**
     * Update prices for configurable and grouped products.
     * @return string
     */
    protected function updateConfigurablePrices()
    {
    	$sql = $this->getConfigurablePriceSql();
    	$this->_runQuery($sql,$this->getMainTable(),"Update prices for configurable products.");
    }
    
    /**
     * Update prices for bundle products.
     * @return string
     */
    protected function updateBundlePrices()
    {
    	$sql = $this->getBundlePriceSql();
    	$this->_runQuery($sql,$this->getMainTable(),"Update prices for bundle products.");
    }
	
	protected function cleanMainTable()
	{
		$sql = $this->getCleanMainTableSql();
		$this->_runQuery($sql,$this->getMainTable(), "Clean nostress_export_cache_price records for store #".$this->getStoreId()." and customer group #".$this->getCustomerGroupId());
		
	}
	
	/************************************ Sql query builders ***************************************/
	
	/**
	 * Insert records with columns min_price, price, qty
	*/
	protected function getInsertRecordsSql()
	{	
		$customerGroupId = $this->getCustomerGroupId();			
		
		$catProductFlatTableAlias = $this->getProductFlatTable(true);
		$catProductFlatTable = $this->getProductFlatTable();
		
		$catProdPriceIdxAlias = self::CIP;
		$catProdPriceIdx =  $this->getTable('catalogindex/price');
		
		$priceIndexColumns = $this->_getTableColumns($catProdPriceIdx);
		$joinByWebsite = true;
		if(isset($priceIndexColumns["store_id"]))
			$joinByWebsite = false;
		
		$select = $this->getEmptySelect();
		
		$columns = array( "product_id" => $catProductFlatTableAlias.".entity_id",
				"store_id" => "({$this->getStoreId()})",
				"customer_group_id" => "({$this->getCustomerGroupId()})",
				"min_price" => "(IFNULL({$catProdPriceIdxAlias}.min_price,{$catProductFlatTableAlias}.price))",
				"price"	=> "(IFNULL({$catProdPriceIdxAlias}.price,{$catProductFlatTableAlias}.price))"
				);
		
		$select->from(array($catProductFlatTableAlias => $catProductFlatTable), $columns);
		
		$joinCondition = "{$catProductFlatTableAlias}.entity_id = {$catProdPriceIdxAlias}.entity_id ";
		
		if($joinByWebsite)
			$joinCondition .= "AND {$catProdPriceIdxAlias}.website_id = {$this->getWebsiteId()} ";
		else
			$joinCondition .= "AND {$catProdPriceIdxAlias}.store_id = {$this->getStoreId()} ";
		
		$joinCondition .= "AND {$catProdPriceIdxAlias}.customer_group_id = {$this->getCustomerGroupId()} ";
		
		$select->joinLeft(
				array($catProdPriceIdxAlias => $catProdPriceIdx),
				$joinCondition,
				null
		);		
		
		//echo $select->__toString(); exit();
		
		$columnLabels = implode(",",$this->_commonColumns);
		$sql =  "INSERT INTO {$this->getMainTable()} ({$columnLabels}) ";
		$sql .= $this->getSubSelectTable($select).";";
		
		
		return $sql;
	}
	
	/**
	 * Returns updates query for update price for configurable and grouped products.
	 * If config product price is 0 than lowest price from child products is taken.
	 * @return string
	 */
	protected function getConfigurablePriceSql()
	{
		$cpeTable = $this->getTable("catalog/product");
		$cpeTableAlias = self::CPE;
	
		$cprTable = $this->getTable("catalog/product_relation");		
		$cprTableAlias = self::CPR;
	
		$select = $this->getEmptySelect();
		$select->from(array($this->_mainTableAlias => $this->getMainTable()),
				array( "product_id" => $cprTableAlias.".parent_id",
						"lowest_child_min_price" => "(MIN({$this->_mainTableAlias}.min_price))",
						"lowest_child_price" => "(MIN({$this->_mainTableAlias}.price))"));
		$select->join(
				array($cprTableAlias => $cprTable),
				"{$cprTableAlias}.child_id = {$this->_mainTableAlias}.product_id AND {$cprTableAlias}.parent_id IS NOT NULL",
				null
		);
		$select->where($this->_mainTableAlias.'.store_id =?', $this->getStoreId());
		$select->where($this->_mainTableAlias.'.customer_group_id=?', $this->getCustomerGroupId());
		$select->group($cprTableAlias.'.parent_id');
	
		$selectTable =  $this->getSubSelectTable($select);
		$selectTableAlias = 'lowest_price_table';
		 
		$updateSql = "UPDATE {$this->getMainTable()} AS {$this->_mainTableAlias} ";
		$updateSql .= "INNER JOIN {$cpeTable} AS {$cpeTableAlias} ON {$cpeTableAlias}.entity_id = {$this->_mainTableAlias}.product_id ";
		$updateSql .= "AND {$cpeTableAlias}.type_id IN ('configurable','grouped')";
		$updateSql .= "INNER JOIN {$selectTable} ";
		$updateSql .= "AS {$selectTableAlias} ON {$this->_mainTableAlias}.product_id = {$selectTableAlias}.product_id ";
		$updateSql .= "SET  {$this->_mainTableAlias}.min_price = {$selectTableAlias}.lowest_child_min_price, {$this->_mainTableAlias}.price = {$selectTableAlias}.lowest_child_price ";
		$updateSql .= "WHERE {$this->_mainTableAlias}.store_id = {$this->getStoreId()} ";
		$updateSql .= "AND {$this->_mainTableAlias}.customer_group_id = {$this->getCustomerGroupId()};";
		
// 		echo $updateSql;//$select->__toString();
//  		exit();
		
		return $updateSql;
	}
	
	/**
	 * Returns updates query for update price for bundle products.	 
	 * @return string
	 */
	public function getBundlePriceSql()
	{
		$cpbsTable = $this->getTable("bundle/selection");
		$cpbsTableAlias = self::CPBS;
		
		$cpboTable = $this->getTable("bundle/option");
		$cpboTableAlias = self::CPBO;
		
		$requiredOptionsOnly = $this->getBundleOptionsRequiredOnly();
		$defaultOptionItemsOnly = $this->getBundleOptionsDefaultItmesOnly();
						
		$optionMinPriceQuery = "(MIN({$this->_mainTableAlias}.min_price * {$cpbsTableAlias}.selection_qty))";
		$optionPriceQuery = "({$this->_mainTableAlias}.price * {$cpbsTableAlias}.selection_qty)";
		if($defaultOptionItemsOnly)
		{
			$optionMinPriceQuery = "(SUM({$this->_mainTableAlias}.min_price * {$cpbsTableAlias}.selection_qty))";
			$optionPriceQuery = "(SUM({$this->_mainTableAlias}.price * {$cpbsTableAlias}.selection_qty))";
		}
				
		//Prepare subselect for options with minimal price
		$subSelect = $this->getEmptySelect();
		$subSelect->from(array($this->_mainTableAlias => $this->getMainTable()),
				array( "product_id" => $this->_mainTableAlias.".product_id",
					   "parent_product_id" => $cpbsTableAlias.".parent_product_id",
					   "option_id" => self::CPBS.".option_id", //Bundle option id
					   "option_min_price" => $optionMinPriceQuery,
					   "option_price" =>  $optionPriceQuery));
		$subSelect->join(
				array($cpbsTableAlias => $cpbsTable),
				"{$this->_mainTableAlias}.product_id = {$cpbsTableAlias}.product_id",				
				null
		);
		
		$subSelect->join(
				array($cpboTableAlias => $cpboTable),
				"{$cpbsTableAlias}.option_id = {$cpboTableAlias}.option_id",
				null
		);
		
		if($requiredOptionsOnly)
			$subSelect->where($cpboTableAlias.'.required = 1');
		
		if($defaultOptionItemsOnly)
			$subSelect->where($cpbsTableAlias.'.is_default = 1');
		
		$subSelect->where($this->_mainTableAlias.'.store_id=?', $this->getStoreId());
		$subSelect->where($this->_mainTableAlias.'.customer_group_id=?', $this->getCustomerGroupId());
		$subSelect->group($cpbsTableAlias.'.option_id');					
		
// 		echo $subSelect->__toString(); exit();
		
		$optionsSubselectTable = $this->getSubSelectTable($subSelect);
		$optionsSubselectTableAlias = 'bundle_options';
		
		//Summarize options prices to get total minimal and standard price for bundle item		
		$select = $this->getEmptySelect();				
		$columns = array( "product_id" => $optionsSubselectTableAlias.".parent_product_id",
				"min_price" => "(SUM({$optionsSubselectTableAlias}.option_min_price))",
				"price"	=> "(SUM({$optionsSubselectTableAlias}.option_price))"
		);
		
		$select->from(array($optionsSubselectTableAlias => $optionsSubselectTable), $columns);
		$select->group($optionsSubselectTableAlias.'.parent_product_id');
		
		$bundleSubselectTable = $this->getSubSelectTable($select);
		$bundleSubselectTableAlias = 'bundle_totals';		

		$updateSql = "UPDATE {$this->getMainTable()} AS {$this->_mainTableAlias} ";
		$updateSql .= "INNER JOIN {$bundleSubselectTable} AS {$bundleSubselectTableAlias} ";
		$updateSql .= "ON {$bundleSubselectTableAlias}.product_id = {$this->_mainTableAlias}.product_id ";		
		$updateSql .= "SET  {$this->_mainTableAlias}.min_price = {$bundleSubselectTableAlias}.min_price, {$this->_mainTableAlias}.price = {$bundleSubselectTableAlias}.price ";
		$updateSql .= "WHERE {$this->_mainTableAlias}.store_id = {$this->getStoreId()} AND {$this->_mainTableAlias}.price <= 0 "; //Update price only if it is 0 (otherwise bundle product price is set as fixed)
		$updateSql .= "AND {$this->_mainTableAlias}.customer_group_id = {$this->getCustomerGroupId()};";

// 		echo $updateSql;//$select->__toString();
// 		exit();
		return $updateSql;	
	}
    
	protected function getCleanMainTableSql()
	{
		return "DELETE FROM {$this->getMainTable()} WHERE store_id = {$this->getStoreId()} AND customer_group_id = {$this->getCustomerGroupId()};";
	}
}