<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Cache_Profilecategory extends Nostress_Nscexport_Model_Mysql4_Cache
{   	
	protected $_mainTableAlias = self::NCPC;
	protected $_cacheName = 'Profilecategory';
    protected $_lowestLevel = 0;
    protected $_exportProfileId = 0;
    protected $_allowInactiveCategoriesExport = "1";
    protected $_loadAllProductCategoriesCondition = 0;
		
    public function _construct()
    {    
        $this->_init('nscexport/cache_profilecategory', 'export_id');
    }
    
    protected function reloadTable()
    {
    	$sql = $this->getCleanMainTableSql();    	
    	$this->_runQuery($sql);
    	
    	$this->insertRecords();
    	$this->updateProductCategoryId();   
    	
    	//Caclutate only if all categpries info is required
    	if($this->getLoadAllProductCategoriesCondition())
    	{	    	 	
	    	$this->updateProductCategories();	    	
    	}    	   	
    	$this->updateParentToChildsCategoryInfo();
    }        

    public function setLowestLevel($level)
    {
    	if(isset($level) && is_numeric($level) && $level >= 0)
    		$this->_lowestLevel = $level;
    } 
    
    public function setExportProfileId($profileId)
    {    	
    	$this->_exportProfileId = $profileId;    	
    }

    public function getExportProfileId()
    {
    	return $this->_exportProfileId;
    }
	
    public function setAllowInactiveCategoriesExport($status)
    {
    	if (isset($status)) {
    		$this->_allowInactiveCategoriesExport = $status;
    	}
    }
    
    protected function allowInactiveCategoriesExport()
    {
    	return $this->_allowInactiveCategoriesExport;
    }
    
    public function setLoadAllProductCategoriesCondition($loadAllProductCategoriesCondition)
    {
    	$this->_loadAllProductCategoriesCondition = $loadAllProductCategoriesCondition;
    }
    
    protected function getLoadAllProductCategoriesCondition()
    {
    	return $this->_loadAllProductCategoriesCondition;
    }
    
	protected function getCleanMainTableSql()
	{
		return "DELETE FROM {$this->getMainTable()} WHERE export_id = {$this->getExportProfileId()};";
	}
	
	/*
	 * Insert records with columns export_id, product_id, category_max_level
	*/
	protected function insertRecords()
	{
		$sql = $this->getInsertRecordsSql();		
		$this->_runQuery($sql, $this->getMainTable(), "Insert records. Filled columns: export_id, product_id, category_max_level .");
	}
	
	protected function updateProductCategoryId()
	{
		$sql = $this->getMaxLevelCategoryIdSql($this->getExportProfileId());
		$this->_runQuery($sql, $this->getMainTable(), "Update category id. Category with max level is selected.");
	}
	
	protected function updateProductCategories()
	{
		$sql = $this->getProductCategoriesSql();
		$this->_runQuery($sql, $this->getMainTable(), "Update product categories");
	}
	
	/**
	 * Updates category info from parent to child.
	 * @return string
	 */
	protected function updateParentToChildsCategoryInfo()
	{
		$sql = $this->getInsertChildProductsRecordsSql();
		$this->_runQuery($sql,$this->getMainTable(),"Update category info from parent to child.");
	}
	
	
	/************************************ Sql query builders ***************************************/
	
	/*
	 * Insert records
	*/
	protected function getInsertRecordsSql()
	{
		$select = $this->getSelectCategoryMaxLevelSql($this->getExportProfileId());
		$sql = $this->getReadConnection()->insertFromSelect($select, $this->getMainTable(), array("export_id", "product_id","category_max_level"));
		return $sql;
	}
	
	/**
	 * Returns updates query for max level column at cache product table
	 * @return string
	 */
	protected function getSelectCategoryMaxLevelSql($profileId)
	{
		$mainTableAlias = self::NCP;
		$mainTable = $this->getTable('nscexport/categoryproducts');
		
		$select = $this->getEmptySelect();
		$select->from(array($mainTableAlias => $mainTable), array("export_id", "product_id"));				
		$select->where("export_id = ?",$profileId);
		$select->group($mainTableAlias.'.product_id');
		 
		$catTableAlias = $this->getCategoryFlatTable(true);
		$catTable = $this->getCategoryFlatTable();
		 		 
		$select->joinLeft(
				array($catTableAlias => $catTable),
				$mainTableAlias.'.category_id='.$catTableAlias.'.entity_id',
				array('category_max_level' => 'MAX(level)')
		);
		 
		return $select;
	}
	
	/**
	 * Returns updates query for category it column, category with max level is selected.
	 * @return string
	 */
	protected function getMaxLevelCategoryIdSql($profileId)
	{
		$mainTableAlias = 'main_table';
		$mainTable = $this->getMainTable();	
		
		$catProdFilterTableAlias = self::NCP;
		$catProdFilterTable = $this->getTable('nscexport/categoryproducts');
	
		$catTableAlias = $this->getCategoryFlatTable(true);
		$catTable = $this->getCategoryFlatTable();	 				
		
		$sql = "UPDATE {$mainTable} AS {$mainTableAlias} ";
		$sql .= "LEFT JOIN {$catProdFilterTable} AS {$catProdFilterTableAlias} ON {$catProdFilterTableAlias}.product_id = {$mainTableAlias}.product_id AND {$catProdFilterTableAlias}.export_id = {$profileId} ";
		$sql .= "INNER JOIN {$catTable} AS {$catTableAlias} ON {$catTableAlias}.entity_id = {$catProdFilterTableAlias}.category_id AND {$catTableAlias}.level = {$mainTableAlias}.category_max_level ";
		$sql .= "SET {$mainTableAlias}.category_id = {$catTableAlias}.entity_id ";
		$sql .= "WHERE {$mainTableAlias}.export_id = {$this->getExportProfileId()} ";
		return $sql;
	}
	
	protected function getProductCategoriesSql()
	{
		$mainTableAlias = self::NCP;
		$mainTable = $this->getTable('nscexport/categoryproducts');
		
		$select = $this->getEmptySelect();
		$select->from(array($mainTableAlias => $mainTable), array("export_id", "product_id"));
		$select->where("export_id = ?",$this->getExportProfileId());
		$select->group($mainTableAlias.'.product_id');
		
		$joinTableAlias = $this->getCategoryFlatTable(true);
		$joinTable = $this->getCategoryFlatTable();
		
		//Join category flat table
		$select->join(
				array($joinTableAlias => $joinTable),
				$joinTableAlias.'.entity_id='.$mainTableAlias.'.category_id ',
				null
		);		
		$select->where($joinTableAlias.'.level>=?', $this->_lowestLevel);
		
		//Join Category Path
		$catPathCacheTableAlias = self::NCCP;
		$catPathCacheTable = $this->getTable('nscexport/cache_categorypath');
		
		$select->joinLeft(
				array($catPathCacheTableAlias => $catPathCacheTable),
				"{$catPathCacheTableAlias}.category_id={$mainTableAlias}.category_id AND {$catPathCacheTableAlias}.store_id ={$this->getStoreId()} ",
				null
		);
		
		
		if (!$this->allowInactiveCategoriesExport()) {
			$select->where($joinTableAlias.'.is_active=?', self::CATEGORY_ACTIVE);
		}
			
		//define concat columns
		$columns = $this->getCacheColumns('category');		
		$columns = Mage::getResourceModel('nscexport/cache_categories')->setStoreId($this->getStoreId())->getCacheColumns();
		
		$select->columns($this->groupConcatColumns($columns));
		$select->columns($this->groupConcatColumns(array("{$joinTableAlias}.entity_id"), ",", "category_ids"));		
		$selectSql = new \Zend_Db_Expr($select);
		
		$updateSql = "UPDATE {$this->getMainTable()} AS {$this->_mainTableAlias} ";
		$updateSql .= "INNER JOIN ( {$selectSql} ) ";
		$updateSql .= "AS categories ON {$this->_mainTableAlias}.product_id = categories.product_id AND  {$this->_mainTableAlias}.export_id = {$this->getExportProfileId()} ";
		$updateSql .= "SET  {$this->_mainTableAlias}.categories =  categories.concat_colum, {$this->_mainTableAlias}.category_ids =  categories.category_ids ";
		return $updateSql;
	}
	
	/*
	 * Insert records
	*/
	protected function getInsertChildProductsRecordsSql()
	{
		$select = $this->getSelectChildProductRecordSql();
		$sql = $select->insertIgnoreFromSelect($this->getMainTable(),array( "export_id","product_id","category_id","category_max_level","categories","category_ids"));
		return $sql;
	}
	
	protected function getSelectChildProductRecordSql()
	{
		$mainTableAlias = 'main_table';
		$mainTable = $this->getMainTable();
		 
		$catProdRelAlias = self::CPR;
		$catProdRel = $this->getTable('catalog/product_relation');
		 
		$parentCacheAlias = "pnkcpc";
		 
		$select = $this->getEmptySelect();
		$select->from(array($mainTableAlias => $mainTable), array( "export_id" => "('{$this->getExportProfileId()}')","product_id" => "{$catProdRelAlias}.child_id","category_id","category_max_level","categories","category_ids"));
		$select->join(
				array($catProdRelAlias => $catProdRel),
				"{$catProdRelAlias}.parent_id = {$mainTableAlias}.product_id",
				null
		);
		 
		$select->where($mainTableAlias.'.export_id=?', $this->getExportProfileId());
    	return $select;
	}
}
