<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Cache_Reviews extends Nostress_Nscexport_Model_Mysql4_Cache
{   	  	
	protected $_cacheName = 'Reviews';
	protected $_timestampFormat = Nostress_Nscexport_Model_Config_Source_Datetimeformat::STANTDARD_DATETIME_SQL;
	const REVIEW_STATUS_APPROVED = 1; 	
	const REVIEW_ENTITY_PRODUCT = 1;
	
	
    public function _construct()
    {    
        $this->_init('nscexport/cache_reviews', 'product_id');
    }
    
    public function setTimestampFormatForSql($timestampFormat)
    {
    	$this->_timestampFormat = $timestampFormat;
    }
    
    protected function getTimestampFormatForSql()
    {
    	return $this->_timestampFormat;
    }
    
	protected function reloadTable()
    {
    	$sql = $this->getCleanMainTableSql();    	
    	$this->_runQuery($sql);
    	
    	$this->insertRecords();
    }
    
    /*
     * Insert records with columns export_id, product_id, category_max_level
    */
    protected function insertRecords()
    {
    	$sql = $this->getInsertRecordsSql();
    	/*GROUP_CONCAT can be limited to 1024 -> increase limit in this session*/
    	$this->_runQuery('SET SESSION group_concat_max_len = 1000000;');
    	$this->_runQuery($sql, $this->getMainTable(), "Insert records. Filled columns: product_id, store_id, reviews .");
    }
    
    /*
     * Insert records
    */
    protected function getInsertRecordsSql()
    {
    	$select = $this->getReviewsSelect();      	
    	$sql = $this->getReadConnection()->insertFromSelect($select, $this->getMainTable(), array("product_id", "store_id","reviews"));
    	return $sql;
    }
    
    protected function getReviewsSelect()
    {
    	$reviewTable = $this->getTable('review/review');
    	$reviewTableAlias = self::R;
    	
    	$reviewDetailTable = $this->getTable('review/review_detail');
    	$reviewDetailTableAlias = self::RD;    	    	
   	    	
    	$subSelect = $this->getEmptySelect();
    	$subSelect->from(array($reviewTableAlias => $reviewTable), array('product_id' => $reviewTableAlias.'.entity_pk_value','store_id' => "({$this->getStoreId()})"));
    	$subSelect->columns($this->groupConcatColumns($this->getCacheColumns(),null,"reviews"));
    	$subSelect->group($reviewTableAlias.'.entity_pk_value');
 	    $subSelect->where('status_id = ?',self::REVIEW_STATUS_APPROVED);
 	    $subSelect->where('entity_id = ?',self::REVIEW_ENTITY_PRODUCT);
 	    $defaultBaseStoreId = Mage_Catalog_Model_Abstract::DEFAULT_STORE_ID;
    	
    	$subSelect->join(
    			array($reviewDetailTableAlias => $reviewDetailTable),
    			"{$reviewTableAlias}.review_id = {$reviewDetailTableAlias}.review_id ".
    			"AND ({$reviewDetailTableAlias}.store_id = {$this->getStoreId()} OR {$reviewDetailTableAlias}.store_id = {$defaultBaseStoreId} OR ISNULL({$reviewDetailTableAlias}.store_id)) ",
    			null
    	);    	    	
    	
    	$ratingSubselectTable = $this->getSubSelectTable($this->getRatingSubselect());
    	$ratingSubselectTableAlias = 'review_rating';
    	
    	$subSelect->joinLeft(
    			array($ratingSubselectTableAlias => $ratingSubselectTable),
    			"{$ratingSubselectTableAlias}.review_id = {$reviewTableAlias}.review_id ",
    			null
    	);
    	
    	return $subSelect;    	    	
    }
    
    protected function getRatingSubselect()
    {    	    	
    	$ratingOptionVoteTable = $this->getTable('rating/rating_option_vote');
    	$ratingOptionVoteTableAlias = self::ROV;
    	
    	$subSelect = $this->getEmptySelect();
    	$subSelect->from(array($ratingOptionVoteTableAlias => $ratingOptionVoteTable), array('review_id','rating' => "(AVG(percent))"));
    	$subSelect->group('review_id');
    	return $subSelect;    	    	
    }
    
    public function getCacheColumns()
    {    	    	
        parent::getCacheColumns();        
        $columns = array("review_id"=>self::R.".review_id",
        		"review_title" => self::RD.".title",
        		"review_detail" => self::RD.".detail",
        		"review_created_at" => new Zend_Db_Expr("DATE_FORMAT(".self::R.".created_at,'{$this->getTimestampFormatForSql()}')"),
        		"reviewer_name" => "IFNULL(".self::RD.".nickname ,'')",
        		"review_customer_id" => "IFNULL(".self::RD.".customer_id ,'')",
        		"review_rating" => "review_rating.rating",
        		);   

        if($this->getTimestampFormatForSql() == Nostress_Nscexport_Model_Config_Source_Datetimeformat::TIMESTAMP)
        	$columns["review_created_at"] = new Zend_Db_Expr("UNIX_TIMESTAMP(".self::R.".created_at)");
        
        return $columns;        
    }
	
    protected function getCleanMainTableSql()
    {
    	return "DELETE FROM {$this->getMainTable()} WHERE store_id = {$this->getStoreId()};";
    }
}