<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Cache_Stock extends Nostress_Nscexport_Model_Mysql4_Cache
{   
	const DEF_BUNDLE_OPTIONS_REQUIRED_ONLY = 1;
	const DEF_BUNDLE_OPTIONS_DEFAULT_ITEMS_ONLY = 0;
	
	protected $_cacheName = 'Stock';
	protected $_commonColumns = array("product_id", "website_id","qty", "qty_abs","stock_status", "always_in_stock");
	protected $_mainTableAlias = self::NCS;
		
	/**
	 * Fetch only required options during bundle product stock and price calculation.
	 * @var unknown_type
	 */
	protected $_bundleOptionsRequiredOnly = self::DEF_BUNDLE_OPTIONS_REQUIRED_ONLY;
	
	/**
	 * Fetch only default option items during bundle product stock and price calculation.
	 * @var unknown_type
	 */
	protected $_bundleOptionsDefaultItmesOnly = self::DEF_BUNDLE_OPTIONS_DEFAULT_ITEMS_ONLY;
	
    public function _construct()
  	{    
        $this->_init('nscexport/cache_stock', 'product_id');
   	}       
    
   	public function reload($websiteId)
   	{
	   	$this->setWebsiteId($websiteId);
	   	$this->setStoreId($this->getStore()->getId());
	   	$this->logStatus(self::STARTED,$this->getWebsiteId());
	   	$this->init();
	   	$this->reloadTable();
	   	$this->logStatus(self::FINISHED,$this->getWebsiteId());
   	}
   	
   	public function setBundleOptionsRequiredOnly($bundleOptionsRequiredOnly)
   	{
   		if(isset($bundleOptionsRequiredOnly))
   			$this->_bundleOptionsRequiredOnly = $bundleOptionsRequiredOnly;
   	}
   	
   	public function setBundleOptionsDefaultItmesOnly($bundleOptionsDefaultItmesOnly)
   	{
   		if(isset($bundleOptionsDefaultItmesOnly))
   			$this->_bundleOptionsDefaultItmesOnly = $bundleOptionsDefaultItmesOnly;
   	}
   	
    protected function getBundleOptionsRequiredOnly()
    {
    	return $this->_bundleOptionsRequiredOnly;
    }
    
    protected function getBundleOptionsDefaultItmesOnly()
    {
    	return $this->_bundleOptionsDefaultItmesOnly;
    }
   
    protected function reloadTable()
    {
    	$this->cleanMainTable();
    	$this->insertRecords();
    	$this->updateConfigurableGroupedStock();
     	$this->updateBundleStock();
    }
    
    protected function getStore()
    {
    	return Mage::app()->getWebsite($this->getWebsiteId())->getDefaultStore();
    }
    
    /**
     * Insert records with columns qty, is_in_stock, always_in_stock (manage stock = 0 for current product)
    */
    protected function insertRecords()
    {
    	$sql = $this->getInsertRecordsSql();
    	$this->_runQuery($sql,$this->getMainTable(), "Insert records. Filled columns: product_id, website_id, qty, stock_status, always_in_stock.");
    }
    
    /**
     * Update qty for configurable and grouped products.
     * @return string
     */
    protected function updateConfigurableGroupedStock()
    {
    	$sql = $this->getConfigurableGroupedStockSql();
    	$this->_runQuery($sql,$this->getMainTable(),"Update qty, stock_status, always_in_stock for configurable and grouped products.");
    }
    
    /**
     * Update stock for bundle products.
     * @return string
     */
    protected function updateBundleStock()
    {
    	$sql = $this->getBundleStockSql();
    	$this->_runQuery($sql,$this->getMainTable(),"Update stock for bundle products.");
    }
	
	protected function cleanMainTable()
	{
		$sql = $this->getCleanMainTableSql();
		$this->_runQuery($sql,$this->getMainTable(), "Clean nostress_export_cache_stock records.");
		
	}
	
	/************************************ Sql query builders ***************************************/
	
	/**
	 * Insert records with columns qty, stock_status
	*/
	protected function getInsertRecordsSql()
	{		
		$catInventoryStockItemTable = $this->getTable("cataloginventory/stock_item");
		$catInventoryStockItemTableAlias = self::CISI;
		
// 		$catInventoryStockStatusTable = $this->getTable("cataloginventory/stock_status");
// 		$catInventoryStockStatusTableAlias = self::CISS;				
		
		$select = $this->getEmptySelect();
		
		$columns = array( "product_id" => $catInventoryStockItemTableAlias.".product_id",
				"website_id" => "({$this->getWebsiteId()})",
				"qty" => $this->helper()->getRoundSql("{$catInventoryStockItemTableAlias}.qty",0),
				"qty_abs" => $this->helper()->getRoundSql("IF ({$catInventoryStockItemTableAlias}.qty < 0,0,{$catInventoryStockItemTableAlias}.qty)",0),
				"stock_status" => "IFNULL({$catInventoryStockItemTableAlias}.is_in_stock,1)",
				"always_in_stock" => new Zend_Db_Expr("IF(".$this->getManageStockCondition($catInventoryStockItemTableAlias).",1,0)")				
				);				
		
// 		$select->from(array($catInventoryStockStatusTableAlias => $catInventoryStockStatusTable), $columns);
		$select->from(array($catInventoryStockItemTableAlias => $catInventoryStockItemTable), $columns);

		
// 		$condition =   "{$catInventoryStockItemTableAlias}.product_id = {$catInventoryStockItemTableAlias}.product_id AND {$catInventoryStockStatusTableAlias}.stock_id = {$catInventoryStockItemTableAlias}.stock_id";		
// 		$select->join(
// 				array($catInventoryStockItemTableAlias => $catInventoryStockItemTable),
// 				$condition,
// 				null
// 		);	
		
// 		$select->where($catInventoryStockStatusTableAlias.'.website_id=?', $this->getWebsiteId());
		$select->where($catInventoryStockItemTableAlias.'.stock_id=?', self::DEF_STOCK_ID);
		
		//echo $select->__toString(); exit();
		
		$columnLabels = implode(",",$this->_commonColumns);
		$sql =  "INSERT INTO {$this->getMainTable()} ({$columnLabels}) ";
		$sql .= $this->getSubSelectTable($select).";";
		return $sql;
	}
	
	/**
	 * Returns updates query for update qty, stock_status, always_in_stock for configurable and grouped products.	 
	 * @return string
	 */
	protected function getConfigurableGroupedStockSql()
	{
		$cpeTable = $this->getTable("catalog/product");
		$cpeTableAlias = self::CPE;
	
		$cprTable = $this->getTable("catalog/product_relation");		
		$cprTableAlias = self::CPR;
	
		$select = $this->getEmptySelect();
		$select->from(array($this->_mainTableAlias => $this->getMainTable()),
				array( "product_id" => $cprTableAlias.".parent_id",
						"total_qty" => "(SUM({$this->_mainTableAlias}.qty_abs))",
						"child_in_stock" => "(BIT_OR({$this->_mainTableAlias}.stock_status))",
						"child_always_in_stock" => "(BIT_OR({$this->_mainTableAlias}.always_in_stock))",		
						));
		$select->join(
				array($cprTableAlias => $cprTable),
				"{$cprTableAlias}.child_id = {$this->_mainTableAlias}.product_id AND {$cprTableAlias}.parent_id IS NOT NULL",
				null
		);
		$select->where($this->_mainTableAlias.'.website_id=?', $this->getWebsiteId());		
		$select->group($cprTableAlias.'.parent_id');
		
		$selectTable =  $this->getSubSelectTable($select);
		$selectTableAlias = 'total_qty_table';
		 
		$updateSql = "UPDATE {$this->getMainTable()} AS {$this->_mainTableAlias} ";
		$updateSql .= "INNER JOIN {$cpeTable} AS {$cpeTableAlias} ON {$cpeTableAlias}.entity_id = {$this->_mainTableAlias}.product_id ";
		$updateSql .= "AND {$cpeTableAlias}.type_id IN ('configurable','grouped')";
		$updateSql .= "INNER JOIN {$selectTable} ";
		$updateSql .= "AS {$selectTableAlias} ON {$this->_mainTableAlias}.product_id = {$selectTableAlias}.product_id ";
		$updateSql .= "SET  {$this->_mainTableAlias}.qty = {$selectTableAlias}.total_qty, ";
		$updateSql .= "{$this->_mainTableAlias}.stock_status = {$selectTableAlias}.child_in_stock, ";
		$updateSql .= "{$this->_mainTableAlias}.always_in_stock = {$selectTableAlias}.child_always_in_stock ";
		$updateSql .= "WHERE {$this->_mainTableAlias}.website_id = {$this->getWebsiteId()};";		
		
// 		echo $updateSql;//$select->__toString();
//  		exit();
		
		return $updateSql;
	}
	
	/**
	 * Returns updates query for update qty, stock_status, always_in_stock for bundle products.	 
	 * @return string
	 */
	public function getBundleStockSql()
	{
		$cpbsTable = $this->getTable("bundle/selection");
		$cpbsTableAlias = self::CPBS;
		
		$cpboTable = $this->getTable("bundle/option");
		$cpboTableAlias = self::CPBO;
		
		$requiredOptionsOnly = $this->getBundleOptionsRequiredOnly();
		$defaultOptionItemsOnly = $this->getBundleOptionsDefaultItmesOnly();
		
		//Prepare subselect for options - calculate minimal qty and if at least one item for each option is in stock (and has always_in_stock)
		$subSelect = $this->getEmptySelect();
		
		$qtyPerSelectionQuery =  "IF({$cpbsTableAlias}.selection_qty > 1, {$this->_mainTableAlias}.qty_abs DIV {$cpbsTableAlias}.selection_qty , {$this->_mainTableAlias}.qty_abs)";
		$optionQqtyQuery = "(IF({$cpboTableAlias}.type IN ('select','radio'),SUM({$qtyPerSelectionQuery}),MIN({$qtyPerSelectionQuery})))";
		
		$optionInStockQuery = "(BIT_OR({$this->_mainTableAlias}.stock_status))";
		$optionAlwaysInStock = "(BIT_OR({$this->_mainTableAlias}.always_in_stock))";
		if($defaultOptionItemsOnly)
		{
			$optionInStockQuery = "(BIT_AND({$this->_mainTableAlias}.stock_status))";
			$optionAlwaysInStock = "(BIT_AND({$this->_mainTableAlias}.always_in_stock))";
		}
		
		$subSelect->from(array($this->_mainTableAlias => $this->getMainTable()),
				array( "product_id" => $this->_mainTableAlias.".product_id",
					   "parent_product_id" => $cpbsTableAlias.".parent_product_id",
					   "option_id" => self::CPBS.".option_id", //Bundle option id
					   "option_qty" => $optionQqtyQuery,					   
					   "option_in_stock" => $optionInStockQuery,
					   "option_always_in_stock" => $optionAlwaysInStock
				));
		$subSelect->join(
				array($cpbsTableAlias => $cpbsTable),
				"{$this->_mainTableAlias}.product_id = {$cpbsTableAlias}.product_id",				
				null
		);
		
		$subSelect->join(
				array($cpboTableAlias => $cpboTable),
				"{$cpbsTableAlias}.option_id = {$cpboTableAlias}.option_id",
				null
		);
		
		
		if($requiredOptionsOnly)
			$subSelect->where($cpboTableAlias.'.required = 1');
		
		if($defaultOptionItemsOnly)
			$subSelect->where($cpbsTableAlias.'.is_default = 1');
		
		$subSelect->where($this->_mainTableAlias.'.website_id=?', $this->getWebsiteId());		
		$subSelect->group($cpbsTableAlias.'.option_id');

		$optionsSubselectTable = $this->getSubSelectTable($subSelect);
		$optionsSubselectTableAlias = 'bundle_options';
			
		//Summarize options prices to get total minimal and standard price for bundle item		
		$select = $this->getEmptySelect();				
		$columns = array( "product_id" => $optionsSubselectTableAlias.".parent_product_id",
				"qty" => "(MIN({$optionsSubselectTableAlias}.option_qty))",
				"stock_status" => "(BIT_AND({$optionsSubselectTableAlias}.option_in_stock))",
				"always_in_stock" => "(BIT_AND({$optionsSubselectTableAlias}.option_always_in_stock))"				
		);
		
		$select->from(array($optionsSubselectTableAlias => $optionsSubselectTable), $columns);
		$select->group($optionsSubselectTableAlias.'.parent_product_id');
		
		$bundleSubselectTable = $this->getSubSelectTable($select);
		$bundleSubselectTableAlias = 'bundle_stock';	
		$updateSql = "UPDATE {$this->getMainTable()} AS {$this->_mainTableAlias} ";
		$updateSql .= "INNER JOIN {$bundleSubselectTable} AS {$bundleSubselectTableAlias} ";
		$updateSql .= "ON {$bundleSubselectTableAlias}.product_id = {$this->_mainTableAlias}.product_id ";		
		$updateSql .= "SET  {$this->_mainTableAlias}.qty = {$bundleSubselectTableAlias}.qty, {$this->_mainTableAlias}.stock_status = {$bundleSubselectTableAlias}.stock_status, ";
		$updateSql .= "{$this->_mainTableAlias}.always_in_stock = {$bundleSubselectTableAlias}.always_in_stock ";
		$updateSql .= "WHERE {$this->_mainTableAlias}.website_id = {$this->getWebsiteId()}; ";		

// 		echo $updateSql;//$select->__toString();
// 		exit();
		return $updateSql;	
	}
    
	protected function getCleanMainTableSql()
	{
		return "DELETE FROM {$this->getMainTable()} WHERE website_id = {$this->getWebsiteId()};";
	}
	
	/**
	 * Condition will return 1 if stock is not managed (=stock status must be 1) or 0 if stock status must be loaded from table.
	 * @param unknown_type $tableAlias
	 * @return string
	 */
	protected function getManageStockCondition($tableAlias)
	{
		$manageStockValue = (int) Mage::getStoreConfig(Mage_CatalogInventory_Model_Stock_Item::XML_PATH_MANAGE_STOCK, $this->getStoreId()); //Same for all stores
		$manageStockCondition = "";
		if($manageStockValue)
			$manageStockCondition = "{$tableAlias}.use_config_manage_stock = '0' AND {$tableAlias}.manage_stock = '0' ";
		else
			$manageStockCondition = "{$tableAlias}.use_config_manage_stock = '1' OR ({$tableAlias}.use_config_manage_stock = '0' AND {$tableAlias}.manage_stock = '0') ";
		return $manageStockCondition;
	}
}