<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Cache_Weee extends Nostress_Nscexport_Model_Mysql4_Cache
{   
	const DISCOUNT_TABLE_ALIAS_SUFFIX = "_discount";
	
	protected $_cacheName = 'Weee';
	protected $_commonColumns = array("product_id","website_id",self::WEEE_COLUMN_TOTAL_FINAL,self::WEEE_COLUMN_TOTAL_ORIGINAL);
	
    public function _construct()
    {    
        $this->_init('nscexport/cache_weee', 'product_id');
    }
    
    public function reload($websiteId)
    {
    	$this->_websiteId = $websiteId;
    	$this->_storeId = $this->getStore()->getId();
    	$this->logStatus(self::STARTED,$this->_websiteId);
    	$this->init();
    	$this->reloadTable();
    	$this->logStatus(self::FINISHED,$this->_websiteId);
    }
    
    protected function reloadTable()
    {
    	//Weee tax enabled
    	if(!$this->isWeeeEnabled())
    		return;
    	
    	$sql = $this->getUpdateTableStructureSql();
//    	echo $sql."<br>";
//    	exit();    		
    	$this->_runQuery($sql);

		$sql = $this->getUpdateTableSql();
//		echo $sql."<br>";
//    	exit();
    	$this->_runQuery($sql);

    	$sql = $this->getUpdateWeeeTotalSql();
//    		echo $sql."<br>";
// 		exit();  	    	
    	$this->_runQuery($sql);    	
    }
    
    protected function getStore()
    {
    	return Mage::app()->getWebsite($this->getWebsiteId())->getDefaultStore();
    }
    
    /**
     * Returns sql which:
     * Clear table rows by website.
     * Add and drop columns.
     */
	protected function getUpdateTableStructureSql()
    {
    	$sql = $this->getCleanMainTableSql();
    	$sql .= $this->getUpdateTableColumnsSql();
		return $sql;
    }

    /**
     * Returns select insert command for table rows update.
     */
    protected function getUpdateTableSql()
    {
    	$select = $this->getSelectWeeeSql();
    	
    	$columns = implode(",",array_merge($this->_commonColumns,$this->convertWeeeAttributesToColumnNames()));
    	$sql =  "INSERT INTO {$this->getMainTable()} ({$columns}) ";
		$sql .= $this->getSubSelectTable($select).";";
		return $sql;
    }
    
    protected function getUpdateWeeeTotalSql()
    {
    	$columnsOriginal = $this->getTableColumnsByString(self::WEEE_COLUMN_SUFFIX_ORIGINAL);
    	$columnsFinal = $this->getTableColumnsByString(self::WEEE_COLUMN_SUFFIX_FINAL);
    	if(empty($columnsOriginal) || empty($columnsFinal))
    		return "";
    	
    	$columnsOriginal = implode(" + ",$columnsOriginal);
    	$columnsFinal = implode(" + ",$columnsFinal);
    		
    	return "UPDATE {$this->getMainTable()} SET ".self::WEEE_COLUMN_TOTAL_ORIGINAL." = ({$columnsOriginal}), ".self::WEEE_COLUMN_TOTAL_FINAL." = ({$columnsFinal}) WHERE website_id = '{$this->getWebsiteId()}';";
    }
    
    protected function getSelectWeeeSql()
    {
    	$mainTable = $this->getTable('weee/tax');
	    $mainTableAlias = self::WT;	    	    
	    
	    $select = $this->getEmptySelect();
	    $select->from(array($mainTableAlias => $mainTable), array('product_id' => 'entity_id','website_id' => "({$this->getWebsiteId()})",'total'.self::WEEE_COLUMN_SUFFIX_ORIGINAL => "(0.00)",'total'.self::WEEE_COLUMN_SUFFIX_FINAL => "(0.00)"));
	    $select->group("{$mainTableAlias}.entity_id");
	    
	    $weeeAttributes = $this->getWeeeAttributes();
	    foreach ($weeeAttributes as $id => $code) 
	    {
	    	$select = $this->joinWeeeAttribute($select,$code,$id);
	    }
        return $select;
    }
    
    protected function joinWeeeAttribute($select,$attributeCode,$attributeId)
    {
    	$defaultCountry = $this->helper()->getDefaultTaxCountry($this->getStore());
    	$defaultState = $this->helper()->getDefaultTaxRegion($this->getStore());
    	
    	$joinTableAlias = self::WT."_{$attributeCode}";    	
    	$select = $this->_joinWeeeAttribute($select,$attributeId,$joinTableAlias,$this->getWebsiteId(),$defaultCountry,$defaultState);
    	
    	$joinTableAliasDefWebsite = $joinTableAlias."_defWebsite";    	
    	$select = $this->_joinWeeeAttribute($select,$attributeId,$joinTableAliasDefWebsite,'0',$defaultCountry,$defaultState);
    	
    	$joinTableAliasDefState = $joinTableAlias."_defState";    	
    	$select = $this->_joinWeeeAttribute($select,$attributeId,$joinTableAliasDefState,$this->getWebsiteId(),$defaultCountry,'*');
    	
    	$joinTableAliasDefWebsiteDefState = $joinTableAliasDefWebsite."_defState";    	
    	$select = $this->_joinWeeeAttribute($select,$attributeId,$joinTableAliasDefWebsiteDefState,'0',$defaultCountry,'*');
    	
    	$joinDiscountTableAlis = $joinTableAlias.self::DISCOUNT_TABLE_ALIAS_SUFFIX;
    	$select = $this->_joinWeeeDiscount($select,$joinDiscountTableAlis,$this->getWebsiteId());
    	
    	$valueOriginal = "(IFNULL({$joinTableAlias}.value,(IFNULL({$joinTableAliasDefWebsite}.value,IFNULL({$joinTableAliasDefState}.value,IFNULL({$joinTableAliasDefWebsiteDefState}.value,'0'))))))";
    	$valueFinal = $valueOriginal;
    	if($this->isWeeeDiscounted())
    	{
    		$valueFinal = "({$valueFinal} * (IFNULL({$joinDiscountTableAlis}.value,100)/100))";
    	}
    	
    	$select->columns(array($attributeCode.self::WEEE_COLUMN_SUFFIX_FINAL => $valueFinal));
    	$select->columns(array($attributeCode.self::WEEE_COLUMN_SUFFIX_ORIGINAL => $valueOriginal));
    	
    	return $select;
    }
    
    protected function _joinWeeeAttribute($select,$attributeId,$joinTableAlias,$websiteId,$country,$state)
    {
    	if($state == '0')
    		$state = "*";
    	
    	$mainTableAlias = self::WT;
    	$joinTable = $this->getTable('weee/tax');
    	$condition = "{$joinTableAlias}.entity_id = {$mainTableAlias}.entity_id AND {$joinTableAlias}.website_id = '{$websiteId}' ";
    	$condition .= "AND {$joinTableAlias}.country = '{$country}' AND {$joinTableAlias}.state = '{$state}' AND {$joinTableAlias}.attribute_id = {$attributeId}";
    	$select->joinLeft(
                    array($joinTableAlias => $joinTable),
                    $condition,
                    null
                    );
        
        return $select;
    }
    
    protected function _joinWeeeDiscount($select,$joinTableAlias,$websiteId)
    {
    	if(!$this->isWeeeDiscounted())
    		return $select;
    	$mainTableAlias = self::WT;
    	$joinTable = $this->getTable('weee/discount');
    	
        $condition = "{$joinTableAlias}.entity_id = {$mainTableAlias}.entity_id AND {$joinTableAlias}.website_id = '{$websiteId}' AND  {$joinTableAlias}.customer_group_id = '".self::DEF_CUSTOMER_GROUP_NOT_LOGGED_IN."'";
        $select->joinLeft(
                    array($joinTableAlias => $joinTable),
                    $condition,
                    null
                    );
    	return $select;
    }

    protected function getUpdateTableColumnsSql()
    {
    	$tableColumns = $this->getAllTableColumns(true);
    	$weeeAttributes = $this->convertWeeeAttributesToColumnNames();      	    	    	
    	
    	//cmp attributes and update columns
    	$columnsToDelete = array_diff($tableColumns,$weeeAttributes);
    	$columnsToAdd = array_diff($weeeAttributes,$tableColumns);
    	$sql = "";
    	foreach ($columnsToDelete as $column)
    	{
    		$sql .= $this->getDropColumnSql($column);
    	}
    	
    	foreach ($columnsToAdd as $column)
    	{
    		$sql .= $this->getAddColumnSql($column);
    	}
    	return $sql;    	
    }         
	
	protected function getCleanMainTableSql()
	{
		return "DELETE FROM {$this->getMainTable()} WHERE website_id = {$this->getWebsiteId()};";
	}
	
	protected function getAddColumnSql($column)
	{
		return "ALTER TABLE {$this->getMainTable()} ADD COLUMN `{$column}` decimal(12,4) default '0.00' AFTER `total".self::WEEE_COLUMN_SUFFIX_FINAL."`; ";			
	}
	
	protected function getDropColumnSql($column)
	{
		return "ALTER TABLE {$this->getMainTable()} DROP `{$column}`; ";			
	}
	
	protected function getAllTableColumns($removeCommonColumns = false)
    {
        $describe = $this->_getTableColumns($this->getMainTable());
        $columns = array_keys($describe);
        if($removeCommonColumns)
        {
        	$columns = array_diff($columns,$this->_commonColumns);
        }
        return $columns;
    }
    
    protected function getTableColumnsByString($containsString)
    {
    	$columns = $this->getAllTableColumns(true);
    	if(empty($columns))
    		return array();
		if(empty($containsString))
    		return array_values($this->getWeeeAttributes());
    		
    	$resultColumns = array();
    	foreach ($columns as $column)
    	{
    		if(strpos($column,$containsString) !== false)
    		{
    			$resultColumns[] = $column;
    		}
    	}
		return $resultColumns;
    }
}