<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Categoryproducts extends Mage_Core_Model_Mysql4_Abstract
{
	const PRODUCT_SQL = "COUNT(DISTINCT product_id)";
	
    public function _construct()
    {    
        // Note that the export_id refers to the key field in your database table.
        $this->_init('nscexport/categoryproducts', 'entity_id');
    }
    
    public function insertRecords($profileId,$categoryProducts)
    {
    	$query = "";
    	foreach($categoryProducts as $categoryId => $productIds)
    	{
    		foreach($productIds as $productId)
    		{
    			$query .= $this->insertRecordQuery($profileId,$categoryId,$productId,true);
    		}
    	}
    	//file_put_contents("query.sql", $query);
    	$this->_runQuery($query);
    }
    
    public function selectInsertRecords($profileId,$categoryIds,$store_id)
    {    	
    	if(empty($categoryIds))
    		return;
    	
    	$mainTableAlias = "ccp";
    	$select = Mage::getModel('nscexport/categoryproducts')->getCollection()->getSelect()->reset();
    	$select->from(array($mainTableAlias => $this->getTable('catalog/category_product')),array("entity_id" => "('')","export_id" => "({$profileId})","category_id","product_id"));
   	    	
    	$joinTable = $this->getTable('catalog/product_website');
    	$joinTableAlias = "cpw";
    	$websiteId = Mage::app()->getStore($store_id)->getWebsiteId();
    	$select->join(
                    array($joinTableAlias => $joinTable),
                    "{$joinTableAlias}.product_id = {$mainTableAlias}.product_id AND {$joinTableAlias}.website_id = '{$websiteId}'",
                    null
                    );

        $joinTable = $this->getTable('catalog/product');
    	$joinTableAlias = "cpe";           
    	$select->join(
                    array($joinTableAlias => $joinTable),
                    "{$joinTableAlias}.entity_id = {$mainTableAlias}.product_id",
                    null
                    );
                    
    	$select->where("category_id IN (?)",$categoryIds); 
    	$selectString = $select->__toString();    	    	
    	
    	$query = "INSERT INTO ".$this->getMainTable()." ($selectString)";
    	$this->_runQuery($query);
    }
    
    public function deleteRecords($profileId,$productId=null)
    {
    	$query = $this->deleteProfileQuery($profileId,$productId);
    	$this->_runQuery($query);
    }
    
    public function isCategoryInProfile($categoryId,$profileId)
    {
    	$queryString = $this->selectCategoryProfileQuery($categoryId,$profileId);
    	$qRes = $this->_runSelectQuery($queryString,false);
    	if(!isset($qRes) || $qRes === false)
    		return false;
    	else
    		return true;
    }
    
    public function getProfileRecordCount($profileId)
    {
    	$queryString = $this->selectProfileRecordsCountQuery($profileId);
    	$result = $this->_runSelectQuery($queryString,false);
    	if(is_array($result) && isset($result[self::PRODUCT_SQL]))
    		return $result[self::PRODUCT_SQL];
    	return 0;
    }
    
    public function duplicateCategoryProducts($oldProfileId, $newProfileId)
    {
    	$selectSql = "SELECT (''), '{$newProfileId}',category_id, product_id FROM {$this->getMainTable()} WHERE export_id = {$oldProfileId}";    	
    	$query = "INSERT INTO ".$this->getMainTable()." ($selectSql)";
    	$this->_runQuery($query);
    }
    
    private function selectCategoryProfileQuery($categoryId,$profileId)
    {
    	$table = $this->getMainTable();
    	return "SELECT 1 FROM ".$table." WHERE ".$table.".export_id = ".$profileId." AND ".$table.".category_id = ".$categoryId.";";    	
    }
    
	private function selectCategoryProductQuery($productId,$categoryId,$profileId)
    {
    	$table = $this->getMainTable();
    	return "SELECT 1 FROM ".$table." WHERE ".$table.".export_id = ".$profileId." AND ".$table.".category_id = ".$categoryId."AND ".$table.".product_id = ".$productId.";";    	
    }
    
    private function selectProfileRecordsCountQuery($profileId)
    {
    	$table = $this->getMainTable();
    	return "SELECT ".self::PRODUCT_SQL." FROM ".$table." WHERE ".$table.".export_id = ".$profileId.";";      	
    }
    
    private function deleteProfileQuery($profileId,$productId)
    {
    	$sql =  "DELETE FROM ".$this->getMainTable()." WHERE ";
    	$where = array();
    	if(isset($profileId))
    		$where[] = $this->getMainTable().".export_id = ".$profileId." "; 
    	if(isset($productId))
    		$where[] = $this->getMainTable().".product_id = ".$productId." ";   
    	
    	$sql .= implode(" AND ",$where);
    	$sql .= ";";
    	return $sql; 	   	
    }
    
    private function insertRecordQuery($profileId,$categoryId,$productId)
    {
    	$query = "INSERT INTO ".$this->getMainTable()." (export_id,category_id,product_id) ";
    	$query .= "SELECT '".$profileId."','".$categoryId."',entity_id FROM {$this->getTable('catalog/product')} WHERE entity_id = {$productId}; ";
    	return $query;
    }
    
    private function _runQuery($queryString)
    {
    	if(!isset($queryString) || $queryString == "")
    		return $this;
    	$this->beginTransaction();
        try {
            
            $this->_getWriteAdapter()->query($queryString);       
            $this->commit();
        }
        catch (Exception $e){
            $this->rollBack();
            throw $e;
        }
        return $this;
    }
    
    private function _runSelectQuery($queryString,$fatchAll)
    {
    	//select query
    	$read = $this->_getReadAdapter();

		if($fatchAll)
			$res = $read->fetchAll($queryString); //get array
		else
			$res = $read->fetchRow($queryString); //fetch row
		
		return $res;
    }
}