<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
* Resource  of product loader for export process
* @category Nostress
* @package Nostress_Nscexport
*
*/

class Nostress_Nscexport_Model_Mysql4_Data_Loader extends Nostress_Nscexport_Model_Mysql4_Abstract
{
    //Category product relation table
    const CPF = 'cpf';
    const CPE = 'cpe'; //catalog product entity
    const PCPF = 'pcpf'; //Parent catalog product flat
    const CCF = 'ccf';
    const PCCF = 'pccf'; //Parent category flat
    const CCPI = 'ccpi';
    const CPR = 'cpr';  //Catalog product relation
    const CPBS = 'cpbs'; //Catalog product bundle selection
    const CPBO = 'cpbo'; //Catalog product bundle option
    const CISS = 'ciss'; //Catalog inventory stock status
    const PCISS = 'pciss'; //Parent product join of Catalog inventory stock status
    const CISI = 'cisi'; //Catalog inventory stock item
    const CPAMG = 'cpamg'; //Catelog product entity media gallery
	const CPAMGV = 'cpamgv'; //Catelog product entity media gallery value
    const PCISI = 'pcisi'; //Parent product - Catalog inventory stock item
    const CPSA = 'cpsa'; //Catalog product super attribute
	const CPSAL = 'cpsal'; //Catalog product super attribute label
	const EA = 'ea'; //Entity attribute
    const RRA = 'rra'; //Review aggregate
	const CIP = 'cip'; //Catalog product price index
	const CUR = 'cur'; // Core Url Rewrite "Url Rewrites"	
	const EUR = 'eur'; // Enterprise Url Rewrite "Url Rewrites"
	const CCUR = 'ccur'; // Core Url Rewrite "Url Rewrites" for category url load
	const WT = 'wt'; // Weee tax
	const R = 'r'; //Review
	const RD = 'rd'; //Review detail
	const ROV = 'rov'; //Rating option vote
	const TC = 'tax_class'; //Tax class

    const NPACD = 'npacd'; // Nostress product all category data
    const NCPML = 'ncpml'; // Nostress category product max level
    const NCCP = 'nccp'; //Nostress cache category path
    const NCSA = 'ncsa'; //Nostress cache super attributes
    const NCT = 'nct'; //Nostress cache tax
    const NCPRICE = 'ncprice'; //Nostress cache price
    const NCS = 'ncs'; //Nostress cache stock
    const PNCS = 'pncs'; //Parent product join of Nostress cache stock
    const NCC = 'ncc'; //Nostress cache categories
    const NCEC = 'ncec'; //Nostress cache enginecategory
	const NCMG = 'ncmg';//Nostress media gallery
    const NR = 'nr';   //Nostress records
    const NCP = 'ncp'; //Nostress category path
	const NSA = 'nsa'; // Nostress super attributes
    const NEC = 'nec'; //Nostress engine category
    const NCW = 'ncw'; // Nostress cache weee
    const NCCML = 'nccml'; //Nostress cache category max level
    const NCPC = 'ncpc'; //Nostress cache profile category
    const NCR = 'ncr'; //Nostress cache reviews
    
	const DEF_IMAGE_FOLDER = "media/catalog/product";
	const DEF_REVIEW_URL = "review/product/list/id/";

	const VALUE_COLUMN_SUFFIX = '_value';
	const MAIN_TABLE_SUBST = '{{main_table}}';
	const CATEGORY_ACTIVE = '1';

	//Weee parameters
	const WEEE_COLUMN_TOTAL_ORIGINAL = "total";
	const WEEE_COLUMN_TOTAL_FINAL = "total_final";
	const WEEE_COLUMN_SUFFIX_ORIGINAL = "";
	const WEEE_COLUMN_SUFFIX_FINAL = "_final";

	//Loader parameters
	const EXPORT_ID = "export_id";
	const TAXONOMY_CODE = "taxonomy_code";
	const CONDITIONS = "conditions";
	const BATCH_SIZE = "batch_size";
	const DATETIME_FORMAT = "datetime_format";
	const CATEGORY_PATH_DELIMITER = "category_path_delimiter";
	const CATEGORY_LOWEST_LEVEL = "category_lowest_level";
	const STOCK_STATUS_DEPENDENCE = "stock_status_dependence";
	const SORT_ATTRIBUTE = "sort_attribute";
	const PRICE_CUSTOMER_GROUP_ID = "price_customer_group_id";
	const SORT_ORDER = "sort_order";
	const CURRENCY = "currency";
	const EXPORT_ALL_IMAGES = "all_images";
	const EXPORT_ALL_CATEGORIES = "all_categories";
	const EXPORT_REVIEWS = 'load_product_reviews';

	//Default values
	const DEF_BATCH_SIZE = 1000;
	const DEF_CATEGORY_PATH_DELIMITER = "/";
	const DEF_CUSTOMER_GROUP_NOT_LOGGED_IN = 0;
	const DEF_STOCK_ID = 1;
	const SORT_ATTRIBUTE_ALIAS = "nsc_sort_attribute";

    /**
     * Current scope (store Id)
     *
     * @var int
     */
    protected $_storeId;
    protected $_websiteId;
    protected $_columns;
    protected $_productFlatMandatoryColumns = array();
    protected $_multiColumns = array();
    protected $_staticColumns = array();
    protected $_data;
    protected $_select;
    protected $_params;
    protected $_helper;
    protected $_atttibutes = '*';
    protected $_defaultAttributes = array();
    protected $_offset = 0;
	protected $_placeholders = array();
	
    public function _construct()
    {
        $this->_init('catalog/product', 'entity_id');
    }

    /**
     * Initialize collection select
     * Redeclared for remove entity_type_id condition
     * in catalog_product_entity we store just products
     *
     * @return Mage_Catalog_Model_Resource_Eav_Mysql4_Product_Collection
     */
    public function init()
    {
    	$this->defineColumns();
    	$this->resetOffset();
		$select = $this->getSelect();
		$select->reset();
        return $this;
    }

    public function loadBatch()
    {
    	$select = clone $this->getSelect();
    	$batchSize = $this->getBatchSize();
    	$offset = $this->getOffset();

    	$select->limit($batchSize,$offset);
    	$batch = $this->_runSelectQuery($select);
    	$this->setOffset($batchSize+$offset);

    	return $batch;
    }

    public function loadAll()
    {
    	$select = clone $this->getSelect();
    	$batch = $this->_runSelectQuery($select);
    	return $batch;
    }

    protected function defineColumns()
    {
        $this->_columns = array();

        $this->_columns[$this->getCategoryFlatTable(true)] = array( "category_id" => "entity_id",
                                                            		"category_name" => "name",
                                                                  	"category_path_ids" => "path",
                                                                 	"category_level" => "level",
                                                                 	"category_parent_id" => "parent_id",
        															"category_url_key" => "url_key");        															        															
        															
        //$this->defineCategoryTaxonomyColumns();

        $this->_columns[self::NCCP] = array("category_path" => "category_path",
        									"category_path_url_key" => "category_path_url_key",
       										"category_root_name" => "category_root_name",
        									"category_root_id" => "category_root_id");

        if($this->getUrlAttributeSource() == Nostress_Nscexport_Model_Config_Source_Urlsource::FLAT_CATALOG)
        {
        	$this->_columns[$this->getCategoryFlatTable(true)]["category_url"] = new Zend_Db_Expr("(CONCAT('{$this->getStoreBaseUrl()}',IFNULL({$this->getCategoryFlatTable(true)}.url_path,CONCAT({$this->getCategoryFlatTable(true)}.url_key,'{$this->getStoreConfig(Mage_Catalog_Helper_Category::XML_PATH_CATEGORY_URL_SUFFIX)}'))))");
        	//category url is loaded from category flat table
        	$this->_columns[self::CCUR] = array();
        }
        else 
        {
        	$this->_columns[self::CCUR] = array("category_url" => new Zend_Db_Expr("(CONCAT('{$this->getStoreBaseUrl()}',".self::CCUR.".request_path))"));
        }                
        
        $defaultCatPathDelim = self::DEF_CATEGORY_PATH_DELIMITER;
		$userCatPathDelim = $this->getCategoryPathDelimiter();
		if($userCatPathDelim !== $defaultCatPathDelim)
		{						 
			$this->_columns[self::NCCP]["category_path"] = new Zend_Db_Expr("REPLACE(".self::NCCP.".category_path,'{$defaultCatPathDelim}','{$userCatPathDelim}')");						
		}
		
        $this->_columns[$this->getCategoryFlatTable(true,null,true)] = array("category_parent_name" => "name");
        $this->dispatchDefineColumnsEvent();
    }

    protected function defineCategoryTaxonomyColumns()
    {
        $tableAlias = $this->getCategoryFlatTable(true);
        $columns = Mage::getModel('nscexport/taxonomy_setup')->getTaxonomyAttributeCodes();
        if(empty($columns))
            return;

        $columns = array_combine($columns,$columns);

        $this->_columns[$tableAlias] = array_merge($this->_columns[$tableAlias],$columns);
    }

    protected function getColumns($tableAlias,$defualt = null,$groupConcat = false,$addTablePrefix = false)
    {
        if(array_key_exists($tableAlias,$this->_columns))
            $result = $this->_columns[$tableAlias];
        else
            $result = $defualt;

       if($groupConcat)
           	$result = $this->groupConcatColumns($result);
       if($addTablePrefix)
			$result = $this->addTablePrefix($tableAlias,$result);
       return $this->filterColumns($result);
    }

    public function getAllColumns()
    {
    	if(!isset($this->_columns))
    		$this->defineColumns();

        $columns = array();
        foreach ($this->_columns as $tableColumns)
        {
        	$columns = array_merge($columns,$tableColumns);
        }
        return $columns;
    }

    public function getMultiColumns()
    {
    	return $this->_multiColumns;
    }

    public function getStaticColumns()
    {
    	return $this->_staticColumns;
    }

    protected function filterColumns($columns)
    {
        if($this->_atttibutes == '*')
            return $columns;

        if(!isset($columns) || !is_array($columns))
        	return $columns;

        if(isset($this->_atttibutes) && is_array($this->_atttibutes))
        {
        	foreach ($columns as $key => $column)
	        {
	            if(!in_array($key,$this->_atttibutes))
	                unset($columns[$key]);
	        }
        }

        return $columns;
    }

	protected function addTablePrefix($tableAlias,$columns)
    {
    	foreach ($columns as $key => $value)
    	{
    		if((is_object($value) && get_class($value) == "Zend_Db_Expr") || (isset($value[0]) && $value[0] == "("))
    			continue;
    		$columns[$key] = $tableAlias.".".$value;
    	}

    	return $columns;
    }

    protected function groupConcatColumns($columns, $groupRowSeparator = null, $groupedColumnAlias = null)
    {
        return $this->helper()->groupConcatColumns($columns, $groupRowSeparator, $groupedColumnAlias);
    }

	public function joinCategoryPath()
	{
		return $this->_joinCategoryPath($this->getSelect());
	}

	protected function _joinCategoryPath($select,$addColumns = true)
	{
		$mainTableAlias = $this->getCategoryFlatTable(true);
		$joinTableAlias = self::NCCP;
		$joinTable = $this->getTable('nscexport/cache_categorypath');
		$columns = null;
		if($addColumns)
			$columns = $this->getColumns($joinTableAlias);

		$select->joinLeft(
                    array($joinTableAlias => $joinTable),
                    "{$joinTableAlias}.category_id ={$mainTableAlias}.entity_id AND {$joinTableAlias}.store_id ={$this->getStoreId()}",
                    $columns
                    );
		
// 		if(!empty($columns))
// 		{
// 			$select->columns( array("category_pathTEST" => new Zend_Db_Expr("REPLACE(nccp.category_path,'/',' ; ')")));//REPLACE(nccp.category_path,'/',' ; ')
// 			var_dump($columns);
			
// 			echo $select->__toString();
// 			exit();
// 		}
		return $select;
	}

    /**
     * Order records by category is
     */
	public function orderByCategory()
	{
		$select = $this->getSelect();
		$select->order($this->getCategoryFlatTable(true).".entity_id");
	}

	public function joinParentCategory()
	{
		$mainTableAlias = $this->getCategoryFlatTable(true);
		$joinTableAlias = $this->getCategoryFlatTable(true,null,true);
		$joinTable =  $this->getCategoryFlatTable();

		$condition =  "{$joinTableAlias}.entity_id = ".self::MAIN_TABLE_SUBST.".parent_id";
	    $this->joinTable($mainTableAlias,$joinTableAlias,$joinTable,false,$condition);

	    return $this;
	}
	
	public function joinCategoryUrlRewrite()
	{
		$joinTableAlias = self::CCUR;
		$joinTable =  $this->getTable('core/url_rewrite');
	
		$condition =  "{$joinTableAlias}.category_id = ".self::MAIN_TABLE_SUBST.".entity_id ";
		$condition .=  "AND {$joinTableAlias}.store_id = {$this->getStoreId()} ";
		$condition .=  "AND {$joinTableAlias}.product_id IS NULL ";
		$this->joinTable($this->getCategoryFlatTable(true),$joinTableAlias,$joinTable,false,$condition);
		return $this;
	}

	protected function joinMainTable($joinTableAlias,$joinTable,$joinIfColumnsEmpty = false,$condition)
	{
	    $mainTableAlias = $this->getMainTable(true);
	    return $this->joinTable($mainTableAlias,$joinTableAlias,$joinTable,$joinIfColumnsEmpty,$condition);
	}

	protected function joinTable($mainTableAlias,$joinTableAlias,$joinTable,$joinIfColumnsEmpty = false,$condition)
	{
	    $selectColumns =  $this->getColumns($joinTableAlias);
	    if(empty($selectColumns) && !$joinIfColumnsEmpty)
	        return false;

	    $select = $this->getSelect();

		$condition = str_replace(self::MAIN_TABLE_SUBST,$mainTableAlias,$condition);

		$select->joinLeft(
                    array($joinTableAlias => $joinTable),
                    $condition,
                    $selectColumns
                    );

		return true;
	}

	//*************************************************************/

	protected function setWebsiteId($websiteId)
	{
		$this->_websiteId = $websiteId;
	}
	
	protected function resetWebsiteId()
	{
		$this->_websiteId = Mage::app()->getStore($this->getStoreId())->getWebsiteId();
	}

    protected function getWebsiteId()
    {
        if (is_null($this->_websiteId)) {
            $this->_websiteId = Mage::app()->getStore($this->getStoreId())->getWebsiteId();
        }
        return $this->_websiteId;
    }

    protected function getProductFlatTable($alias = false,$storeId = null)
    {
        if (is_null($storeId))
        {
            $storeId = $this->getStoreId();
        }

        if($alias)
            return self::CPF.$storeId;
        else
            return  Mage::getResourceModel('catalog/product_flat')->getFlatTableName($storeId);
    }

    protected function getCategoryFlatTable($alias = false,$storeId = null,$parent = false)
    {
        if (is_null($storeId))
        {
            $storeId = $this->getStoreId();
        }

        if($alias)
        {
        	if($parent)
        		return self::PCCF.$storeId;
        	else
        		return self::CCF.$storeId;
        }
        else
            return  Mage::getResourceModel('catalog/category_flat')->getMainStoreTable($storeId);
    }

	public function resetOffset()
    {
    	$this->_offset = 0;
    }

    public function setOffset($offset)
    {
    	$this->_offset = $offset;
    }

    public function getOffset()
    {
    	return $this->_offset;
    }

    public function setAttributes($attributes)
    {
        if(is_array($attributes) && isset($this->_defaultAttributes) && !empty($this->_defaultAttributes))
        {
            $attributes = array_merge($attributes,$this->_defaultAttributes);
        }
        $this->_atttibutes = $attributes;
        $this->addSuperAttributes();
    }

    protected function addSuperAttributes()
    {
    	if(!$this->allowSuperAttributesExport())
    		return;
    	
        if(!in_array("super_attributes",$this->_atttibutes))
            return;

        $superAttributes = Mage::getResourceModel('nscexport/cache_superattributes')->getAllSuperAttributes();
        $this->_atttibutes = array_merge($superAttributes,$this->_atttibutes);
    }

    public function getAttributes()
    {
        return $this->_atttibutes;
    }

    protected function getUnknownAttributes($attributes = null)
    {
    	if(!isset($attributes))
    		$attributes = $this->_atttibutes;
        $unknownAttributes = array();
        $allColumns = $this->getAllColumns();
        if(is_array($attributes))
	        foreach ($attributes as $attribName)
	        {
	        	if(!array_key_exists($attribName,$allColumns))
	        	    $unknownAttributes[] = $attribName;
	        }
	    foreach($unknownAttributes as $key => $value)
	    {
	    	if(in_array($value, $this->_staticColumns))
	    		unset($unknownAttributes[$key]);
	    }
        return $unknownAttributes;
    }

    public function getCondition($name,$default = null)
    {
    	$conditions = $this->getConditions();
        if(array_key_exists($name,$conditions))
            return $conditions[$name];
        else
            return $default;
    }

    /**
     * Set store scope
     *
     * @param int|string|Mage_Core_Model_Store $store
     * @return Mage_Catalog_Model_Resource_Collection_Abstract
     */
    public function setStore($store)
    {
        $this->setStoreId(Mage::app()->getStore($store)->getId());
        return $this;
    }

    /**
     * Set store scope
     *
     * @param int|string|Mage_Core_Model_Store $storeId
     * @return Mage_Catalog_Model_Resource_Collection_Abstract
     */
    public function setStoreId($storeId)
    {
        if ($storeId instanceof Mage_Core_Model_Store) {
            $storeId = $storeId->getId();
        }
        $this->_storeId = (int)$storeId;
        $this->resetWebsiteId();
        return $this;
    }

    /**
     * Return current store id
     *
     * @return int
     */
    public function getStoreId()
    {
        if (is_null($this->_storeId)) {
            $this->setStoreId(Mage::app()->getStore()->getId());
        }
        return $this->_storeId;
    }

    protected function getStore()
    {
    	return Mage::app()->getStore($this->getStoreId());
    }

    protected function getEmptySelect()
    {
    	$select = $this->_getReadAdapter()->select();
    	$res = clone $select;
    	$res->reset();
    	return $res;
    }

    protected function getSubSelectTable($select)
    {
    	return new Zend_Db_Expr( "(" . $select . ") ");
    }
    
    public function getSelect()
    {
    	if(!isset($this->_select))
    	{
    		$this->_select = $this->getEmptySelect();
    	}
    	return $this->_select;
    }

    protected function helper()
    {
        if(!isset($this->_helper))
    	    $this->_helper = Mage::helper('nscexport/data_loader');
    	return $this->_helper;
    }

    protected function getParam($index,$default = null)
    {
    	if(!isset($this->_params))
    		$this->_params = $this->helper()->getCommonParameters();
    	if(array_key_exists($index,$this->_params))
    		return $this->_params[$index];
    	else
    		return $default;
    }

    protected function allowInactiveCategoriesExport()
    {
    	return $this->getParam(Nostress_Nscexport_Helper_Data_Loader::PARAM_ALLOW_INACTIVE_CATEGORIES_EXPORT,false);    	
    }
    
//     protected function allowChildProductsExport()
//     {
//     	return $this->getParam(Nostress_Nscexport_Helper_Data_Loader::PARAM_ALLOW_CHILD_PRODUCTS_EXPORT,false);  
//     }
    
    protected function allowPlaceholderImageExport()
    {
    	return $this->getParam(Nostress_Nscexport_Helper_Data_Loader::PARAM_ALLOW_PLACEHOLDER_IMAGES_EXPORT,true);    	
    }
    
    protected function allowSuperAttributesExport()
    {
    	return $this->getParam(Nostress_Nscexport_Helper_Data_Loader::PARAM_ALLOW_SUPER_ATTRIBUTES_EXPORT,true);
    }
    
    protected function getImageFolder()
    {
    	return $this->getParam(Nostress_Nscexport_Helper_Data_Loader::PARAM_IMAGE_FOLDER,self::DEF_IMAGE_FOLDER);
    }

    protected function getReviewUrlPrefix()
    {
    	return $this->getParam(Nostress_Nscexport_Helper_Data_Loader::PARAM_REVIEW_URL,self::DEF_REVIEW_URL);
    }
    
    protected function getUrlAttributeSource()
    {
    	return $this->getParam(Nostress_Nscexport_Helper_Data_Loader::PARAM_URL_ATTRIBUTE_SOURCE,'0');
    }

    protected function getStoreBaseUrl($type = null)
    {
    	if(isset($type))
    		return $this->getStore()->getBaseUrl($type);
    	else
        	return $this->getStore()->getBaseUrl();
    }

    protected function getBaseUrl($type)
    {
    	$defStore = Mage::app()->getStore(Mage_Catalog_Model_Abstract::DEFAULT_STORE_ID);
    	if(isset($type))
    		return $defStore->getBaseUrl($type);
    	else
        	return $defStore->getBaseUrl();
    }

    protected function checkProductAttributes($attributes)
    {
        $flatColumns = $this->helper()->getProductFlatColumns($this->getStoreId());
		return $this->checkAttributes($attributes,$flatColumns);
    }

    protected function checkCategoryAttributes($attributes)
    {
		$flatColumns = $this->helper()->getCategoryFlatColumns($this->getStoreId());
		return $this->checkAttributes($attributes,$flatColumns,'category');
    }

    protected function checkAttributes($attributes,$flatColumns,$flatCatalogType = 'product')
    {
        $missingAttributes = array();
        
        if($flatCatalogType == 'product')
        {
        	foreach($attributes as $attribute)
        	{
        		$attributeCode = $attribute;
        		//If attribute contains duplicate prefix than remove this prexif and check if product is in flat catalog
        		if(strpos($attribute,Nostress_Nscexport_Helper_Data_Feed::DUPLICATE_FLAT_CAT_COLUMN_PREFIX)!== false)
        		{
        			$attributeCode =  str_replace(Nostress_Nscexport_Helper_Data_Feed::DUPLICATE_FLAT_CAT_COLUMN_PREFIX, "", $attribute);
        		}
        			
        		if(!in_array($attributeCode,$flatColumns))
        			$missingAttributes[] = $attributeCode;
        	}
        }
        else 
        {
        	foreach($attributes as $attribute)
        	{
           		if(!in_array($attribute,$flatColumns))
                	$missingAttributes[] = $attribute;
        	}
        }

        if(count($missingAttributes) > 0)
        {
        	$missingAttributes = array_unique($missingAttributes);
            $attributes = implode(", ",$missingAttributes);
            if($flatCatalogType == "category")
            	$errorCode = "6";
            else
            	$errorCode = "7";
            throw new Exception($errorCode." ".$attributes);

            return false;
        }

        return true;
    }

    protected function prepareColumnsFromAttributes($attributes)
    {
        $columns = array();
        $allAttributesInfo = $this->helper()->getVisibleProductAttributes(true);
        foreach ($attributes as $attributeCode)
        {        	
        	//If attribute code contains prefix due to attriute code duplicity, than adjust $productFlatAttributeDuplicatesCode
        	$productFlatAttributeDuplicatesCode = false;
        	if(strpos($attributeCode,Nostress_Nscexport_Helper_Data_Feed::DUPLICATE_FLAT_CAT_COLUMN_PREFIX)!== false)
        		$productFlatAttributeDuplicatesCode = true;
        	
            if(!array_key_exists($attributeCode,$allAttributesInfo) && !$productFlatAttributeDuplicatesCode)
            {
                throw new Exception("8 ".$attributeCode);
            }
            
            $value = $attributeCode;
            //Get original attribute code
            if($productFlatAttributeDuplicatesCode)
            	$value =  str_replace(Nostress_Nscexport_Helper_Data_Feed::DUPLICATE_FLAT_CAT_COLUMN_PREFIX, "", $attributeCode);

            $info = $allAttributesInfo[$value];
            
            switch($info->getFrontendInput())
            {
                case 'media_image':
                    $value = $this->mediaImageColumn($value);
                    break;
                //case 'multiselect':
                case 'select':
                    $value = $this->multiSelectColumn($value);
                    break;
                case 'date':
                	$value = $this->dateColumn("{$this->getProductFlatTable(true)}.{$value}");
                	break;
                case 'price':
                	$value = $this->priceColumn($value);
                default:
                    break;
            }

            $columns[$attributeCode] = $value;
        }
        return $columns;
    }

	protected function dateColumn($columnCode,$formatType = self::DATE_TIME)
    {
    	$format = $this->getDatetimeFormat();
    	if($format == Nostress_Nscexport_Model_Config_Source_Datetimeformat::TIMESTAMP)        
    		return new Zend_Db_Expr("UNIX_TIMESTAMP($columnCode)");
    	else
    		return new Zend_Db_Expr("DATE_FORMAT($columnCode,'{$this->getSqlTimestampFormat($formatType)}')");
    }

    protected function mediaImageColumn($code)
    {    	
    	$imageUrlPrefix = $this->getStoreBaseUrl(Mage_Core_Model_Store::URL_TYPE_MEDIA).$this->getImageFolder();
    	
    	$placeholderUrl = "";    	
    	if($this->allowPlaceholderImageExport())
    	{
	    	$placeholder = $this->getPlaceholder($code);
	        if(!empty($placeholder))
	        {
	        	$placeholderUrl = $imageUrlPrefix.$placeholder;
	        }
    	}
    	
    	return "IF( {$this->getProductFlatTable(true)}.{$code} = 'no_selection','{$placeholderUrl}',CONCAT('{$imageUrlPrefix}', {$this->getProductFlatTable(true)}.{$code}))";   	                
    }

    protected function multiSelectColumn($code)
    {
        return $code.self::VALUE_COLUMN_SUFFIX;
    }

    protected function priceColumn($code)
    {
        return $this->helper()->getRoundSql($this->getProductFlatTable(true).".".$code);
    }

	protected function getTaxonomyColumnName()
	{
	    $taxonomyCode = $this->getTaxonomyCode();
	    if(!isset($taxonomyCode))
            return null;
        return $this->helper()->createCategoryAttributeCode($taxonomyCode);
	}

	protected function getTaxonomyLocale()
	{
	    $taxonomyCode = $this->getTaxonomyCode();
	    if(!isset($taxonomyCode))
            return null;

	    $locale = $this->helper()->getStoreLocale($this->getStore());	    
	    $locale = $this->helper()->getStoreTaxonomyLocale($taxonomyCode,$locale);
	    return $locale;
	}

	protected function getSqlTimestampFormat($type = self::DATE_TIME)
	{
		return $this->datetimeModel()->getSqlFormat($this->getDatetimeFormat(),$type);
	}

	protected function datetimeModel()
	{
		return  Mage::getModel("nscexport/config_source_datetimeformat");
	}

	/********************************BASIC GETTERS********************************************/

	public function setData($data)
	{
		$this->_data = $data;
	}

	public function getData($index)
	{
		if(isset($this->_data[$index]))
			return $this->_data[$index];
		else
		{
			//$this->helper()->log("Loader parameter {$index} not found.");
			return false;
		}
	}

    public function getExportId()
    {
        return $this->getData(self::EXPORT_ID);
    }

    public function  getTaxonomyCode()
    {
        return $this->getData(self::TAXONOMY_CODE);
    }

    public function getBatchSize()
    {
        $value = $this->getData(self::BATCH_SIZE);
        if(!$value)
        	$value = self::DEF_BATCH_SIZE;
        return $value;
    }
    
    public function getCurrency()
    {
    	return $this->getData(self::CURRENCY);    	
    }

    public function getStockStatusDependence()
    {
    	return $this->getData(self::STOCK_STATUS_DEPENDENCE);
    }

    /**
     * Returns product type, category conditions.
     */
    public function getConditions()
    {
        $value = $this->getData(self::CONDITIONS);
        if(!$value)
        	$value = array();
        return $value;
    }

    public function getCategoryPathDelimiter()
    {
    	$value = $this->getData(self::CATEGORY_PATH_DELIMITER);
    	if(!$value)
    		$value = self::DEF_CATEGORY_PATH_DELIMITER;
    	return $value;
    }

    public function getDatetimeFormat()
    {
        return $this->getData(self::DATETIME_FORMAT);
    }

    public function getCategoryLowestLevel()
    {
        return $this->getData(self::CATEGORY_LOWEST_LEVEL);
    }

    protected function dispatchDefineColumnsEvent($eventSuffix = "")
    {
        $columnsContainer = new Varien_Object();
        $columnsContainer->setColumns($this->_columns);
        Mage::dispatchEvent('nscexport_define_columns'.$eventSuffix, array('columns_container'=> $columnsContainer));
        $this->_columns = $columnsContainer->getColumns();
    }

    protected function dispatchJoinTableEvent($mainTable,$eventSuffix = "")
    {
    	$container = new Varien_Object();
    	$container->setStoreId($this->getStoreId());
    	$container->setSelect($this->getSelect());
    	$container->setMainTable($mainTable);
    	$container->setAttributes($this->getAttributes());
    	Mage::dispatchEvent('nscexport_join_table'.$eventSuffix, array('container'=> $container));
    }


    protected function isWeeeEnabled()
    {
    	return Mage::helper('weee')->isEnabled($this->getStore());
    }

    protected function isWeeeTaxable()
    {
    	return Mage::helper('weee')->isTaxable($this->getStore());
    }

    protected function isWeeeDiscounted()
    {
    	return Mage::helper('weee')->isDiscounted($this->getStore());
    }

    protected function getWeeeListPriceDisplayType()
    {
    	return Mage::helper('weee')->getListPriceDisplayType($this->getStore());
    }

    protected function getWeeeAttributes()
    {
        $select = $this->_getReadAdapter()->select()
            ->from(array('ca' => $this->getTable('catalog/eav_attribute')), array('attribute_id'))
            ->join(
                array('ea' => $this->getTable('eav/attribute')),
                'ca.attribute_id = ea.attribute_id',
                array('attribute_code'));

		$select->where('ea.frontend_input = ?', 'weee');
        $collection = $this->_getReadAdapter()->fetchAll($select);
        $items = array();
        foreach ($collection as $item)
        {
        	if(isset($item['attribute_id']) && isset($item['attribute_code']))
        		$items[$item['attribute_id']] = $item['attribute_code'];
        }
        return $items;
    }

    protected function getWeeeColumns()
    {
    	$columns = array("weee_".self::WEEE_COLUMN_TOTAL_ORIGINAL => self::WEEE_COLUMN_TOTAL_ORIGINAL);
    	$columns["weee_".self::WEEE_COLUMN_TOTAL_FINAL] = self::WEEE_COLUMN_TOTAL_FINAL;
    	$attributes = $this->convertWeeeAttributesToColumnNames();
    	foreach ($attributes as $code)
    	{
    		$columns[$code] = $this->helper()->getRoundSql($code);
    	}
    	return $columns;
    }

    protected function convertWeeeAttributesToColumnNames($weeeAttributes = null)
    {
    	if(empty($weeeAttributes))
        	$weeeAttributes = array_values($this->getWeeeAttributes());
    	$attributes = array();
    	foreach($weeeAttributes as $code)
    	{
    		$attributes[] = $code.self::WEEE_COLUMN_SUFFIX_FINAL;
    		$attributes[] = $code.self::WEEE_COLUMN_SUFFIX_ORIGINAL;
    	}
    	return $attributes;
    }
    
    public function getStoreConfig($configPath)
    {
    	return $this->getStore()->getConfig($configPath);
    }
    
    protected function getPlaceholder($code)
    {
    	if(empty($this->_placeholderss[$code]))
    	{	
    		$this->_placeholderss[$code] =  $this->getStoreConfig("catalog/placeholder/{$code}_placeholder");   
    		if(!empty($this->_placeholderss[$code]))
    			$this->_placeholderss[$code] = "/placeholder/".$this->_placeholderss[$code];
    	} 
    	return $this->_placeholderss[$code];
    }
}
?>
