<?php 
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Resource  of data loader for export process
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Data_Loader_Category extends Nostress_Nscexport_Model_Mysql4_Data_Loader
{    
	const ZERO_LEVEL = 0;
	
    /**
     * Initialize collection select
     * Redeclared for remove entity_type_id condition
     * in catalog_product_entity we store just products
     *
     * @return Mage_Catalog_Model_Resource_Eav_Mysql4_Product_Collection
     */
    public function init()
    {     
		parent::init();        
		$select = $this->getSelect();
		$mainTableAlias = $this->getMainTable(true);
        $select->from(array($mainTableAlias => $this->getMainTable()),$this->getColumns($mainTableAlias));
       	$select->distinct();
       	if(!$this->allowInactiveCategoriesExport())
       		$select->where($mainTableAlias.'.is_active=?', self::CATEGORY_ACTIVE);
       	$select->where($mainTableAlias.'.level>=?', $this->getCategoryLowestLevel());
        return $this;
    }
    
    public function getMainTable($alias = false)
    {
        return $this->getCategoryFlatTable($alias);
    }	
    
	protected function defineColumns()
    {
    	$defaultCatPathDelim = self::DEF_CATEGORY_PATH_DELIMITER;
    	
        parent::defineColumns();              
        $this->_columns[$this->getCategoryFlatTable(true)] = array( "id" => "entity_id",
                                                            		"name" => "name",
                                                                  	"path_ids" => "(SUBSTRING_INDEX({$this->getCategoryFlatTable(true)}.path,'{$defaultCatPathDelim}',-{$this->getCategoryFlatTable(true)}.level+{$this->getCategoryLowestLevel(true)}))",
                                                                 	"level" => "({$this->getCategoryFlatTable(true)}.level - {$this->getCategoryLowestLevel()})",
                                                                 	"parent_id" => "parent_id",
        															"url_key" => "url_key",        															
        															"path_url_key" => "(REPLACE(REPLACE(IFNULL({$this->getCategoryFlatTable(true)}.url_path,''), '.html',''),'".self::DEF_CATEGORY_PATH_DELIMITER."','-'))",
        															"meta_description" => "meta_description",
        															"meta_title" => "meta_title",
        															"meta_keywords" => "meta_keywords",
        															"description" => "description"
        );        															
        
        $this->_columns[self::NCCP] = array("path" => "category_path",       										           
       										"root_name" => "category_root_name");
        
        if($this->getUrlAttributeSource() == Nostress_Nscexport_Model_Config_Source_Urlsource::FLAT_CATALOG)
        {
        	$categoryUrlSuffix = $this->getStoreConfig(Mage_Catalog_Helper_Category::XML_PATH_CATEGORY_URL_SUFFIX);
        	if(!empty($categoryUrlSuffix))
        	{
        		$firstChar = substr($categoryUrlSuffix, 0,1);
        		if($firstChar != ".")
        			$categoryUrlSuffix = ".".$categoryUrlSuffix;
        	}
        	
        	$this->_columns[$this->getCategoryFlatTable(true)]["url"] = "(CONCAT('{$this->getStoreBaseUrl()}',IFNULL({$this->getCategoryFlatTable(true)}.url_path,CONCAT({$this->getCategoryFlatTable(true)}.url_key,'{$categoryUrlSuffix}'))))";
        	//category url is loaded from category flat table
        	$this->_columns[self::CCUR] = array();
        }
        else
        {
        	$this->_columns[self::CCUR] = array("category_url" => "(CONCAT('{$this->getStoreBaseUrl()}',".self::CCUR.".request_path))");
        }
        
		$userCatPathDelim = $this->getCategoryPathDelimiter();
		if($userCatPathDelim !== $defaultCatPathDelim)
        	$this->_columns[self::NCCP]["path"] = "REPLACE(".self::NCCP.".category_path,'{$defaultCatPathDelim}','{$userCatPathDelim}')";       
       
        $this->_columns[$this->getCategoryFlatTable(true,null,true)] = array("parent_name" => "name");
        $this->dispatchDefineColumnsEvent("_category");
    }
    //*************************************** BASE PART ******************************************   

     /**
     * Joint product filter
     */
	public function joinProductFilter()
	{
	    $this->joinExportCategoryProduct();
	}
    /**
     * Joint export categoryproduct table
     */
	protected function joinExportCategoryProduct()
	{
		$select = $this->getSelect();		
		$mainTableAlias = $this->getMainTable(true);
		
		$joinTableAlias = self::NCP;
		$joinTable = $this->getTable('nscexport/categoryproducts');
		$select->join(
                    array($joinTableAlias => $joinTable),
                    $joinTableAlias.'.category_id='.$mainTableAlias.'.entity_id ',
                    $this->getColumns($joinTableAlias));	
        $select->where($joinTableAlias.'.export_id=?', $this->getExportId());                   
	}

	/**
	 * Order by category level
	 */
	public function orderByLevel()
	{
		$select = $this->getSelect();
		$select->order($this->getMainTable(true).".level");
	}
	
	public function addAttributeFilter() 
	{
		$flatAlias = $this->getCategoryFlatTable(true);
	
		// copy columns
		$columns = $this->_columns;
	
		$catTableAliases = array(self::NCCP,$flatAlias);
		foreach ($catTableAliases as $tableAlias)
		{	
			foreach ($columns[$tableAlias] as $alias => $flatColumn)
			{
				$columns[$tableAlias]["category_".$alias] = $flatColumn;
				unset($columns[$tableAlias][$alias]);
			}
		}
		 
		// create condition
		$conditions = $this->getConditions();
		if( isset( $conditions[self::CONDITIONS][self::CONDITIONS]))
		{	
			$registredKey = Mage::registry('nscexport_store');
			if(!empty($registredKey))
				Mage::unregister('nscexport_store');
	
			Mage::register('nscexport_store', $this->getStoreId());
			$rule = Mage::getModel( 'nscexport/rule')->initConditions( $conditions[self::CONDITIONS]);
			$where = $rule->getConditions()->asSqlWhere( $columns);
				
			// add to select
			if( $where !== false && $where != '()') {
				$this->getSelect()->where( $where);
			}
		}
	
		return $this;
	}	
	
	//**********************************COMMON PART*****************************************************
	

    protected function getColumns($tableAlias,$defualt = null,$groupConcat = false,$addTablePrefix = false)
    {
        if(array_key_exists($tableAlias,$this->_columns))
            $result = $this->_columns[$tableAlias];
        else
            $result = $defualt;
            
       if($groupConcat)
           	$result = $this->groupConcatColumns($result);
       if($addTablePrefix)
			$result = $this->addTablePrefix($tableAlias,$result);
       return $result;
    }	
    
    public function getCategoryLowestLevel($modify = false)
    {
    	$level = parent::getCategoryLowestLevel();
    	if(empty($level))
    		$level = self::ZERO_LEVEL;
    	
    	if(!$modify)
    	{
    		return $level;
    	}    		
    	else 
    	{    		
    		$level--;
    		if($level < 0)
    			$level = 0;
    		return $level;
    	}
    	
    }
}
?>