<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
* Resource  of data loader for export process
* @category Nostress
* @package Nostress_Nscexport
*
*/

class Nostress_Nscexport_Model_Mysql4_Data_Loader_Product extends Nostress_Nscexport_Model_Mysql4_Data_Loader
{
 	protected $_lastCategoryProductTableAlias = self::NCP;
	protected $_defaultAttributes = array("id","product_type","group_id","is_child");
 	protected $_productFlatMandatoryColumns = array('visibility','tax_class_id');
 	protected $_multiColumns = array("super_attributes","categories","media_gallery","reviews");
 	protected $_staticColumns = array("currency","country_code","locale","language","shipping_method_name","shipping_cost");

 	const VISIBILITY = 4;
	const STOCK_STATUS = 1;

    /**
     * Initialize collection select
     * Redeclared for remove entity_type_id condition
     * in catalog_product_entity we store just products
     *
     * @return Mage_Catalog_Model_Resource_Eav_Mysql4_Product_Collection
     */
    public function init()
    {
    	parent::init();
		$select = $this->getSelect();
        $select->from(array($this->getMainTable(true) => $this->getMainTable()),$this->getProductFlatColumns());
        return $this;
    }

    public function getMainTable($alias = false)
    {
        return $this->getProductFlatTable($alias);
    }

    public function getAllColumns()
    {
    	$columns = parent::getAllColumns();
    	foreach ($this->_productFlatMandatoryColumns as $value)
    	{
    		$columns[$value] = $value;
    	}
    	return $columns;
    }

    protected function getProductFlatColumns()
    {
        $columns = $this->getColumns($this->getMainTable(true));
        $unknownAttributes = $this->getUnknownAttributes();
        $this->checkProductAttributes(array_merge($unknownAttributes,$this->_productFlatMandatoryColumns));
        return array_merge($columns,$this->prepareColumnsFromAttributes($unknownAttributes));
    }

    protected function defineColumns()
    {
        parent::defineColumns();

        $this->_columns[$this->getProductFlatTable(true)] =
        array(  "id" => "entity_id",
                "product_type" => "type_id",                
         		"name_and_sku" => new Zend_Db_Expr("CONCAT({$this->getProductFlatTable(true)}.name,' ',{$this->getProductFlatTable(true)}.sku)")
        );
        
        if($this->getUrlAttributeSource() == Nostress_Nscexport_Model_Config_Source_Urlsource::FLAT_CATALOG)
        {
        	$productUrlSuffix = $this->getStoreConfig(Mage_Catalog_Helper_Product::XML_PATH_PRODUCT_URL_SUFFIX);
        	if(!empty($productUrlSuffix))
        	{
        		$firstChar = substr($productUrlSuffix, 0,1);
        		if($firstChar != ".")
        			$productUrlSuffix = ".".$productUrlSuffix;
        	}
        	$this->_columns[$this->getProductFlatTable(true)]["url"] = new Zend_Db_Expr("CONCAT('{$this->getStoreBaseUrl()}',IFNULL({$this->getProductFlatTable(true)}.url_path,CONCAT({$this->getProductFlatTable(true)}.url_key,'{$productUrlSuffix}')))");
        }
        else
        {
        	//product url is loaded from catalog product table
        	$this->_columns[self::CUR] = array(	"url" => new Zend_Db_Expr("CONCAT('{$this->getStoreBaseUrl()}',".self::CUR.".request_path)"));
        	$this->_columns[self::EUR] = array(	"url" => new Zend_Db_Expr("CONCAT('{$this->getStoreBaseUrl()}',".self::EUR.".request_path)"));
        }
        
        $this->_columns[self::NPACD] = array(  Nostress_Nscexport_Helper_Data_Loader::COLUMN_CATEGORIES_DATA => Nostress_Nscexport_Helper_Data_Loader::GROUPED_COLUMN_ALIAS  );

        $this->_columns[self::NCS] = array(  "stock_status" => new Zend_Db_Expr("IF(".self::NCS.".stock_status = 1 OR ".self::NCS.".always_in_stock = 1, 1, 0)"),
        									  "qty" => "qty");
        
        $this->_columns[self::PNCS] = array(  "parent_stock_status" => new Zend_Db_Expr("IF(".self::PNCS.".stock_status = 1 OR ".self::PNCS.".always_in_stock = 1, 1, 0)"),
        									  "parent_qty" => "qty");

        if($this->getStockStatusDependence() == Nostress_Nscexport_Model_Config_Source_Stockdependence::QTY)
        {        	
        	$this->_columns[self::NCS]["stock_status"] = new Zend_Db_Expr("IF(".self::NCS.".qty > 0 OR ".self::NCS.".always_in_stock = 1, 1, 0)");
        	$this->_columns[self::PNCS]["parent_stock_status"] = new Zend_Db_Expr("IF(".self::PNCS.".qty > 0 OR ".self::PNCS.".always_in_stock = 1, 1, 0)");
        }

        $this->_columns[self::CISI] = array( "minimum_qty_allowed_in_shopping_cart" => $this->helper()->getRoundSql($this->getMinSaleQtyCondition(self::CISI),0));
        $this->_columns[self::PCPF] = array("parent_sku" => "sku");

        $this->_columns[self::NCSA] = array("super_attributes" => "super_attributes");
        
        $this->_columns[self::NCC] = array("categories" => "categories");
        
        $this->_columns[self::NCPC] = array("categories" => "categories");
        
        $this->_columns[self::NCR] = array("reviews" => "reviews");
        
        $this->_columns[self::NCMG] = array("media_gallery" => "media_gallery");
        $this->_columns[self::RRA] = array("reviews_count" => "reviews_count",
                                           "reviews_url" => new Zend_Db_Expr("CONCAT('{$this->getStoreBaseUrl()}','{$this->getReviewUrlPrefix()}', CAST({$this->getProductFlatTable(true)}.entity_id AS CHAR))"));

        $this->_columns[self::NCT] = array("tax_percent" => new Zend_Db_Expr("IFNULL({$this->helper()->getRoundSql("tax_percent*100")},0)"),
        								    "tax_class_id" => "tax_class_id");
         
        $this->_columns[self::TC] = array("tax_class_name" => "class_name");         
         
        $this->_columns[self::NEC] = array(	"taxonomy_name" => "name",
                                            "taxonomy_id" => "id",
                                            "taxonomy_path" => "path",
                                            "taxonomy_ids_path" => "ids_path",
                                            "taxonomy_level" => "level",
                                            "taxonomy_parent_name" => "parent_name",
                                            "taxonomy_parent_id" => "parent_id",
							        		"taxonomy_code1" => "code1",
							        		"taxonomy_code2" => "code2"        		
                                        	);
        
        $this->_columns[self::NCEC] = array(	"taxonomy_name" => "name",
								        		"taxonomy_id" => "id",
								        		"taxonomy_path" => "path",
								        		"taxonomy_ids_path" => "ids_path",
								        		"taxonomy_level" => "level",
								        		"taxonomy_parent_name" => "parent_name",
								        		"taxonomy_parent_id" => "parent_id",
								        		"taxonomy_code1" => "code1",
								        		"taxonomy_code2" => "code2"
								        );        
        
        $this->_columns[self::CPE] = array(	"update_datetime" => $this->dateColumn(self::CPE.".updated_at"),
									        "update_date" => $this->dateColumn(self::CPE.".updated_at",self::DATE),
        									"update_time" => $this->dateColumn(self::CPE.".updated_at",self::TIME),
									        "creation_datetime" =>  $this->dateColumn(self::CPE.".created_at"),
									        "creation_date" => $this->dateColumn(self::CPE.".created_at",self::DATE),
									        "creation_time" => $this->dateColumn(self::CPE.".created_at",self::TIME)
        							        );        

		$this->_columns[self::NCW] = $this->getWeeeColumns();

		$this->_columns[self::CPR] = array(	"parent_id" => "parent_id",
                                            "group_id" => new Zend_Db_Expr("IFNULL(".self::CPR.".parent_id,{$this->getMainTable(true)}.entity_id)"),
                                            "is_child" => new Zend_Db_Expr("(".self::CPR.".parent_id IS NOT NULL)"),
        									"is_parent" => new Zend_Db_Expr("(".self::CPR.".parent_id IS NULL)"));

        $this->definePriceColumns();
        $this->dispatchDefineColumnsEvent("_product");
    }

    //*************************************** BASE PART ******************************************
     /**
     * Joint product filter
     */
	public function joinProductFilter()
	{
	    $this->joinProfileCategoryCache();
	}
	
	/**
	 * Joint filtred products with cached categories
	 */
	protected function joinProfileCategoryCache()
	{
		$select = $this->getSelect();
		$mainTableAlias = $this->getMainTable(true);
		
		//if($this->allowChildProductsExport())
			
		$joinTableAlias = self::NCPC;
		$joinTable = $this->getTable('nscexport/cache_profilecategory');
		
		$select->join(
				array($joinTableAlias => $joinTable),
				"{$joinTableAlias}.product_id={$mainTableAlias}.entity_id AND {$joinTableAlias}.export_id = {$this->getExportId()}",
				$this->getColumns($joinTableAlias));
		$this->_lastCategoryProductTableAlias = $joinTableAlias;
	}	
	
	/**
     * Joint export categoryproduct table
     */
	public function joinCategoryProduct()
	{
		$select = $this->getSelect();
		$mainTableAlias = $this->getMainTable(true);

		$joinTableAlias = self::CCPI;
		$joinTable = $this->getTable('catalog/category_product_index');
		$select->join(
                    array($joinTableAlias => $joinTable),
                    "{$joinTableAlias}.product_id={$mainTableAlias}.entity_id AND {$joinTableAlias}.store_id= {$this->getStoreId()}",
                    $this->getColumns($joinTableAlias));        

        $this->_lastCategoryProductTableAlias = $joinTableAlias;
	}
	
	/**
	 * Joint export product category max level table
	 */
	public function joinProductCategoryMaxLevel()
	{
		$select = $this->getSelect();
		$mainTableAlias = $this->getMainTable(true);
	
		$joinTableAlias = self::NCCML;		
		$joinTable = $this->getTable('nscexport/cache_categorymaxlevel');
		$select->join(
				array($joinTableAlias => $joinTable),
				"{$joinTableAlias}.product_id={$mainTableAlias}.entity_id AND {$joinTableAlias}.store_id= {$this->getStoreId()}",
				$this->getColumns($joinTableAlias));
	
				$this->_lastCategoryProductTableAlias = $joinTableAlias;
	}

	/**
	 * Joint left category table
	 */
	public function joinCategoryFlatWithFilteredProducts()
	{
		$joinLeft = false;			
		return $this->_joinCategoryFlat($joinLeft);
	}
	
	/**
	 * Joint inner category table
	 */
	public function joinCategoryFlat()
	{
		return $this->_joinCategoryFlat(false);
	}
	
	/**
	 * Join left category table
	 */
	public function joinLeftCategoryFlat()
	{
		return $this->_joinCategoryFlat(true);
	}
	
    /**
     * Joint category table
     */
	protected function _joinCategoryFlat($joinLeft = false)
	{
		$select = $this->getSelect();
		$mainTableAlias = $this->_lastCategoryProductTableAlias;
		$joinTableAlias = $this->getCategoryFlatTable(true);
		$joinTable = $this->getCategoryFlatTable();
		$columns = $this->getColumns($joinTableAlias);

		if(!$joinLeft)
		{
			$select->join(		
                    array($joinTableAlias => $joinTable),
                    $joinTableAlias.'.entity_id='.$mainTableAlias.'.category_id',
                    $columns
                    );
		}
		else 
		{
			$select->joinLeft(
					array($joinTableAlias => $joinTable),
					$joinTableAlias.'.entity_id='.$mainTableAlias.'.category_id',
					$columns
			);
		}
		
        $this->joinCategoryPath();

		return $this;
	}

	/**
	 * Subselect is too time consumpting
	 */
	public function joinAllCategoriesData()
	{		
	    $mainTable = $this->getTable('catalog/category_product_index');
	    $mainTableAlias = self::CCPI;
	    $joinTableAlias = $this->getCategoryFlatTable(true);
		$joinTable = $this->getCategoryFlatTable();

	    $subSelect = $this->getEmptySelect();
	    $subSelect->from(array($mainTableAlias => $mainTable), array( "product_id"));
	    $subSelect->joinLeft(
                    array($joinTableAlias => $joinTable),
                    $joinTableAlias.'.entity_id='.$mainTableAlias.'.category_id',
                    $this->getColumns($joinTableAlias,null,true)
                    );
        $subSelect->where($mainTableAlias.'.store_id=?', $this->getStoreId());
	    $subSelect->group('product_id');



	    $select = $this->getSelect();
		$mainTableAlias = $this->getMainTable(true);
		$joinTableAlias = self::$this->getSubSelectTable($this->getProductCategoriesSql());
		$joinTable =  $this->getSubSelectTable($subSelect);

		$select->joinLeft(
                    array($joinTableAlias => $joinTable),
                    $joinTableAlias.'.product_id ='.$mainTableAlias.'.entity_id',
                    $this->getColumns($joinTableAlias)
                    );
		return $this;
	}

	/**
	 * Subselect is too time consumpting
	 */
	public function joinAllCategoriesCache()
	{
		$exportAllCategories = $this->getData(self::EXPORT_ALL_CATEGORIES,"1");
		if($exportAllCategories === "0")
			return $this;
		
		$select = $this->getSelect();
		$mainTableAlias = $this->getMainTable(true);
		$joinTableAlias = self::NCC;
		$joinTable =  $this->getTable('nscexport/cache_categories');

		$select->joinLeft(
                    array($joinTableAlias => $joinTable),
                    "{$joinTableAlias}.product_id ={$mainTableAlias}.entity_id AND {$joinTableAlias}.store_id = {$this->getStoreId()}",
                    $this->getColumns($joinTableAlias)
                    );
		return $this;
	}

    /**
     * Joint export categoryproduct table
     */
	public function groupByProduct()
	{
		$select = $this->getSelect();
		$select->group($this->getMainTable(true).".entity_id");
	}

	public function addSortAttribute()
	{
		$sortAttribute = $this->getData(self::SORT_ATTRIBUTE);
		if(empty($sortAttribute))
			return;
		$select = $this->getSelect();
		if($this->parentsOnly())
			$select->columns(array(self::SORT_ATTRIBUTE_ALIAS => "{$this->getMainTable(true)}.{$sortAttribute}"));
		else
			$select->columns(array(self::SORT_ATTRIBUTE_ALIAS => "IF(".self::CPR.".parent_id IS NOT NULL,".self::PCPF.".{$sortAttribute},{$this->getMainTable(true)}.{$sortAttribute})"));
	}

	/**
	 * Set sorting attributes
	 */
	public function setProductsOrder()
	{
		$select = $this->getSelect();
		$sortAttribute = $this->getData(self::SORT_ATTRIBUTE);
		if(!empty($sortAttribute))//order condition is set
		{
			//order by custom attribute						
			$select->order("ISNULL(".self::SORT_ATTRIBUTE_ALIAS.") ".$this->getData(self::SORT_ORDER,"ASC"));
			$select->order(self::SORT_ATTRIBUTE_ALIAS." ".$this->getData(self::SORT_ORDER,"ASC"));
		}

		//order by group id
		$select->order("group_id");
		//order by product type
		$select->order($this->getMainTable(true).".type_id");	
	}

	//**********************************COMMON PART*****************************************************
	public function joinProductRelation()
	{
		$joinTableAlias = self::CPR;
		$joinTable = $this->getTable("catalog/product_relation");

        $condition =  $joinTableAlias.'.child_id='.self::MAIN_TABLE_SUBST.".entity_id ";
        $this->joinMainTable($joinTableAlias,$joinTable,true,$condition);

        $this->addParentsCondition();

		return $this;
	}
	

	/**
	 * Parent products relation is filtred
	 */
	public function joinProductRelationFilteredByProfileCategoryCache()
	{
		$catalogProductRelationAlias = self::CPR;
		$catalogProductRelationParentFilteringAlias = "parent_filtering".self::CPR;
		$catalogProductRelationTable = $this->getTable("catalog/product_relation");
	
		$profileCategoryCacheParentFilteringAlias = "parent_filtering".self::NCPC;
		$profileCategoryCacheTable = $this->getTable('nscexport/cache_profilecategory');
	
		
		$subSelect = $this->getEmptySelect();
		$subSelect->from(array($catalogProductRelationParentFilteringAlias => $catalogProductRelationTable), "*");
		$subSelect->join(
				array($profileCategoryCacheParentFilteringAlias => $profileCategoryCacheTable),
				"{$catalogProductRelationParentFilteringAlias}.parent_id={$profileCategoryCacheParentFilteringAlias}.product_id AND {$profileCategoryCacheParentFilteringAlias}.export_id = {$this->getExportId()}",
				null);
		
		
		
		$joinTable = $this->getSubSelectTable($subSelect);
		$condition =  $catalogProductRelationAlias.'.child_id='.self::MAIN_TABLE_SUBST.".entity_id ";
		$this->joinMainTable($catalogProductRelationAlias,$joinTable,true,$condition);
		
		$this->addParentsCondition();
		
		return $this;					
	}

	public function addParentsCondition()
	{
	    if($this->parentsOnly())
	    {
	        $select = $this->getSelect();
		    $select->where(self::CPR.".parent_id IS NULL ");
	    }
	}

	public function addAttributeFilter() {

	    // copy columns
	    $columns = $this->_columns;

	    // add flat columns
	    $flatAlias = $this->getProductFlatTable(true);

	    $flatColumns = Mage::helper('nscexport/data_loader')->getProductFlatColumns( $this->getStoreId());
	    $flatColumns2 = array();
	    foreach( $flatColumns as $column) {
	        $flatColumns2[ $column] = $column;
	    }
	    $columns[ $flatAlias] = array_merge( $columns[ $flatAlias], $flatColumns2);
	    //Remove table Nostress engine category. Table Nostress cache enginecategory is used instead.
	    // Tables Nostress cache enginecategory and Nostress engine category contains same categories.
	    unset($columns[self::NEC]); 
	    
	    // create condition
	    $conditions = $this->getConditions();
	    if( isset( $conditions[self::CONDITIONS][self::CONDITIONS]))
	    {	    
	    	$res = $this->getProcessConditionsResult($conditions[self::CONDITIONS]);
	    	$allAttribtues = $res['attributes'];    		    	
	    	$conditions[self::CONDITIONS] = $res['cond'];

	    	//check if filter attributes are in flat catalog
	    	$attributesToCheck = $this->getUnknownAttributes($allAttribtues);
	    	$this->checkProductAttributes($attributesToCheck);
	    	
	    	$registredKey = Mage::registry('nscexport_store');
	    	if(!empty($registredKey))
	    		Mage::unregister('nscexport_store');

	        Mage::register('nscexport_store', $this->getStoreId());
    	    $rule = Mage::getModel( 'nscexport/rule')->initConditions( $conditions[self::CONDITIONS]);
    	    $where = $rule->getConditions()->asSqlWhere( $columns);
    	    
    	    // add to select
    	    if( $where !== false && $where != '()') {
    	        $this->getSelect()->where( $where);
    	    }
	    }

	    return $this;
	}
	
	/**
	 * Renames duplicated attriubte labels
	 * Returns attriute codes of all used attriutes
	 * @param unknown_type $condition
	 * @return multitype:Ambigous <multitype:unknown , multitype:> Ambigous <unknown, mixed>
	 */
	protected function getProcessConditionsResult($condition)
	{
		$attribtues = array();		
		foreach($condition[self::CONDITIONS] as $key => $conditionItem)
		{
			if(isset($conditionItem["conditions"]) && is_array($conditionItem["conditions"]))
			{
				$res = $this->getProcessConditionsResult($conditionItem);
				$attribtues = array_merge($attribtues,$res['attributes']);
				$attribtues = array_unique($attribtues);
				$condition[self::CONDITIONS][$key] = $res['cond'];
			}
			else
			{
				$attributeCode = $conditionItem['attribute'];
				$attribtues[] = $attributeCode;
				if(strpos($attributeCode,Nostress_Nscexport_Helper_Data_Feed::DUPLICATE_FLAT_CAT_COLUMN_PREFIX)!== false)
				{
					$attributeCode =  str_replace(Nostress_Nscexport_Helper_Data_Feed::DUPLICATE_FLAT_CAT_COLUMN_PREFIX, "", $attributeCode);
					$condition[self::CONDITIONS][$key]['attribute'] = $attributeCode;
				}
			}
		}
		
		$result = array("cond" => $condition,"attributes" => $attribtues);
		return $result;
	}

	protected function parentsOnly()
	{
	    if($this->getCondition(Nostress_Nscexport_Helper_Data_Loader::CONDITION_PARENTS_CHILDS,0) == Nostress_Nscexport_Model_Config_Source_Parentschilds::PARENTS_ONLY)
	        return true;
	    else
	        return false;
	}


	public function addTypeCondition()
	{
	    $allTypes = $this->helper()->getAppProductTypes();
	    $attribute = Mage::getModel('catalog/product_type')->getTypes();

	    $types = $this->getCondition(Nostress_Nscexport_Helper_Data_Loader::CONDITION_TYPES,$allTypes);

	    if(empty($types))
	        return $this;
	    if(!is_array($types))
	        $types = explode(",",$types);

        if(count($allTypes) != count($types) )
        {
	        $select = $this->getSelect();
	        $types = "'".implode("','",$types) ."'";
		    $select->where($this->getMainTable(true).".type_id IN ({$types})");
	    }

	    return $this;
	}

	 /**
     * Joint product table
     */
	protected function joinParentProductFlat()
	{
	    if($this->parentsOnly())
	        return $this;

		$mainTableAlias = self::CPR;
		$joinTableAlias = self::PCPF;
		$joinTable = $this->getProductFlatTable();

        $condition = $joinTableAlias.'.entity_id='.self::MAIN_TABLE_SUBST.'.parent_id';
        $this->joinTable($mainTableAlias,$joinTableAlias,$joinTable,true,$condition);
		return $this;
	}

	public function addVisibilityCondition()
	{
		$this->joinParentProductFlat();

		$select = $this->getSelect();

		$visibility = $this->getCondition( Nostress_Nscexport_Model_Profile::VISIBILITY);
		$parentVisibility = $this->getCondition( Nostress_Nscexport_Model_Profile::VISIBILITY_PARENT);

		$where = "";
		if( count($visibility)) {
		    $where .= $select->getAdapter()->quoteInto(
                "{$this->getMainTable(true)}.visibility IN(?)", $visibility);
		}
	    if(!$this->parentsOnly() && count( $parentVisibility)) {
	        if( !empty( $where)) {
	            $where .= " OR ";
	        }

	        $where .= $select->getAdapter()->quoteInto(
                self::PCPF.".visibility  IN(?)", $parentVisibility);
	    }

	    if( !empty( $where)) {
		    $select->where($where);
	    }

	}

	//////////////////////////////////STOCK////////////////////////////////////
	
	protected function getAlwaysInStockCondition($tableAlias)
	{
		return "{$tableAlias}.always_in_stock = ".self::STOCK_STATUS;		
	}
	
	protected function getMinSaleQtyCondition($tableAlias)
	{
		$globalMinSaleQty = (int) Mage::getStoreConfig(Mage_CatalogInventory_Model_Stock_Item::XML_PATH_MIN_SALE_QTY, $this->getStoreId());
		$minSaleQtyCondition = "IF({$tableAlias}.use_config_min_sale_qty = '1', '{$globalMinSaleQty}',{$tableAlias}.min_sale_qty) ";
		return $minSaleQtyCondition;
	}

	public function addStockCondition()
	{
	    if($this->getCondition(Nostress_Nscexport_Helper_Data_Loader::CONDITION_EXPORT_OUT_OF_STOCK,0) == 0)
	    {
	        $this->joinParentStock();
	        $select = $this->getSelect();

			$alwaysInStockCondition = $this->getAlwaysInStockCondition(self::NCS);
			$parentAlwaysInStockCondition = $this->getAlwaysInStockCondition(self::PNCS);


	        $condition = "";
	        switch($this->getStockStatusDependence())
	        {
	        	case Nostress_Nscexport_Model_Config_Source_Stockdependence::QTY:	//hooked on qty only
		    	    $condition = self::NCS.".qty > 0  OR {$alwaysInStockCondition}";		    	    
		    		break;
	        	case Nostress_Nscexport_Model_Config_Source_Stockdependence::STOCK: //hooked on stock_status only
		    		$condition = "(".self::NCS.".stock_status = ".self::STOCK_STATUS." OR {$alwaysInStockCondition}) ".
							"AND (".self::CPR.".parent_id IS NULL OR ".self::PNCS.".stock_status = ".self::STOCK_STATUS." OR {$parentAlwaysInStockCondition} )";		    		
		    		break;
	        	default:
	        		//hooked on qty and Is in stock attribute
	        		$condition = "((".self::NCS.".stock_status = ".self::STOCK_STATUS." AND ".self::NCS.".qty > 0) OR {$alwaysInStockCondition}) ".
	        				"AND (".self::CPR.".parent_id IS NULL OR (".self::PNCS.".stock_status = ".self::STOCK_STATUS." ) OR {$parentAlwaysInStockCondition}) "; //Parent qty is not necessary to check	        		
	        		break;
	        }
	        $select->where("({$condition})");
	    }
		return $this;
	}

	public function joinStock()
	{
		$this->joinStockCache();
		//$this->joinNormalStockStatus();
		$this->joinNormalStockItem();
	}

	public function joinParentStock()
	{
		$this->joinParentStockCache();
//  		$this->joinParentStockStatus();
// 		$this->joinParentStockItem();

	}

// 	protected function joinNormalStockStatus()
// 	{
// 		$mainTableAlias = $this->getMainTable(true);
// 		$productIdColumnName = 'entity_id';
// 		$joinTableAlias = self::CISS;
// 		$this->joinStockStatus($mainTableAlias,$joinTableAlias,$productIdColumnName);
// 	}

//  	public function joinParentStockStatus()
// 	{
// 		$mainTableAlias = self::CPR;
// 		$productIdColumnName = 'parent_id';
// 		$joinTableAlias = self::PCISS;
// 		$this->joinStockStatus($mainTableAlias,$joinTableAlias,$productIdColumnName);
// 	}

// 	protected function joinStockStatus($mainTableAlias,$joinTableAlias,$productIdColumnName)
// 	{
// 		$joinIfColumnsEmpty = $this->getCondition(Nostress_Nscexport_Helper_Data_Loader::CONDITION_EXPORT_OUT_OF_STOCK,0) == 0;
// 		$joinTable = $this->getTable("cataloginventory/stock_status");
// 		$condition = $joinTableAlias.'.product_id='.self::MAIN_TABLE_SUBST.".{$productIdColumnName} AND {$joinTableAlias}.website_id ={$this->getWebsiteId()} ";

// 		$this->joinTable($mainTableAlias,$joinTableAlias,$joinTable,$joinIfColumnsEmpty,$condition);
// 	}

	protected function joinNormalStockItem()
	{
		$mainTableAlias = $this->getMainTable(true);
		$productIdColumnName = 'entity_id';
		$joinTableAlias = self::CISI;
		$this->joinStockItem($mainTableAlias,$joinTableAlias,$productIdColumnName);
	}
	
	

// 	public function joinParentStockItem()
// 	{
// 		$mainTableAlias = self::CPR;
// 		$productIdColumnName = 'parent_id';
// 		$joinTableAlias = self::PCISI;
// 		$this->joinStockItem($mainTableAlias,$joinTableAlias,$productIdColumnName);
// 	}

	protected function joinStockItem($mainTableAlias,$joinTableAlias,$productIdColumnName)
	{
// 		$joinIfColumnsEmpty = $this->getCondition(Nostress_Nscexport_Helper_Data_Loader::CONDITION_EXPORT_OUT_OF_STOCK,0) == 0;
// 		if($this->getStockStatusDependence() == Nostress_Nscexport_Model_Config_Source_Stockdependence::QTY)
// 		    $joinIfColumnsEmpty = true;
		
		$joinIfColumnsEmpty = false;
		$joinTable = $this->getTable("cataloginventory/stock_item");
		$condition = $joinTableAlias.'.product_id='.self::MAIN_TABLE_SUBST.".{$productIdColumnName} ";

		$this->joinTable($mainTableAlias,$joinTableAlias,$joinTable,$joinIfColumnsEmpty,$condition);
	}

	protected function joinStockCache()
	{
		$mainTableAlias = $this->getMainTable(true);
		$productIdColumnName = 'entity_id';
		$joinTableAlias = self::NCS;
		$this->joinStockCacheTable($mainTableAlias,$joinTableAlias,$productIdColumnName);
	}
	
	protected function joinParentStockCache()
	{		
		$mainTableAlias = self::CPR;
		$productIdColumnName = 'parent_id';
		$joinTableAlias = self::PNCS;
		$this->joinStockCacheTable($mainTableAlias,$joinTableAlias,$productIdColumnName);
	}
	
	protected function joinStockCacheTable($mainTableAlias,$joinTableAlias,$productIdColumnName)
	{
		$joinIfColumnsEmpty = $this->getCondition(Nostress_Nscexport_Helper_Data_Loader::CONDITION_EXPORT_OUT_OF_STOCK,0) == 0;
		if($this->getStockStatusDependence() == Nostress_Nscexport_Model_Config_Source_Stockdependence::QTY)
			$joinIfColumnsEmpty = true;
		$joinTable = $this->getTable("nscexport/cache_stock");
		
		$condition = $joinTableAlias.'.product_id='.self::MAIN_TABLE_SUBST.".{$productIdColumnName} AND {$joinTableAlias}.website_id ={$this->getWebsiteId()} ";		
	
		$this->joinTable($mainTableAlias,$joinTableAlias,$joinTable,$joinIfColumnsEmpty,$condition);
	}
	
	public function joinSuperAttributesCache()
	{
	    $joinTableAlias = self::NCSA;
		$joinTable =  $this->getTable('nscexport/cache_superattributes');
		$condition = "{$joinTableAlias}.product_id =".self::MAIN_TABLE_SUBST.".entity_id AND {$joinTableAlias}.store_id = {$this->getStoreId()}";

	    $this->joinMainTable($joinTableAlias,$joinTable,false,$condition);
	    return $this;
	}

	public function joinMediaGalleryCache()
	{
		$exportAllImages = $this->getData(self::EXPORT_ALL_IMAGES,"1");
		if($exportAllImages === "0")
			return $this;
		
		$joinTableAlias = self::NCMG;
		$joinTable =  $this->getTable('nscexport/cache_mediagallery');
		$condition = "{$joinTableAlias}.product_id =".self::MAIN_TABLE_SUBST.".entity_id AND {$joinTableAlias}.store_id = {$this->getStoreId()}";

	    $this->joinMainTable($joinTableAlias,$joinTable,false,$condition);
	    return $this;
	}

	public function joinReview()
	{
		$joinTableAlias = self::RRA;
		$joinTable =  $this->getTable('review/review_aggregate');
		$condition =  "{$joinTableAlias}.entity_pk_value = ".self::MAIN_TABLE_SUBST.".entity_id AND {$joinTableAlias}.store_id = {$this->getStoreId()}";

	    $this->joinMainTable($joinTableAlias,$joinTable,false,$condition);
	    return $this;
	}
	
	public function joinReviewsCache()
	{
		$exportReviews = $this->getData(self::EXPORT_REVIEWS,"0");
		if($exportReviews === "0")
			return $this;
	
		$joinTableAlias = self::NCR;
		$joinTable =  $this->getTable('nscexport/cache_reviews');
		$condition = "{$joinTableAlias}.product_id =".self::MAIN_TABLE_SUBST.".entity_id AND {$joinTableAlias}.store_id = {$this->getStoreId()}";
	
		$this->joinMainTable($joinTableAlias,$joinTable,false,$condition);
		return $this;
	}

	public function joinTaxonomy()
	{		
		$mainTableAlias = $this->getCategoryFlatTable(true);
		$joinTableAlias = self::NCEC;
		$joinTable =  $this->getTable('nscexport/cache_enginecategory');
		$taxonomyColumn = $this->getTaxonomyColumnName();
		if(!isset($taxonomyColumn))
			return $this;
	
		$locale = $this->getTaxonomyLocale();
		$code = $this->getTaxonomyCode();
		if(!isset($locale))
			return $this;
	
		$condition =  "{$joinTableAlias}.hash = ".self::MAIN_TABLE_SUBST.".{$taxonomyColumn} AND locale = '{$locale}' AND taxonomy_code ='{$code}'";
	
		if($this->joinTable($mainTableAlias,$joinTableAlias,$joinTable,false,$condition))
			$this->checkCategoryAttributes(array($taxonomyColumn));
	
		return $this;
	}

	public function joinProductEntity()
	{
		$mainTableAlias = $this->getMainTable(true);
		$joinTableAlias = self::CPE;
		$joinTable =  $this->getTable('catalog/product');
		$condition =  "{$joinTableAlias}.entity_id = ".self::MAIN_TABLE_SUBST.".entity_id";
	    $this->joinTable($mainTableAlias,$joinTableAlias,$joinTable,false,$condition);
	    return $this;
	}

	/**
	 * Join product url rewrite table.
	 * Only if any columns are requested for load from database.
	 * @return Nostress_Nscexport_Model_Mysql4_Data_Loader_Product
	 */
	public function joinProductUrlRewrite()
	{
	    $joinTableAlias = self::CUR;
	    $joinTable =  $this->getTable('core/url_rewrite');

	    if($this->helper()->isMageEnterprise())
	    {
	        try
	        {
	            $joinTableAlias = self::EUR;
	            $joinTable =  $this->getTable('enterprise_urlrewrite/url_rewrite');
	            if(empty($joinTable))
	                $joinTable =  $this->getTable('core/url_rewrite');
	        }
	        catch(Exception $e)
	        {
	            $joinTableAlias = self::CUR;
	            $joinTable =  $this->getTable('core/url_rewrite');
	        }
	    }

		$condition =  "{$joinTableAlias}.product_id = ".self::MAIN_TABLE_SUBST.".entity_id ";
		$condition .=  "AND {$joinTableAlias}.store_id = {$this->getStoreId()} ";
		$condition .=  "AND {$joinTableAlias}.category_id IS NULL ";
	    $this->joinMainTable($joinTableAlias,$joinTable,false,$condition);
	    return $this;
	}

	/**
	 * Join fixed product tac table - only if fixed tax is enabled
	 */
	public function joinWeee()
	{
		if(!$this->isWeeeEnabled())
			return $this;

	    $joinTableAlias = self::NCW;
		$joinTable =  $this->getTable('nscexport/cache_weee');
		$condition =  "{$joinTableAlias}.product_id = ".self::MAIN_TABLE_SUBST.".entity_id AND {$joinTableAlias}.website_id = '{$this->getWebsiteId()}' ";
	    $this->joinMainTable($joinTableAlias,$joinTable,true,$condition);
	    return $this;
	}

	public function joinTaxCache()
	{
	    $joinTableAlias = self::NCT;
		$joinTable =  $this->getTable('nscexport/cache_tax');
		$condition =  "{$joinTableAlias}.tax_class_id = ".self::MAIN_TABLE_SUBST.".tax_class_id AND {$joinTableAlias}.store_id = {$this->getStoreId()}";
		$priceColumns = $this->getColumns(self::NCPRICE);
		$joinIfColumnsEmpty = !empty($priceColumns);

	    $this->joinMainTable($joinTableAlias,$joinTable,$joinIfColumnsEmpty,$condition);
	    return $this;
	}
	
	public function joinTaxClass()
	{
		$joinTableAlias = self::TC;
		$joinTable =  $this->getTable('tax/tax_class');
		$condition =  "{$joinTableAlias}.class_id = ".self::NCT.".tax_class_id";
	
		$this->joinMainTable($joinTableAlias,$joinTable,false,$condition);
		return $this;
	}

	public function joinPrice()
	{
		$customerGroupId = self::DEF_CUSTOMER_GROUP_NOT_LOGGED_IN;
		$configCustomerGroupId = $this->getData(self::PRICE_CUSTOMER_GROUP_ID);
		if(!empty($configCustomerGroupId))
			$customerGroupId = $configCustomerGroupId;		
		
	    $joinTableAlias = self::NCPRICE;
		$joinTable = $this->getTable('nscexport/cache_price');
		$condition =   "{$joinTableAlias}.product_id = ".self::MAIN_TABLE_SUBST.".entity_id AND {$joinTableAlias}.store_id = {$this->getStoreId()} AND {$joinTableAlias}.customer_group_id = {$customerGroupId}";

	    $this->joinMainTable($joinTableAlias,$joinTable,false,$condition);
	    return $this;
	}

	public function joinCustomTables()
	{
		$this->dispatchJoinTableEvent($this->getMainTable(true),"_product");
	}

	protected function definePriceColumns()
	{
		$originalPricesIncludesTax = Mage::helper('tax')->priceIncludesTax($this->getStore());		
		$taxRateColumnName = self::NCT.'.tax_percent';
		$weeeColumnOriginal = "";
		$weeeColumnFinal = "";

		if($this->isWeeeEnabled())
		{
			$weeeColumnOriginal = self::NCW.".".self::WEEE_COLUMN_TOTAL_ORIGINAL;
			$weeeColumnFinal = self::NCW.".".self::WEEE_COLUMN_TOTAL_FINAL;
		}

		$this->_columns[self::NCPRICE] = array("price_final_exclude_tax" => $this->getPriceColumnFormat(self::NCPRICE."."."min_price",false,$weeeColumnFinal),
		 									"price_final_include_tax" => $this->getPriceColumnFormat(self::NCPRICE."."."min_price",true,$weeeColumnFinal),
		 									"price_original_exclude_tax" => $this->getPriceColumnFormat(self::NCPRICE."."."price",false,$weeeColumnOriginal,true),
		 									"price_original_include_tax" => $this->getPriceColumnFormat(self::NCPRICE."."."price",true,$weeeColumnOriginal,true),
											"price_discount_percent" => "ROUND(((".self::NCPRICE.".price - ".self::NCPRICE.".min_price)*100)/".self::NCPRICE.".price,0)",
											"price_discount_exclude_tax" => $this->getPriceColumnFormat("(".self::NCPRICE."."."price - ".self::NCPRICE."."."min_price)",false),
											"price_discount_include_tax" => $this->getPriceColumnFormat("(".self::NCPRICE."."."price - ".self::NCPRICE."."."min_price)",true),
		 									);
	}

	protected function getPriceColumnFormat($comunName,$includeTax,$weeeColumn = "",$weeeOriginal = false)
	{
		$originalPricesIncludesTax = Mage::helper('tax')->priceIncludesTax($this->getStore());
		
		//pridat nacteni vybrane meny
		$currencyRate = $this->helper()->getStoreCurrencyRate($this->getStore(),$this->getCurrency());
		$taxRateColumnName = self::NCT.'.tax_percent';
		$weeeTaxable = $this->isWeeeTaxable();
		$weeeColumnTaxable = $weeeColumnNonTaxable = "";

		if($this->isWeeeEnabled() && !empty($weeeColumn))
		{
			$weeeColumn = "IFNULL({$weeeColumn},0)";
			switch($this->getWeeeListPriceDisplayType())
			{
				case Mage_Weee_Model_Tax::DISPLAY_INCL:
				case Mage_Weee_Model_Tax::DISPLAY_INCL_DESCR:
					if($weeeOriginal && $weeeTaxable)
					{
						$weeeColumnTaxable = "IF(".self::NCPRICE."."."price = ".self::NCPRICE."."."min_price ,{$weeeColumn},0)";
						$weeeColumnNonTaxable = "IF(".self::NCPRICE."."."price = ".self::NCPRICE."."."min_price ,0,{$weeeColumn})";
					}
					break;
            	case Mage_Weee_Model_Tax::DISPLAY_EXCL_DESCR_INCL:
            		if(!$includeTax)
            			$weeeColumn = "";
            		if($includeTax && $weeeOriginal)
            		    $weeeColumn = "IF(".self::NCPRICE."."."price = ".self::NCPRICE."."."min_price ,{$weeeColumn},0)";
            		break;
            	case Mage_Weee_Model_Tax::DISPLAY_EXCL:
            		$weeeColumn = "";
            		if($weeeOriginal)
            			$weeeTaxable = false;
            		break;
            	default:
            		break;
        	}

        	if($weeeTaxable && empty($weeeColumnTaxable))
				$weeeColumnTaxable = $weeeColumn;
			if(!$weeeTaxable && empty($weeeColumnNonTaxable))
				$weeeColumnNonTaxable = $weeeColumn;
		}

		$columnFormat = $this->helper()->getPriceColumnFormat($comunName, $taxRateColumnName,$currencyRate, $originalPricesIncludesTax,$includeTax,true,$weeeColumnTaxable,$weeeColumnNonTaxable);
		return $columnFormat;
	}
}
?>