<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */ 

/** 
* Model for Export
* 
* @category Nostress 
* @package Nostress_Nscexport
* 
*/

class Nostress_Nscexport_Model_Mysql4_Enginecategory extends Nostress_Nscexport_Model_Mysql4_Abstract
{	
	const TAXONOMY_CODE = 'taxonomy_code';
	const LOCALE = 'locale';
	const LIMIT = 100;
	const COUNT_FIELD = "COUNT(*)";
	
    public function _construct()
    {    
        // Note that the export_id refers to the key field in your database table.
        $this->_init('nscexport/enginecategory', 'entity_id');
    }    
    
    public function insertEngineCategoryRecords($engineCode,$locale,$records)
    {    	
    	$emptyRecord = array(self::LOCALE=>$locale, self::TAXONOMY_CODE => $engineCode);
    	
    	$sql = "";    	
    	$i = 0;
    	
    	foreach($records as $row)
    	{
    		if(isset($row['path']))
    		{
    			$row['hash'] = $this->getTaxonomyPathHash($row['path']);
    		}
    		else
    			throw new Exception("Category with id %s is missing path.",isset($row['id'])?$row['id']:"N/A");
    		
    		$i++;
    		$sql .= $this->insertRecordQuery(array_merge($emptyRecord,$row));
    		if($i > self::LIMIT)
    		{
    			$this->_runQuery($sql);
    			$sql = "";
    			$i = 0;
    		}
    	}    
    	if(!empty($sql))
    	    $this->_runQuery($sql);   	    	    	
    } 

    protected function getTaxonomyPathHash($path)
    {
    	return $this->helper()->getHash($path);
    }
    
	protected function insertRecordQuery($row)
    {
    	$columns = implode(",",array_keys($row));    	
    	$values = $this->addEscapeChars(array_values($row));
    	$values = implode("','",$values);
    	
    	$sql = "INSERT INTO {$this->getMainTable()} ({$columns}) VALUES ('{$values}');";
    	
    	return $sql;  	
    }
    
    protected function addEscapeChars($items)
    {
    	foreach($items as $key => $item)
    	{
    		$items[$key] = str_replace("'","\\'",$item);
    	}	
    	return $items;
    }
    
    public function cleanTable()
    {    	
    	$this->_runQuery($this->deleteRecordQuery());
    }
    
    protected function deleteRecordQuery($where='')
    {
    	$sql = "DELETE FROM ".$this->getMainTable();
    	if($where != '')
    		$sql .= ' '.$where;
		$sql .= ";";
    	return $sql;  	
    }
    
    public function countColumns($engineCode,$locale)
    {
        $result = $this->_runOneRowSelectQuery($this->countColumnsQuery($engineCode,$locale));
        if(array_key_exists(self::COUNT_FIELD,$result))
            return $result[self::COUNT_FIELD];
        else 
            return 0;
    }
    
    public function updateTaxonomyCategoryIdentificators($taxonomyCode,$locale,$storeId,$attributeId,$valueColumn)
    {
    	$catalogCategoryEntityTextTable = Mage::getSingleton('core/resource')->getTableName('catalog_category_entity_text');
    	$updateQuery = "UPDATE {$catalogCategoryEntityTextTable} AS dest,
	    	(
	    	SELECT *
	    	FROM {$this->getMainTable()}
				WHERE taxonomy_code = '{$taxonomyCode}'
	    				AND locale = '{$locale}'
	    	) AS src
	    	SET dest.value = src.hash WHERE store_id = {$storeId} AND attribute_id = '{$attributeId}' AND dest.value = src.{$valueColumn};
	    	";
    	
    	$this->helper()->log($updateQuery);    
    	$this->_runQuery($updateQuery);    	
    }
    
    protected function countColumnsQuery($engineCode,$locale)
    {
        $sql = "SELECT ".self::COUNT_FIELD." FROM {$this->getMainTable()} WHERE taxonomy_code = '{$engineCode}' AND locale = '{$locale}';";
        return $sql;
    }
            
    protected function helper()
    {
    	if(!isset($this->_helper))
    	{
    		$this->_helper = Mage::helper('nscexport');
    	}
    	return $this->_helper;
    }
    

}