<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

/**
* Model for Export
*
* @category Nostress
* @package Nostress_Nscexport
*
*/

class Nostress_Nscexport_Model_Mysql4_Submission_Items extends Mage_Core_Model_Mysql4_Abstract
{
    public function _construct()
    {
        // Note that the export_id refers to the key field in your database table.
        $this->_init('nscexport/submission_items', 'entity_id');
    }
    
    public function insertItems( $profileID, $items, $itemIdName = 'externalId') {
        
        $data = array();
        foreach( $items as $item) {
            $data[] = $this->_formatItem($profileID, $item, $itemIdName);
        }
        
        return $this->_getWriteAdapter()->insertMultiple( $this->getTable( $this->_mainTable), $data);
    }
    
    public function updateItems( $profileID, $items, $itemIdName = 'externalId') {
            
        $db = $this->_getWriteAdapter();
        $table = $this->getTable( $this->_mainTable);
        $count = 0;
        foreach( $items as $item) {
            $data = $this->_formatItem($profileID, $item, $itemIdName);
            $count += $db->update( $table, $data,
                $db->quoteInto( 'profile_id = ?', $profileID)." AND ".$db->quoteInto( 'item_id = ?', $item[ $itemIdName])
            );
        }
        return $count;
    }
    
    protected function _formatItem( $profileID, $item, $itemIdName) {
        
        $data = $item;
        unset( $data['status'], $data['message']);
        
        return array(
            'profile_id' => $profileID,
            'item_id' => $item[ $itemIdName],
            'data' => json_encode( $data),
            'updated_at' => new Zend_Db_Expr( 'now()'),
            'status' => isset($item['status']) ? $item['status'] : null,
            'message' => isset($item['message']) ? $item['message'] : null,
        );
    }
}