<?php
/**
 * Magento Module developed by NoStress Commerce
 *
 * NOTICE OF LICENSE
 *
 * This program is licensed under the Koongo software licence (by NoStress Commerce). 
 * With the purchase, download of the software or the installation of the software 
 * in your application you accept the licence agreement. The allowed usage is outlined in the
 * Koongo software licence which can be found under https://docs.koongo.com/display/koongo/License+Conditions
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at https://store.koongo.com/.
 *
 * See the Koongo software licence agreement for more details.
 * @copyright Copyright (c) 2017 NoStress Commerce (http://www.nostresscommerce.cz, http://www.koongo.com/)
 *
 */

require_once 'abstract.php';

/**
 * Magento Compiler Shell Script
 *
 * @category    Mage
 * @package     Mage_Shell
 * @author      Magento Core Team <core@magentocommerce.com>
 */
class Mage_Shell_Generate_Profiles extends Mage_Shell_Abstract
{
  
    /**
     * Run script
     *
     */
    public function run()
    {
        if ( $this->getArg('help')) {
            echo $this->usageHelp();
        } elseif( $this->getArg('profiles')) {
            
            $helper = Mage::helper('nscexport/data_profile');
            
            $profiles = explode(",",$this->getArg('profiles'));
            
            $profileNames = array();
            $profileIds = array();
            foreach ($profiles as $profile)
            {
                if(is_numeric($profile))
                    $profileIds[] = $profile;
                else
                    $profileNames[] = $profile;
            }
            // run profile using profile IDs or names
            $helper->runProfilesByNames($profileNames);
            $helper->runProfilesByIds($profileIds);
            
            $count = count( $profiles);
            
            echo "$count profiles finished successfully.";
            
        } else {
            Mage::helper('nscexport/data_profile')->runAllProfiles();
            echo "All profiles finished successfully.";
        }
    }

    /**
     * Retrieve Usage Help Message
     *
     */
    public function usageHelp()
    {
        return <<<USAGE
Usage:  php -f generate_feeds.php -- [options]

  --profiles <profiles> Profiles ids or names separated by comma
  help          This help
  no argument   Generate all profiles

USAGE;
    }
}

$shell = new Mage_Shell_Generate_Profiles();
$shell->run();
